﻿<#
.Synopsis.DESCRIPTION
   This script get HPC related properties from service fabric cluster property store
.EXAMPLE
   Get-HpcReliableProperty.ps1 -Property "ClusterName"
#>
Param
(
    [Parameter(Mandatory=$false)]
    [String] $ConnectionEndpoint = "",

    [Parameter(Mandatory=$false)]
    [String] $PropertyName = ""
)

$source = @"
    public class HpcVariable
    {
        public string Name { get; set; }
        public string Value { get; set; }

        public HpcVariable(string name, string value)
        {
            this.Name = name;
            this.Value = value;
        }
    }
"@

$ccpHome = Get-ChildItem env:CCP_HOME
Add-Type -Path "$($ccpHome.Value)bin\HpcCommon.dll"
Add-Type -TypeDefinition $source

function GetServiceFabricProperties($property)
{
    $var = @()   
    if([String]::IsNullOrEmpty($property))
    {
        foreach($parent in $ParentNames)
        {
            [Uri]$parentName = new-object system.uri("fabric:/$($parent -replace '\\','/')".ToLower())
            [Boolean]$includeValues = $true
            [Object[]]$params = @($parentName,$includeValues,$null)
            $nameProps = $EnumeratePropertiesAsync.Invoke($client, $params).Result
            foreach($nameProp in $nameProps)
            {
                if ($ReliableProperties.ContainsKey($nameProp.Metadata.PropertyName))
                {
                    $prop = $ReliableProperties[$nameProp.Metadata.PropertyName]
                    $var += GetHpcVariable $prop $nameProp
                }
            }
        }
    }
    elseif($ReliableProperties.ContainsKey($Property))
    {
        $prop = $ReliableProperties[$Property]
        [Uri]$parentName = new-object system.uri("fabric:/$($prop.ParentName -replace '\\','/')".ToLower())
        [String]$propertyName = $Property
        [Object[]]$params = @($parentName,$propertyName)
        $nameProp = $GetPropertyAsync.Invoke($client, $params).Result
        $var += GetHpcVariable $prop $nameProp
    }
    else
    {
        throw "Property name $PropertyName is not existed for current cluster"
    }

    return $var
}

function GetHpcVariable($prop,$nameProp)
{
    $type = $prop.ValueType
    if($type -eq [Int32])
    {
        $type = [Long]
    }
    elseif($type -eq [String[]])
    {
        $type = [String]
    }

    if($nameProp -eq $null)
    {
        return $null
    }
    $GetValue = $nameProp.GetType().GetMethod("GetValue").MakeGenericMethod($type)
    $value = $GetValue.Invoke($nameProp, $null)
    return New-Object HpcVariable -argumentList $prop.Name,$value.ToString()
}

$script:ParentNames = [Microsoft.Hpc.HpcConstants]::ParentNames
$script:ReliableProperties = [Microsoft.Hpc.HpcConstants]::ReliableProperties
if([String]::IsNullOrWhiteSpace($ConnectionEndpoint))
{
    $cluster = Connect-ServiceFabricCluster
}
else
{
    $cluster = Connect-ServiceFabricCluster -ConnectionEndpoint $ConnectionEndpoint
}

$script:client = $cluster.FabricClient.PropertyManager
$script:GetPropertyAsync = $client.GetType().GetMethod("GetPropertyAsync", [Type[]]@([Uri],[String]))
$script:EnumeratePropertiesAsync = $client.GetType().GetMethod("EnumeratePropertiesAsync", [Type[]]@([Uri],[Boolean],[System.Fabric.PropertyEnumerationResult]))
GetServiceFabricProperties $PropertyName