﻿<#
.Synopsis
   View or update Azure endpoints for current HPC head node
.DESCRIPTION
   This script view or update Azure endpoints for current HPC head node, current support GA and China MoonCake
#>
[CmdletBinding(DefaultParametersetName="Update")]
Param
(
    [Parameter(Mandatory=$true, ParameterSetName='View')]
    [Switch] $View, 
    		
    [Parameter(Mandatory=$true, ParameterSetName='LegacyUpdate')]
    [ValidateSet('Global','China')]
    [String] $AzureType,

    [Parameter(Mandatory=$true, ParameterSetName='EnvironmentUpdate')]
    [ValidateSet('AzureCloud', 'AzureChinaCloud')]
    [String] $AzureEnvironmentName
)


if (-not ([Security.Principal.WindowsPrincipal] [Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole([Security.Principal.WindowsBuiltInRole]::Administrator))
{
    Write-Error "You must run this script as administrator."
    return
}

$ValidEnvironmentNames = @('AzureCloud', 'AzureChinaCloud')

Add-PSSnapin Microsoft.Hpc

if ($View.IsPresent)
{
    $item = Get-HpcClusterRegistry -PropertyName AzureEnvironment
    if(($null -eq $item) -or (-not $item.Value))
    {
        Write-Host "The Azure Environment for the cluster is not set, use 'AzureCloud' by default."
    }
    elseif($item.Value -in $ValidEnvironmentNames)
    {
        Write-Host "The Azure Environment for the cluster is '$($item.Value)'."
    }
    else
    {
        Write-Error "The Azure Environment for the cluster is unknown: '$($item.Value)'."
    }
}
else
{
    if($PSCmdlet.ParameterSetName -eq 'LegacyUpdate')
    {
        if($AzureType -eq 'Global')
        {
            $AzureEnvironmentName = 'AzureCloud'
        }
        else
        {
            $AzureEnvironmentName = 'AzureChinaCloud'
        }

        Write-Warning "'-AzureType' will be deprecated, please use '-AzureEnvironmentName' instead."
    }

    Set-HpcClusterRegistry -PropertyName AzureEnvironment -PropertyValue $AzureEnvironmentName
    Write-Warning "Existing Azure node templates and previously added Azure nodes not belonging to $AzureEnvironmentName can no longer be used."
    Write-Host "Update the Azure service endpoints successfully!"
}
