@echo off

set _DismError=
set _SAVEERR=

echo **** Upgrading Windows PE image
if not exist "%CCP_DATA%\Boot\x64\bootr2.wim" (
   echo **** New HPC Windows PE image not found
   exit /b 1
)

if not exist "%CCP_DATA%\Boot\boot.sdi" goto :ReplaceBootWim

:CreateMountDir
set _RANDROOTDIR=%TEMP%\HPC.WIM.TEMP%RANDOM%
if exist "%_RANDROOTDIR%" goto :CreateMountDir
echo **** Creating temp folder: %_RANDROOTDIR%
mkdir "%_RANDROOTDIR%"
icacls.exe "%_RANDROOTDIR%" /grant:r Administrators:F SYSTEM:F /inheritance:r

set _MOUNTDIRORG=%_RANDROOTDIR%\original
set _MOUNTDIRNEW=%_RANDROOTDIR%\new
set _TMPSTOREDIR=%_RANDROOTDIR%\store
set _TEMPBOOTWIMR2=%_RANDROOTDIR%\bootr2.wim
echo **** Creating MountDir for original Windows PE image: %_MOUNTDIRORG%
mkdir "%_MOUNTDIRORG%"
echo **** Creating MountDir for new Windows PE image: %_MOUNTDIRNEW%
mkdir "%_MOUNTDIRNEW%"
echo **** Creating StoreDir: %_TMPSTOREDIR%
mkdir "%_TMPSTOREDIR%"
echo **** Copying bootr2.wim to %_TEMPBOOTWIMR2%
copy /y "%CCP_DATA%\Boot\x64\bootr2.wim" "%_TEMPBOOTWIMR2%"

echo **** Mounting Original HPC Windows PE image
DISM.exe /Mount-Wim /WimFile:"%CCP_DATA%\Boot\x64\boot.wim" /index:1 /MountDir:"%_MOUNTDIRORG%" /ReadOnly

echo **** Copy files from original image to %_TMPSTOREDIR%"
echo **** Copying startnet.cmd file
copy /y "%_MOUNTDIRORG%\Windows\System32\startnet.cmd" "%_TMPSTOREDIR%\startnet.cmd"

echo **** Copying hosts file
copy /y "%_MOUNTDIRORG%\Windows\System32\Drivers\etc\hosts" "%_TMPSTOREDIR%\hosts"

if exist "%_MOUNTDIRORG%\CCPDrivers" (
   echo **** Copying CCPDrivers to %_TMPSTOREDIR%
   robocopy "%_MOUNTDIRORG%\CCPDrivers" "%_TMPSTOREDIR%\CCPDrivers" /E /NP /NFL /NDL /NJH
)

echo **** Unmounting Original HPC Windows PE image
DISM.exe /Unmount-Wim /MountDir:"%_MOUNTDIRORG%" /discard

echo **** Mounting New HPC Windows PE image
DISM.exe /Mount-Wim /WimFile:"%_TEMPBOOTWIMR2%" /index:1 /MountDir:"%_MOUNTDIRNEW%"

echo **** Copy files from %_TMPSTOREDIR% to MountDir of New HPC Windows PE image
echo **** Copying startnet.cmd file
copy /y "%_TMPSTOREDIR%\startnet.cmd" "%_MOUNTDIRNEW%\Windows\System32\startnet.cmd"

echo **** Copying ExecutionClient.exe file
copy /y "%CCP_HOME%\Bin\executionclient.exe" "%_MOUNTDIRNEW%\Windows\System32\executionclient.exe"

echo **** Copying hosts file
copy /y "%_TMPSTOREDIR%\hosts" "%_MOUNTDIRNEW%\Windows\System32\Drivers\etc\hosts"

if exist "%_TMPSTOREDIR%\CCPDrivers" (
   echo **** Copying CCPDrivers
   robocopy "%_TMPSTOREDIR%\CCPDrivers" "%_MOUNTDIRNEW%\CCPDrivers" /E /NP /NFL /NDL /NJH
	  
   if not defined DismDir (
      echo **** [Error]
      echo **** [Error] Environment variable "DismDir" not set, the drivers will not be injected to NEW HPC Windows PE image****
	  echo **** [Error]
	  set _DismError=1
   ) else (
	  echo **** Injecting Drivers to New HPC Windows PE image
	  for /d %%d in ("%_TMPSTOREDIR%\CCPDrivers\*") do (
	     echo **** Injecting Driver %%d
	     "%DismDir%\dism.exe" /Add-Driver /Image:"%_MOUNTDIRNEW%" /Driver:%%d
      )
   )
)

echo **** Copy files from New HPC Windows PE image to %_TMPSTOREDIR%
echo **** Copying PXE files from New HPC Windows PE image to %_TMPSTOREDIR%
robocopy "%_MOUNTDIRNEW%\Windows\Boot\PXE" "%_TMPSTOREDIR%\Boot\x64" /E /NP /NFL /NDL /NJH

echo **** Copying Fonts files from New HPC Windows PE image to %_TMPSTOREDIR%
robocopy "%_MOUNTDIRNEW%\Windows\Boot\Fonts" "%_TMPSTOREDIR%\Boot\Fonts" /E /NP /NFL /NDL /NJH

echo **** Committing changes to New HPC Windows PE image
DISM.exe /Unmount-Wim /MountDir:"%_MOUNTDIRNEW%" /commit
if not "%errorlevel%" == "0" (
   echo **** Failed to save New HPC Windows PE image
   echo **** Deleting temp folder: %_RANDROOTDIR%
   rmdir /s /q "%_RANDROOTDIR%"
   exit /b 1
)

echo **** Copy files from %_TMPSTOREDIR% to %CCP_DATA%\Boot
robocopy "%_TMPSTOREDIR%\Boot" "%CCP_DATA%\Boot" /E /NP /NFL /NDL /NJH

echo **** Copying "%_TEMPBOOTWIMR2%" to bootr2.wim
move /y "%_TEMPBOOTWIMR2%" "%CCP_DATA%\Boot\x64\bootr2.wim"

echo **** Deleting temp folder: %_RANDROOTDIR%
rmdir /s /q "%_RANDROOTDIR%"

:ReplaceBootWim
echo **** Backuping Original HPC Windows PE image
move /y "%CCP_DATA%\Boot\x64\boot.wim" "%CCP_DATA%\Boot\x64\boot.wim.bak"
   
echo **** Copying New HPC Windows PE image
move /y "%CCP_DATA%\Boot\x64\bootr2.wim" "%CCP_DATA%\Boot\x64\boot.wim"

if defined _DismError (
   echo **** Upgrading done with error:
   echo **** The drivers are not injected to new HPC Windows PE image due to old Dism tool version.
   echo ***  Please download from http://www.microsoft.com/en-us/download/details.aspx?id=39982
   echo ***  For more information, see http://go.microsoft.com/fwlink/?LinkID=386546
) else (
   echo **** Upgrading done
)
exit /b 0
