SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO

--Turn off AUTO_CLOSE
DECLARE @sqlScript NVARCHAR(2000);
SET @sqlScript = ('ALTER DATABASE ' + QUOTENAME(DB_NAME()) + N' SET AUTO_CLOSE OFF');
EXEC sp_executesql @sqlScript;
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[DiagnosticTests]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
	AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[DiagnosticTests]') AND name = N'TargetNodeTypes')
BEGIN

    ALTER TABLE DiagnosticTests
    ADD
	    TargetNodeTypes			int				NOT NULL DEFAULT(1)
		/* Default target node types are every other types beside azurenode */
        
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'AddTestV2'))
DROP PROCEDURE AddTestV2
GO

CREATE PROCEDURE AddTestV2(
    @displayName        nvarchar(255),
    @shortName          nvarchar(64),
    @displayVersion     nvarchar(64),
    @companyName        nvarchar(128),
    @suite              nvarchar(128),
    @description        nvarchar(255),
    @type               int,
    @targetNodeTypes	int,
    @outputFormat       nvarchar(16),
    @installedBy        nvarchar(64),
    @newId              int out,
    @timeStamp			timestamp out
    )
AS
BEGIN
    INSERT INTO DiagnosticTests
    (DisplayName, ShortName, DisplayVersion, CompanyName, Suite, Description, Type, TargetNodeTypes, OutputFormat, InstalledBy)
    VALUES
    (@displayName, @shortName, @displayVersion, @companyName, @suite, @description, @type, @targetNodeTypes, @outputFormat, @installedBy)
    
    SET @newId = @@IDENTITY
    
    SET @timeStamp = (SELECT TimeStamp FROM DiagnosticTests WHERE Id = @newId)
END
GO


IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'ListTests'))
DROP PROCEDURE ListTests
GO

CREATE PROCEDURE ListTests
AS
BEGIN 
    SELECT Id, InstallTime, DisplayName, ShortName, DisplayVersion, CompanyName, Suite, Description, Type, OutputFormat, InstalledBy, ArchiveTime, TimeStamp, TargetNodeTypes
    FROM DiagnosticTests
END 
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'RequestCancelRun'))
DROP PROCEDURE RequestCancelRun
GO

CREATE PROCEDURE RequestCancelRun(
    @runId          int,
    @message        nvarchar(255),
    @succeeded      int out
)
AS
BEGIN
    UPDATE TestRuns
    SET RequestCancel=1, Message=@message, UpdateTime=GETUTCDATE()
    -- State >7 (PoststepFinished) included all finalize state, in these kind of states, test can't be cancelled
    WHERE Id=@runId AND State <= 7
    
    SET @succeeded = @@ROWCOUNT
END
GO

EXEC sp_updateextendedproperty @name = N'dbVersion', @value = '3.1';
GO

SET ANSI_PADDING OFF
