
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO

/*
** Drop all existing tables
*/

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[ClusterConfig]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP table [dbo].[ClusterConfig]
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[TestResults]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP table [dbo].[TestResults]
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[NodeMappings]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP table [dbo].[NodeMappings]
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[NodeNames]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP table [dbo].[NodeNames]
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[TestRunParameters]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP table [dbo].[TestRunParameters]
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[TestRunVariables]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP table [dbo].[TestRunVariables]
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[TestRuns]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP table [dbo].[TestRuns]
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[TaskParameters]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP table [dbo].[TaskParameters]
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[TaskEnvironmentVariables]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP table [dbo].[TaskEnvironmentVariables]
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[DiagnosticTasks]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP table [dbo].[DiagnosticTasks]
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[TestEnvironmentVariables]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP table [dbo].[TestEnvironmentVariables]
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[TestParameters]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP table [dbo].[TestParameters]
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[DiagnosticTests]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP table [dbo].[DiagnosticTests]
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Users]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP table [dbo].[Users]
GO

/*****************************************************************************
** ClusterConfig
** 
** This table contains a list of configuration for the diagnostic
*****************************************************************************/

CREATE TABLE ClusterConfig
(
    Name					nvarchar(128)   NOT NULL PRIMARY KEY CLUSTERED,
    Value					nvarchar(2048)  NOT NULL DEFAULT(''),
) ON [PRIMARY]
GO

INSERT INTO ClusterConfig (Name,Value)	VALUES ('TtlCompletedRuns', '5')
INSERT INTO ClusterConfig (Name,Value)	VALUES ('RunCleanUpHour', '3')
INSERT INTO ClusterConfig (Name,Value)	VALUES ('ConcurrencyTestRunNumber', '5')
GO

/*****************************************************************************
** Users
** 
** This table contains a list of all the users who have installed or have
** requested a diagnostic test run.
** It also include the job submitting credential
*****************************************************************************/

CREATE TABLE Users
(
    Id                      int             IDENTITY(1,1) NOT NULL PRIMARY KEY CLUSTERED,
    Name                    nvarchar(64)    NOT NULL DEFAULT(''),
    RunName					nvarchar(64)	NOT NULL DEFAULT(''),
    RunPassword				varbinary(800)	,
) ON [PRIMARY]    

CREATE NONCLUSTERED INDEX Users_Name on Users(Name)
GO


/*****************************************************************************
** DiagnosticTests Table
**
** This table is the master table for definitions of the diagnostic tests
** installed within the system.
**
** Note that this table has two sets of columns.  The first is for properties
** that are maintained by the system (local ID, install time, etc..). The 
** second set is about the definition of the test from the author.  Since
** there is a one to one mapping here (a test can only be installed once) both
** are kept in this one table rather than seperate tables.
*****************************************************************************/

CREATE TABLE DiagnosticTests
(
    /* Local system test properties */
    Id                      int             IDENTITY(1,1) NOT NULL PRIMARY KEY CLUSTERED,
    InstallTime             datetime        NOT NULL DEFAULT(GETUTCDATE()),   

    /* Using ISO 8601 format for datetime. It's culture irrelevant. Original code was in Numeric, which has different meaning on non-us_english environment. See MSDN:
    http://msdn.microsoft.com/en-us/library/ms187819(v=sql.105).aspx */
    ArchiveTime		        datetime        NOT NULL DEFAULT('20991231'),
    LastRunTime             datetime        NOT NULL DEFAULT(0),
    InstalledBy             nvarchar(64)    NOT NULL DEFAULT(''),

    /* Test definition properties */
    DisplayName             nvarchar(255)   NOT NULL DEFAULT(''),
    ShortName               nvarchar(64)    NOT NULL DEFAULT(''),
    DisplayVersion          nvarchar(64)    NOT NULL DEFAULT(''),
    CompanyName             nvarchar(128)   NOT NULL DEFAULT(''),
    Suite                   nvarchar(128)   NOT NULL DEFAULT(''),
    Description             nvarchar(1024)  NOT NULL DEFAULT(''),
    Type                    int             NOT NULL DEFAULT(0),
    TargetNodeTypes			int				NOT NULL DEFAULT(1),
    /* Default target node types are every other types beside azurenode */
    OutputFormat            nvarchar(16)    NOT NULL DEFAULT(''),
    TimeStamp               timestamp   
) ON [PRIMARY]
GO


/*****************************************************************************
** TestParameters Table
**
** This table is used to store test parameter definitions.  These definitions
** are used by the system to present the user with options for running the
** test on the system.
*****************************************************************************/

CREATE TABLE TestParameters
(
    Id                      int             IDENTITY(1,1) NOT NULL PRIMARY KEY CLUSTERED,
    TestId                  int             NOT NULL FOREIGN KEY REFERENCES DiagnosticTests(Id), 
	Name                    nvarchar(64)    NOT NULL DEFAULT(''),
	SwitchName              nvarchar(64)    NOT NULL DEFAULT(''),
	Description             nvarchar(255)   NOT NULL DEFAULT(''),
    Visible                 bit             NOT NULL DEFAULT(1),
	Type                    int             NOT NULL DEFAULT(0),
	Scope                   int             NOT NULL DEFAULT(7),
	DefaultIntegerValue     int             NOT NULL DEFAULT(0),
	MinimumValue            int             NOT NULL DEFAULT(0),
	MaximumValue            int             NOT NULL DEFAULT(0),
	DefaultBooleanValue     bit             NOT NULL DEFAULT(0),
	DefaultStringValue      nvarchar(255)   NOT NULL DEFAULT(''),
	AllowedStrings          nvarchar(2000)  NOT NULL DEFAULT(''),
	Format                  nvarchar(64)    NOT NULL DEFAULT('-{0}:{1}')
)
GO

CREATE NONCLUSTERED INDEX TestParameters_TestId ON TestParameters(TestId)
GO



/*****************************************************************************
** TestEnvironmentVariables table
**
** This table is used to define default environment variables that must be
** set for when the test is run on the cluster for the test tasks.
*****************************************************************************/


CREATE TABLE TestEnvironmentVariables
(
    Id                      int             IDENTITY(1,1) NOT NULL PRIMARY KEY CLUSTERED,
    TestId                  int             NOT NULL FOREIGN KEY REFERENCES DiagnosticTests(Id), 
    Name                    nvarchar(255)   NOT NULL DEFAULT(''),
    Value                   nvarchar(255)   NOT NULL DEFAULT(''),
) ON [PRIMARY]
GO

CREATE NONCLUSTERED INDEX TestEnvironmentVariables_TestId ON TestEnvironmentVariables(TestId)
GO



/*****************************************************************************
** DiagnosticTasks table
**
** This table defines the tasks that are to be run for a test on the cluster.
** Each test can have up to three tasks definitions (Pre, Run, Post).
*****************************************************************************/


CREATE TABLE DiagnosticTasks
(
    Id                      int             IDENTITY(1,1) NOT NULL PRIMARY KEY CLUSTERED,
    TestId                  int             NOT NULL FOREIGN KEY REFERENCES DiagnosticTests(Id), 
    Type                    int             NOT NULL DEFAULT(0),
    Command                 nvarchar(255)   NOT NULL DEFAULT(''),
    Path                    nvarchar(255)   NOT NULL DEFAULT(''),
    ResultXml               bit             NOT NULL DEFAULT(0),
    UseUserCred				bit				NOT NULL DEFAULT(0)
) ON [PRIMARY]
GO

CREATE NONCLUSTERED INDEX DiagnosticTasks_TestId ON DiagnosticTasks(TestId)
GO



/*****************************************************************************
** TaskParameters Table
**
** This table is used to set default task parameters that must be set for a
** task when it runs as part of the test.  It is used for example when the 
** same command is used for the Pre, Run, and Post tasks, but the task is 
** put into a different mode for each step.
*****************************************************************************/


CREATE TABLE TaskParameters
(
    Id                      int             IDENTITY(1,1) NOT NULL PRIMARY KEY CLUSTERED,
    TaskId                  int             NOT NULL FOREIGN KEY REFERENCES DiagnosticTasks(Id), 
    Name                    nvarchar(64)    NOT NULL DEFAULT(''),
    Value                   nvarchar(255)   NOT NULL DEFAULT(''),
    Format                  nvarchar(64)    NOT NULL DEFAULT('-{0}:{1}')
) ON [PRIMARY]
GO

CREATE NONCLUSTERED INDEX TaskParameters_TaskId ON TaskParameters(TaskId)
GO


/*****************************************************************************
** TaskEnvironmentVariables Table
** 
** This table is used to set task environment variables that must be set for
** a task when it is run for the test. Potentially used for the same reasons
** as the TaskParameters table.
*****************************************************************************/


CREATE TABLE TaskEnvironmentVariables
(
    Id                      int             IDENTITY(1,1) NOT NULL PRIMARY KEY CLUSTERED,
    TaskId                  int             NOT NULL FOREIGN KEY REFERENCES DiagnosticTasks(Id), 
    Name                    nvarchar(255)   NOT NULL DEFAULT(''),
    Value                   nvarchar(255)   NOT NULL DEFAULT(''),
) ON [PRIMARY]
GO

CREATE NONCLUSTERED INDEX TaskEnvironmentVariables_TaskId ON TaskEnvironmentVariables(TaskId)
GO



/*****************************************************************************
** TestRun Table
** 
** This table contains a list of all the test runs for the system
*****************************************************************************/

CREATE TABLE TestRuns
(
    Id                      int             IDENTITY(1,1) NOT NULL PRIMARY KEY CLUSTERED,
    TestId                  int             NOT NULL FOREIGN KEY REFERENCES DiagnosticTests(Id), 
    State                   int             NOT NULL DEFAULT(0),
    PreviousState           int             NOT NULL DEFAULT(0),
    CreateTime              datetime        NOT NULL DEFAULT(GETUTCDATE()),
    StartTime               datetime        NOT NULL DEFAULT(0),
    EndTime                 datetime        NOT NULL DEFAULT(0),
    PreJobId                int             NOT NULL DEFAULT(0),
    RunJobId                int             NOT NULL DEFAULT(0),
    PostJobId               int             NOT NULL DEFAULT(0),
    RequestCancel           bit             NOT NULL DEFAULT(0),
    PendingArchive          bit             NOT NULL DEFAULT(0),
    Message                 nvarchar(255)   NOT NULL DEFAULT(''),
    PublicState             int             NOT NULL DEFAULT(0),
    RequestedBy             int             NOT NULL FOREIGN KEY REFERENCES Users(Id),
    UpdateTime              datetime        NOT NULL DEFAULT(GETUTCDATE()),
    Reviewed				int				NOT NULL DEFAULT(0),
    RunName					nvarchar(64),
    RunPassword				varbinary(800),
    TimeStamp               timestamp       NOT NULL
) ON [PRIMARY]
GO

CREATE NONCLUSTERED INDEX IDX_TestRuns_State ON TestRuns (State) 
GO

CREATE NONCLUSTERED INDEX IDX_TestRuns_PendingArchive ON TestRuns (PendingArchive) 
GO

CREATE NONCLUSTERED INDEX IDX_TestRuns_Reviewed ON TestRuns (Reviewed) 
GO

CREATE NONCLUSTERED INDEX IDX_TestRuns_TimeStamp ON TestRuns (TimeStamp) 
GO

CREATE NONCLUSTERED INDEX IDX_TestRuns_PublicState ON TestRuns (PublicState) 
GO

/*****************************************************************************
** TestRunParameters Table
** 
** 
*****************************************************************************/

CREATE TABLE TestRunParameters
(
    Id                      int             IDENTITY(1,1) NOT NULL PRIMARY KEY CLUSTERED,
    RunId                   int             NOT NULL FOREIGN KEY REFERENCES TestRuns(Id), 
    Name                    nvarchar(64)    NOT NULL DEFAULT(''),
    Value                   nvarchar(255)   NOT NULL DEFAULT(''),
    Format                  nvarchar(64)    NOT NULL DEFAULT('-{0}:{1}'),
) ON [PRIMARY]
GO

CREATE NONCLUSTERED INDEX TestRunParameters_RunId ON TestRunParameters(RunId)
GO


/*****************************************************************************
** TestRunVariables Table
** 
** 
*****************************************************************************/

CREATE TABLE TestRunVariables
(
    Id                      int             IDENTITY(1,1) NOT NULL PRIMARY KEY CLUSTERED,
    RunId                   int             NOT NULL FOREIGN KEY REFERENCES TestRuns(Id),
    Name                    nvarchar(255)   NOT NULL DEFAULT(''),
    Value                   nvarchar(255)   NOT NULL DEFAULT('')
) ON [PRIMARY]
    
CREATE NONCLUSTERED INDEX TestRunVariables_RunId ON TestRunVariables(RunId)
GO


/*****************************************************************************
** NodeNames Table
** 
** 
*****************************************************************************/

CREATE TABLE NodeNames
(
    Id                      int             IDENTITY(1,1) NOT NULL PRIMARY KEY CLUSTERED,
    Name                    nvarchar(64)    NOT NULL DEFAULT(''),
) ON [PRIMARY]
GO

CREATE NONCLUSTERED INDEX NodeNames_Name ON NodeNames(Name)
GO



/*****************************************************************************
** NodeMappings
** 
** 
*****************************************************************************/

CREATE TABLE NodeMappings
(
    Id                      int             IDENTITY(1,1) NOT NULL PRIMARY KEY CLUSTERED,
    RunId                   int             NOT NULL FOREIGN KEY REFERENCES TestRuns(Id),
    NodeId                  int             NOT NULL FOREIGN KEY REFERENCES NodeNames(Id),
    Result                  int             NOT NULL DEFAULT(0),
) ON [PRIMARY]
GO

CREATE NONCLUSTERED INDEX NodeMappings_RunId ON NodeMappings(RunId)
GO


CREATE NONCLUSTERED INDEX NodeMappings_NodeId ON NodeMappings(NodeId)
GO




/*****************************************************************************
** TestResults
** 
** 
*****************************************************************************/

CREATE TABLE TestResults
(
    Id                      int             IDENTITY(1,1) NOT NULL PRIMARY KEY CLUSTERED,
    RunId                   int             NOT NULL FOREIGN KEY REFERENCES TestRuns(Id),
    Filename                nvarchar(64)    NOT NULL,
    Data                    varbinary(max)  NOT NULL,
) ON [PRIMARY]

CREATE NONCLUSTERED INDEX TestResults_RunId on TestResults(RunId)

GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO


/*****************************************************************************
** DiagnosticTests Procedures
*****************************************************************************/

/*
** AddTest()
**
** This procedure is used to add a new record to the DiagnosticsTest table.
** It will return to the caller the new id (int) of the test.  The id will
** then be used as the foreign key for the other tables that make up the
** definition of a test.
**
*/

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'GetTestUpdate'))
DROP PROCEDURE GetTestUpdate
GO

CREATE PROCEDURE GetTestUpdate(
    @timeStamp			timestamp
    )
AS
BEGIN
	SELECT Id, TimeStamp FROM DiagnosticTests
	WHERE TimeStamp > @timeStamp
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'AddTest'))
DROP PROCEDURE AddTest
GO

CREATE PROCEDURE AddTest(
    @displayName        nvarchar(255),
    @shortName          nvarchar(64),
    @displayVersion     nvarchar(64),
    @companyName        nvarchar(128),
    @suite              nvarchar(128),
    @description        nvarchar(255),
    @type               int,
    @outputFormat       nvarchar(16),
    @installedBy        nvarchar(64),
    @newId              int out,
    @timeStamp			timestamp out
    )
AS
BEGIN
    INSERT INTO DiagnosticTests
    (DisplayName, ShortName, DisplayVersion, CompanyName, Suite, Description, Type, OutputFormat, InstalledBy)
    VALUES
    (@displayName, @shortName, @displayVersion, @companyName, @suite, @description, @type, @outputFormat, @installedBy)
    
    SET @newId = @@IDENTITY
    
    SET @timeStamp = (SELECT TimeStamp FROM DiagnosticTests WHERE Id = @newId)
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'AddTestV2'))
DROP PROCEDURE AddTestV2
GO

CREATE PROCEDURE AddTestV2(
    @displayName        nvarchar(255),
    @shortName          nvarchar(64),
    @displayVersion     nvarchar(64),
    @companyName        nvarchar(128),
    @suite              nvarchar(128),
    @description        nvarchar(1024),
    @type               int,
    @targetNodeTypes	int,
    @outputFormat       nvarchar(16),
    @installedBy        nvarchar(64),
    @newId              int out,
    @timeStamp			timestamp out
    )
AS
BEGIN
    INSERT INTO DiagnosticTests
    (DisplayName, ShortName, DisplayVersion, CompanyName, Suite, Description, Type, TargetNodeTypes, OutputFormat, InstalledBy)
    VALUES
    (@displayName, @shortName, @displayVersion, @companyName, @suite, @description, @type, @targetNodeTypes, @outputFormat, @installedBy)
    
    SET @newId = @@IDENTITY
    
    SET @timeStamp = (SELECT TimeStamp FROM DiagnosticTests WHERE Id = @newId)
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'ArchiveTest'))
DROP PROCEDURE ArchiveTest
GO

CREATE PROCEDURE ArchiveTest(
	@testId             int,
	@timeStamp			timestamp out
    )
AS
BEGIN
	UPDATE DiagnosticTests SET ArchiveTime = GETUTCDATE() WHERE Id = @testId
	
	SET @timeStamp = (SELECT TimeStamp FROM DiagnosticTests WHERE Id = @testId)
END
GO

/*
** ListTests
**
** This procedure will select all the tests that are installed in the system.
**
*/


IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'ListTests'))
DROP PROCEDURE ListTests
GO

CREATE PROCEDURE ListTests
AS
BEGIN 
    SELECT Id, InstallTime, DisplayName, ShortName, DisplayVersion, CompanyName, Suite, Description, Type, OutputFormat, InstalledBy, ArchiveTime, TimeStamp, TargetNodeTypes
    FROM DiagnosticTests
END 
GO


/*****************************************************************************
** TestParameters Procedures
*****************************************************************************/

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'ListTestParameters'))
DROP PROCEDURE ListTestParameters
GO

CREATE PROCEDURE ListTestParameters
AS
BEGIN
    SELECT TestId, Name, SwitchName, Description, Visible, Type, DefaultIntegerValue, MinimumValue, MaximumValue, DefaultBooleanValue, DefaultStringValue, AllowedStrings, Format, Scope
    FROM TestParameters
END
GO


IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'AddTestParameter'))
DROP PROCEDURE AddTestParameter
GO

CREATE PROCEDURE AddTestParameter(
	@testId                 int,
	@name                   nvarchar(64),
	@switchName             nvarchar(64),
	@description            nvarchar(255),
	@visible                bit,
	@type                   int,
	@defaultIntegerValue    int,
	@minimumValue           int,
	@maximumValue           int,
	@defaultBooleanValue    bit,
	@defaultStringValue     nvarchar(64),
	@allowedStrings         nvarchar(255),
	@format                 nvarchar(64),
	@scope                  int
    )
AS
BEGIN
    INSERT INTO TestParameters
    (TestId, Name, SwitchName, Description, Visible, Type, DefaultIntegerValue, MinimumValue, MaximumValue, DefaultBooleanValue, DefaultStringValue, AllowedStrings, Format, Scope)
    VALUES
    (@testId, @name, @switchName, @description, @visible, @type, @defaultIntegerValue, @minimumValue, @maximumValue, @defaultBooleanValue, @defaultStringValue, @allowedStrings, @format, @scope)
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'AddIntegerTestParameter'))
DROP PROCEDURE AddIntegerTestParameter
GO

CREATE PROCEDURE AddIntegerTestParameter(
	@testId                 int,
	@name                   nvarchar(64),
	@switchName             nvarchar(64),
	@description            nvarchar(255),
	@visible                bit,
	@type                   int,
	@defaultIntegerValue    int = NULL,
	@minimumValue           int,
	@maximumValue           int,
	@format                 nvarchar(64),
	@scope                  int
    )
AS
BEGIN
    INSERT INTO TestParameters
    (TestId, Name, SwitchName, Description, Visible, Type, DefaultIntegerValue, MinimumValue, MaximumValue, Format, Scope)
    VALUES
    (@testId, @name, @switchName, @description, @visible, @type, @defaultIntegerValue, @minimumValue, @maximumValue, @format, @scope)
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'AddBooleanTestParameter'))
DROP PROCEDURE AddBooleanTestParameter
GO

CREATE PROCEDURE AddBooleanTestParameter(
	@testId                 int,
	@name                   nvarchar(64),
	@switchName             nvarchar(64),
	@description            nvarchar(255),
	@visible                bit,
	@type                   int,
	@defaultBooleanValue    bit = NULL,
	@format                 nvarchar(64),
	@scope                  int
    )
AS
BEGIN
    INSERT INTO TestParameters
    (TestId, Name, SwitchName, Description, Visible, Type, DefaultBooleanValue, Format, Scope)
    VALUES
    (@testId, @name, @switchName, @description, @visible, @type, @defaultBooleanValue, @format, @scope)
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'AddStringTestParameter'))
DROP PROCEDURE AddStringTestParameter
GO

CREATE PROCEDURE AddStringTestParameter(
	@testId                 int,
	@name                   nvarchar(64),
	@switchName             nvarchar(64),
	@description            nvarchar(255),
	@visible                bit,
	@type                   int,
	@defaultStringValue     nvarchar(255) = NULL,
	@allowedStrings         nvarchar(2000),
	@format                 nvarchar(64),
	@scope					int
    )
AS
BEGIN
    INSERT INTO TestParameters
    (TestId, Name, SwitchName, Description, Visible, Type, DefaultStringValue, AllowedStrings, Format, Scope)
    VALUES
    (@testId, @name, @switchName, @description, @visible, @type, @defaultStringValue, @allowedStrings, @format, @scope)
END
GO

/*****************************************************************************
** TestEnvironmentVariables Procedures
*****************************************************************************/

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'AddTestEnvironmentVariable'))
DROP PROCEDURE AddTestEnvironmentVariable
GO

CREATE PROCEDURE AddTestEnvironmentVariable(
    @testId                 int,
	@name                   nvarchar(255),
	@value                  nvarchar(255)
    )
AS
BEGIN
    INSERT INTO TestEnvironmentVariables
    (TestId, Name, Value)
    VALUES
    (@testId, @name, @value)
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'ListTestEnvironmentVariables'))
DROP PROCEDURE ListTestEnvironmentVariables
GO

CREATE PROCEDURE ListTestEnvironmentVariables
AS
BEGIN
    SELECT TestId, Name, Value
    FROM TestEnvironmentVariables
END
GO


/*****************************************************************************
** DiagnosticTasks (steps) Procedures
*****************************************************************************/

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'AddTask'))
DROP PROCEDURE AddTask
GO

CREATE PROCEDURE AddTask(
    @testId                 int,
    @type                   int,
    @command                nvarchar(255),
    @path                   nvarchar(255),
    @resultXml              bit,
    @useUserCred			bit,
    @newId                  int out
    )
AS
BEGIN
    INSERT INTO DiagnosticTasks
    (TestId, Type, Command, Path, ResultXml, UseUserCred)
    VALUES
    (@testId, @type, @command, @path, @resultXml, @useUserCred)
    
    SET @newId = @@IDENTITY
END
GO


IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'ListTasks'))
DROP PROCEDURE ListTasks
GO

CREATE PROCEDURE ListTasks
AS
BEGIN
    SELECT Id, TestId, Type, Command, Path, ResultXml, UseUserCred
    FROM DiagnosticTasks
END
GO



/*****************************************************************************
** TaskParameters (steps) Procedures
*****************************************************************************/

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'AddTaskParameter'))
DROP PROCEDURE AddTaskParameter
GO

CREATE PROCEDURE AddTaskParameter(
    @taskId                 int,
	@name                   nvarchar(64),
	@value                  nvarchar(255),
	@format                 nvarchar(64)
    )
AS
BEGIN
    INSERT INTO TaskParameters
    (TaskId, Name, Value, Format)
    VALUES
    (@taskId, @name, @value, @format)
END
GO


IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'ListTaskParameters'))
DROP PROCEDURE ListTaskParameters
GO

CREATE PROCEDURE ListTaskParameters
AS
BEGIN
    SELECT TaskId, Name, Value, format
    FROM TaskParameters
END
GO



/*****************************************************************************
** TaskEnvironmentVariables Procedures
*****************************************************************************/

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'AddTaskEnvironmentVariable'))
DROP PROCEDURE AddTaskEnvironmentVariable
GO

CREATE PROCEDURE AddTaskEnvironmentVariable(
    @taskId                 int,
	@name                   nvarchar(255),
	@value                  nvarchar(255)
    )
AS
BEGIN
    INSERT INTO TaskEnvironmentVariables
    (TaskId, Name, Value)
    VALUES
    (@taskId, @name, @value)
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'ListTaskEnvironmentVariables'))
DROP PROCEDURE ListTaskEnvironmentVariables
GO

CREATE PROCEDURE ListTaskEnvironmentVariables
AS
BEGIN
    SELECT TaskId, Name, Value
    FROM TaskEnvironmentVariables
END
GO

/*****************************************************************************
** User table Procedures
*****************************************************************************/

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'SetRunCredential'))
DROP PROCEDURE SetRunCredential
GO

CREATE PROCEDURE SetRunCredential
    @userName				nvarchar(64),
    @runName	            nvarchar(64),
    @password				varbinary(800)
AS
BEGIN

    /* Find a user name id, if not found add it */
    DECLARE @userId INT
    SELECT @userId=Id FROM Users WITH(UPDLOCK) WHERE Name=@userName
    
    IF (@userId is NULL)
		BEGIN
			INSERT INTO Users (Name, RunName, RunPassword ) VALUES (@userName, @runName, @password)
		END
    ELSE
		BEGIN
			UPDATE Users SET RunName = @runName, RunPassword = @password WHERE Name=@userName
		END
		
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'ClearRunCredential'))
DROP PROCEDURE ClearRunCredential
GO

CREATE PROCEDURE ClearRunCredential
    @userName				nvarchar(64)
AS
BEGIN

    /* Find a user name id, if not found add it */
    DECLARE @userId INT
    SELECT @userId=Id FROM Users WITH(UPDLOCK) WHERE Name=@userName
    
    IF NOT (@userId is NULL)
		BEGIN
			UPDATE Users SET RunName = '', RunPassword = NULL WHERE Name=@userName
		END
		
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'GetRunCredentialByRunId'))
DROP PROCEDURE GetRunCredentialByRunId
GO

CREATE PROCEDURE GetRunCredentialByRunId
	@runId          int
AS
BEGIN

	SELECT
		Users.RunName,
		Users.RunPassword
	FROM
		Users
	JOIN
		TestRuns ON TestRuns.RequestedBy = Users.Id
	WHERE
		TestRuns.Id = @runId
		
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'GetRunCredentialByUser'))
DROP PROCEDURE GetRunCredentialByUser
GO

CREATE PROCEDURE GetRunCredentialByUser
	@userName			nvarchar(64)
AS
BEGIN

	SELECT
		RunName,
		RunPassword
	FROM
		Users
	WHERE
		Name = @userName
END
GO

/*****************************************************************************
** TestRuns table Procedures
*****************************************************************************/

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'AddTestRun'))
DROP PROCEDURE AddTestRun
GO

CREATE PROCEDURE AddTestRun(
    @testId                 int,
    @state                  int,
    @prevState              int,
    @userName               nvarchar(64),
    @runUserName			nvarchar(64),
    @runPassword			varbinary(800),
    @newId                  int out
    )
AS
BEGIN

    /* Find a user name id, if not found add it */
    DECLARE @userId INT
    SELECT @userId=Id FROM Users WITH(UPDLOCK) WHERE Name=@userName
    
    IF (@userId is NULL)
    BEGIN
        INSERT INTO Users (Name) VALUES (@userName)
        SELECT @userId=@@IDENTITY
    END
    
    /* Insert a new record into the testruns table*/
    INSERT INTO TestRuns
    (TestId, State, PreviousState,RequestedBy, RunName, RunPassword)
    VALUES
    (@testId, @state, @prevState,@userId, @runUserName, @runPassword)
    
    SET @newId = @@IDENTITY
END
GO


IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'QueryTestRunById'))
DROP PROCEDURE QueryTestRunById
GO

CREATE PROCEDURE QueryTestRunById(
    @runId          int
    )
AS
BEGIN
    SELECT 
        TestRuns.TestId, 
        TestRuns.State,
        TestRuns.PreviousState,
        TestRuns.CreateTime,
        TestRuns.StartTime,
        TestRuns.EndTime,
        TestRuns.PreJobId,
        TestRuns.RunJobId,
        TestRuns.PostJobId,
        TestRuns.RequestCancel,
        TestRuns.Message,
        TestRuns.PublicState,
        TestRuns.UpdateTime,
        TestRuns.PendingArchive,
        TestRuns.Reviewed,
        Users.Name,
        TestRuns.RunName
    FROM 
        TestRuns
    JOIN
        Users ON TestRuns.RequestedBy = Users.Id
    WHERE
        TestRuns.Id=@runId
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'GetRunStepCredential'))
DROP PROCEDURE GetRunStepCredential
GO

CREATE PROCEDURE GetRunStepCredential
	@runId          int
AS
BEGIN

	SELECT
		RunName,
		RunPassword
	FROM
		TestRuns
	WHERE
		TestRuns.Id = @runId
		
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'CheckTestRun'))
DROP PROCEDURE CheckTestRun
GO

CREATE PROCEDURE CheckTestRun(
    @testId         int
)
AS
BEGIN
	SELECT 1 FROM DiagnosticTests test
		JOIN TestRuns run
		ON test.Id = run.TestId
		WHERE test.Id = @testId
			AND run.EndTime = 0
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'IsValidRun'))
DROP PROCEDURE IsValidRun
GO

CREATE PROCEDURE IsValidRun(
    @runId         int
)
AS
BEGIN
	SELECT 1 FROM TestRuns run
		WHERE run.Id = @runId AND run.PendingArchive = 0
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'SetRunState'))
DROP PROCEDURE SetRunState
GO

CREATE PROCEDURE SetRunState(
    @runId          int,
    @newState       int,
    @prevState      int out
)
AS
BEGIN
    SELECT @prevState=State FROM TestRuns with(updlock) WHERE Id=@runId
    
    UPDATE TestRuns
    SET State=@newState,PreviousState=@prevState, UpdateTime=GETUTCDATE()
    WHERE Id=@runId
END
GO


IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'SetRunStateWithPreJobId'))
DROP PROCEDURE SetRunStateWithPreJobId
GO

CREATE PROCEDURE SetRunStateWithPreJobId(
    @runId          int,
    @newState       int,
    @preJobId       int,
    @runJobId       int,
    @prevState      int out
)
AS
BEGIN
    SELECT @prevState=State FROM TestRuns with(updlock) WHERE Id=@runId
    
    UPDATE TestRuns
    SET State=@newState,PreviousState=@prevState,PreJobId=@preJobId, RunJobId=@runJobId, StartTime=GETUTCDATE(), UpdateTime=GETUTCDATE()
    WHERE Id=@runId
END
GO


IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'SetRunStateWithRunJobId'))
DROP PROCEDURE SetRunStateWithRunJobId
GO

CREATE PROCEDURE SetRunStateWithRunJobId(
    @runId          int,
    @newState       int,
    @jobId          int,
    @prevState      int out
)
AS
BEGIN

	DECLARE @startTime datetime
    SELECT @prevState=State, @startTime=StartTime FROM TestRuns with(updlock) WHERE Id=@runId

	IF (YEAR(@startTime) = 1900)
	BEGIN
		-- No prestep
		UPDATE TestRuns
		SET State=@newState,PreviousState=@prevState,RunJobId=@jobId, StartTime=GETUTCDATE(), UpdateTime=GETUTCDATE()
		WHERE Id=@runId
	END
	ELSE
	BEGIN
		UPDATE TestRuns
		SET State=@newState,PreviousState=@prevState,RunJobId=@jobId, UpdateTime=GETUTCDATE()
		WHERE Id=@runId
	END
END
GO


IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'SetRunStateWithPostJobId'))
DROP PROCEDURE SetRunStateWithPostJobId
GO

CREATE PROCEDURE SetRunStateWithPostJobId(
    @runId          int,
    @newState       int,
    @jobId          int,
    @prevState      int out
)
AS
BEGIN
    SELECT @prevState=State FROM TestRuns with(updlock) WHERE Id=@runId
    
    UPDATE TestRuns
    SET State=@newState,PreviousState=@prevState,PostJobId=@jobId, UpdateTime=GETUTCDATE()
    WHERE Id=@runId
END
GO


IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'RequestCancelRun'))
DROP PROCEDURE RequestCancelRun
GO

CREATE PROCEDURE RequestCancelRun(
    @runId          int,
    @message        nvarchar(255),
    @succeeded      int out
)
AS
BEGIN
    UPDATE TestRuns
    SET RequestCancel=1, Message=@message, UpdateTime=GETUTCDATE()
    -- State > 7 (PoststepFinished) included all finalize state, in these kind of states, test can't be cancelled
    WHERE Id=@runId AND State <= 7
    
    SET @succeeded = @@ROWCOUNT
END
GO


IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'SetTestRunMessage'))
DROP PROCEDURE SetTestRunMessage
GO

CREATE PROCEDURE SetTestRunMessage(
    @runId          int,
    @message        nvarchar(255)
)
AS
BEGIN
    UPDATE TestRuns
    SET Message=@message, UpdateTime=GETUTCDATE()
    WHERE Id=@runId
END
GO


IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'SetTestRunPublicState'))
DROP PROCEDURE SetTestRunPublicState
GO

CREATE PROCEDURE SetTestRunPublicState(
    @runId          int,
    @publicState    int
)
AS
BEGIN
    UPDATE TestRuns
    SET PublicState=@publicState, UpdateTime=GETUTCDATE()
    WHERE Id=@runId
END
GO


IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'SetTestRunComplete'))
DROP PROCEDURE SetTestRunComplete
GO

CREATE PROCEDURE SetTestRunComplete(
    @runId          int,
    @publicState    int,
    @runState       int,
    @message        nvarchar(255),
    @now            datetime,
    @needReview		int = 0
)
AS
BEGIN
	IF @message = ''
		UPDATE TestRuns
		SET PublicState=@publicState, PreviousState = State, State=@runState, EndTime=@now, Reviewed=@needReview, UpdateTime=GETUTCDATE()
		WHERE Id=@runId
	ELSE
		UPDATE TestRuns
		SET PublicState=@publicState, PreviousState = State, State=@runState, Message=@message, EndTime=@now, Reviewed=@needReview, UpdateTime=GETUTCDATE()
		WHERE Id=@runId
END
GO


IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'MarkReviewed'))
DROP PROCEDURE MarkReviewed
GO

CREATE PROCEDURE MarkReviewed(
    @runId          int
)
AS	
BEGIN
    UPDATE TestRuns
    SET Reviewed=2, UpdateTime=GETUTCDATE()
    WHERE Id=@runId AND Reviewed = 1
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'GetFailedNodes'))
DROP PROCEDURE GetFailedNodes
GO

CREATE PROCEDURE GetFailedNodes
AS	
BEGIN
    SELECT Name, COUNT(Name) AS Count FROM NodeNames
    JOIN NodeMappings ON NodeNames.Id = NodeMappings.NodeId
    JOIN TestRuns ON TestRuns.Id = NodeMappings.RunId
    WHERE TestRuns.Reviewed = 1 AND TestRuns.PendingArchive = 0 AND (NodeMappings.Result=4 OR NodeMappings.Result=5)
    GROUP BY Name
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'GetAllRunsStatus'))
DROP PROCEDURE GetAllRunsStatus
GO

CREATE PROCEDURE GetAllRunsStatus
AS	
BEGIN
    SELECT PublicState, COUNT(PublicState) As Count FROM TestRuns
    WHERE PendingArchive = 0
    GROUP BY PublicState
END
GO


/*****************************************************************************
** TestRunParameters table Procedures
*****************************************************************************/

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'AddTestRunParameter'))
DROP PROCEDURE AddTestRunParameter
GO

CREATE PROCEDURE AddTestRunParameter(
    @runId                  int,
    @name                   nvarchar(64),
    @value                  nvarchar(255),
    @format                 nvarchar(64)
    )
AS
BEGIN
    INSERT INTO TestRunParameters
    (RunId, Name, Value, Format)
    VALUES
    (@runId, @name, @value, @format)
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'QueryTestRunParametersByRunId'))
DROP PROCEDURE QueryTestRunParametersByRunId
GO

CREATE PROCEDURE QueryTestRunParametersByRunId(
    @runId          int
    )
AS
BEGIN
    SELECT 
        Name, Value, Format
    FROM 
        TestRunParameters
    WHERE
        RunId=@runId
END
GO

/*****************************************************************************
** NodeNames table Procedures
*****************************************************************************/


IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'AddNode'))
DROP PROCEDURE AddNode
GO

CREATE PROCEDURE AddNode(
    @name                   nvarchar(64),
    @newId                  int out
    )
AS
BEGIN
    INSERT INTO NodeNames
    (Name)
    VALUES
    (@name)
    
    SET @newId = @@IDENTITY
END
GO


/*****************************************************************************
** NodeMappings table Procedures
*****************************************************************************/

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'AddNodeMapping'))
DROP PROCEDURE AddNodeMapping
GO

CREATE PROCEDURE AddNodeMapping(
    @runId                  int,
    @nodeId                 int
    )
AS
BEGIN
    INSERT INTO NodeMappings
    (RunId, NodeId)
    VALUES
    (@runId, @nodeId)
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'QueryNodeNamesByRunId'))
DROP PROCEDURE QueryNodeNamesByRunId
GO

CREATE PROCEDURE QueryNodeNamesByRunId(
    @runId                  int
    )
AS
BEGIN
    SELECT 
        NodeNames.Name, NodeMappings.Result
    FROM
        NodeMappings
    JOIN
        NodeNames ON NodeNames.Id=NodeMappings.NodeId
    WHERE
        RunId=@runId
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'SetNodeResult'))
DROP PROCEDURE SetNodeResult
GO

CREATE PROCEDURE SetNodeResult(
    @runId                  int,
    @nodeId					int,
    @result					int
    )
AS
BEGIN
    UPDATE NodeMappings
    SET Result=@result
    WHERE @runId=RunId AND @nodeId=NodeId
END
GO


/*****************************************************************************
** TestResuls table Procedures
*****************************************************************************/


IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'AddResultFile'))
DROP PROCEDURE AddResultFile
GO

CREATE PROCEDURE AddResultFile(
    @runId                  int,
    @filename               nvarchar(64),
    @data                   varbinary(max)
    )
AS
BEGIN
    INSERT INTO TestResults
        (RunId, Filename, Data)
    VALUES
        (@runId, @filename, @data)
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'GetTestResultFileNames'))
DROP PROCEDURE GetTestResultFileNames
GO

CREATE PROCEDURE GetTestResultFileNames(
    @runId                  int
    )
AS
BEGIN
    SELECT Filename FROM TestResults WHERE RunId=@runId
END
GO

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'GetTestResultFile'))
DROP PROCEDURE GetTestResultFile
GO

CREATE PROCEDURE GetTestResultFile(
    @runId                  int,
    @filename               nvarchar(64)
    )
AS
BEGIN
    SELECT Data FROM TestResults WHERE RunId=@runId and Filename=@filename
END
GO

/*****************************************************************************
** Table clean up Procedures
*****************************************************************************/

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'CleanupArchives'))
DROP PROCEDURE CleanupArchives
GO

CREATE PROCEDURE CleanupArchives
AS
BEGIN

	DECLARE @deletedCount int 
	
    -- Delete the runs
    -- Only delete that is archived

    --
    -- Grab the all pending archive rows to delete.
    --
    SELECT Id
    INTO #t
    FROM TestRuns WITH (UPDLOCK)
    WHERE PendingArchive = 1

    SET @deletedCount = @@rowcount

    --
    -- Break if there is no more work to do.
    --
    IF @deletedCount > 0 
    BEGIN

        --
        -- Delete all the rows within a transaction. this would be updated to delete all the tables
        --
        
        DELETE FROM TestRunParameters WHERE RunId IN (SELECT Id FROM #t)
        
        DELETE FROM TestRunVariables WHERE RunId IN (SELECT Id FROM #t)
        
        DELETE FROM TestResults WHERE RunId IN (SELECT Id FROM #t)
        
        DELETE FROM NodeMappings WHERE RunId IN (SELECT Id FROM #t)
        
        DELETE FROM TestRuns WHERE Id IN (SELECT Id FROM #t)
        
    END -- if @@rowcount > 0

	SELECT * FROM #t
	
    DROP TABLE #t

END
GO


IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'ArchiveOldRuns'))
DROP PROCEDURE ArchiveOldRuns
GO

CREATE PROCEDURE ArchiveOldRuns(
    @dayAgeLimit int,
    @archivedCount int out
    )
AS
BEGIN

    -- Archive the runs
    -- Only archive runs that are older than the limit and only
    -- jobs that are in a ended state (has result)
    -- and have been completed at least two days.

	UPDATE TestRuns 
	SET PendingArchive = 1
    WHERE Endtime < DATEADD(DAY, -@dayAgeLimit, GETUTCDATE()) AND PublicState != 0 AND PendingArchive = 0

    SET @archivedCount = @@rowcount

END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'LoadConfig')) 
DROP PROCEDURE LoadConfig
GO

CREATE PROCEDURE LoadConfig
AS
BEGIN
    SELECT Name, Value FROM ClusterConfig
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'UpdateConfig')) 
DROP PROCEDURE UpdateConfig
GO

CREATE PROCEDURE UpdateConfig(
    @name nvarchar(128),
    @value nvarchar(2048)
)
AS
BEGIN
	IF EXISTS (SELECT 1 FROM ClusterConfig WHERE Name = @name)
	    UPDATE ClusterConfig SET Value = @value WHERE Name = @name
	ELSE
		INSERT INTO ClusterConfig (Name, Value) VALUES (@name, @value)
END
GO
DECLARE @dbname varchar(100)
SET @dbname=QUOTENAME(DB_NAME())
BEGIN TRY
  EXEC('ALTER DATABASE '+@dbname+' SET AUTO_CLOSE OFF')
  EXEC('ALTER DATABASE '+@dbname+' SET RECOVERY SIMPLE')
END TRY
BEGIN CATCH
END CATCH
GO
IF EXISTS(SELECT * FROM sys.extended_properties WHERE name='dbVersion') exec sp_dropextendedproperty 'dbVersion'
GO
exec sp_addextendedproperty 'dbVersion', '5.1'
GO
