  /*-------------------------------------------------------------------------------------------------
  * <copyright file="HpcMonitoring.sql" company="Microsoft">
  *     Copyright (c) Microsoft Corporation.  All rights reserved.
  * </copyright>
  * 
  * <summary>
  *    Table & Stored Procedures for Hpc Monitoring
  * </summary>
  *-------------------------------------------------------------------------------------------------
  */

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO

/*
 * Drop all existing tables and views
 */
IF EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[tbhpcmon_vMetricValues]'))
DROP VIEW [dbo].[tbhpcmon_vMetricValues]
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[tbhpcmon_MetricDefinitions]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP table [dbo].[tbhpcmon_MetricDefinitions]
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[tbhpcmon_MinuteMetricValues]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP table [dbo].[tbhpcmon_MinuteMetricValues]
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[tbhpcmon_HourMetricValues]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP table [dbo].[tbhpcmon_HourMetricValues]
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[tbhpcmon_DayMetricValues]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP table [dbo].[tbhpcmon_DayMetricValues]
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[tbhpcmon_MetricInstanceNames]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP table [dbo].[tbhpcmon_MetricInstanceNames]
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[tbhpcmon_ConfigurationSettings]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP table [dbo].[tbhpcmon_ConfigurationSettings]
GO

/*
 * Drop all existing stored procedures
 */

IF EXISTS (SELECT * from sysobjects WHERE name='sphpcmon_AddMetric')
DROP PROCEDURE sphpcmon_AddMetric
GO

IF EXISTS (SELECT * from sysobjects WHERE name='sphpcmon_DeleteMetric')
DROP PROCEDURE sphpcmon_DeleteMetric
GO

IF EXISTS (SELECT * from sysobjects WHERE name='sphpcmon_ParseIntIds')
DROP FUNCTION sphpcmon_ParseIntIds
GO

IF EXISTS (SELECT * from sysobjects WHERE name='sphpcmon_FindMetrics')
DROP PROCEDURE sphpcmon_FindMetrics
GO

IF EXISTS (SELECT * from sysobjects WHERE name='sphpcmon_AddMetricTarget')
DROP PROCEDURE sphpcmon_AddMetricTarget
GO

IF EXISTS (SELECT * from sysobjects WHERE name='sphpcmon_DeleteMetricTarget')
DROP PROCEDURE sphpcmon_DeleteMetricTarget
GO

IF EXISTS (SELECT * from sysobjects WHERE name='sphpcmon_AddMetricInstanceName')
DROP PROCEDURE sphpcmon_AddMetricInstanceName
GO

IF EXISTS (SELECT * from sysobjects WHERE name='sphpcmon_FindInstances')
DROP PROCEDURE sphpcmon_FindInstances
GO

IF EXISTS (SELECT * from sysobjects WHERE name='sphpcmon_DeleteMetricInstanceName')
DROP PROCEDURE sphpcmon_DeleteMetricInstanceName
GO

IF EXISTS (SELECT * from sysobjects WHERE name='sphpcmon_PurgeMetricValues')
DROP PROCEDURE sphpcmon_PurgeMetricValues
GO

IF EXISTS (SELECT * from sysobjects WHERE name='sphpcmon_ReadMetricValues')
DROP PROCEDURE sphpcmon_ReadMetricValues
GO

IF EXISTS (SELECT * from sysobjects WHERE name='sphpcmon_GetConfigurationSettings')
DROP PROCEDURE sphpcmon_GetConfigurationSettings
GO

IF EXISTS (SELECT * from sysobjects WHERE name=N'sphpcmon_SetconfigurationSettings')
DROP PROCEDURE sphpcmon_SetConfigurationSettings
GO


/*
 * Drop all existing table types
 */

IF  EXISTS (SELECT * FROM sys.types st JOIN sys.schemas ss ON st.schema_id = ss.schema_id WHERE st.name = N'GuidList' AND ss.name = N'dbo')
DROP TYPE [dbo].[GuidList]
GO

IF  EXISTS (SELECT * FROM sys.types st JOIN sys.schemas ss ON st.schema_id = ss.schema_id WHERE st.name = N'IntList' AND ss.name = N'dbo')
DROP TYPE [dbo].[IntList]
GO


/*
 * Create table types
 */

CREATE TYPE [dbo].[GuidList] AS TABLE(
	[Id] [uniqueidentifier] NOT NULL,
	PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)
)
GO

CREATE TYPE [dbo].[IntList] AS TABLE(
	[Id] [int] NOT NULL,
	PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)
)

/*
 * Create Metric Definition table
 */

CREATE TABLE [dbo].[tbhpcmon_MetricDefinitions]
(
    [MetricId]          [int]               IDENTITY(1,1) NOT NULL,
    [MetricType]        [int]               NOT NULL,
    [Alias]             [nvarchar](256)     NOT NULL,
    [DisplayName]       [nvarchar](256)     NOT NULL,
    [Description]       [nvarchar](256)     NOT NULL,
    [Category]          [nvarchar](256)     NOT NULL,
    [Name]              [nvarchar](256)     NOT NULL,
    [InstanceFilter]    [nvarchar](256)     NULL,
    [BaseMetricAlias]   [nvarchar](256)     NULL,
    [Calculation]       [int]               NOT NULL,
    [Minimum]           [float]             NOT NULL,
    [Maximum]           [float]             NOT NULL,
    [MetricTarget]      [int]               NULL,
    [StoreRate]         [int]               NOT NULL,
    [SampleRate]        [int]               NOT NULL
) ON [PRIMARY]
GO

ALTER TABLE [dbo].[tbhpcmon_MetricDefinitions] ADD CONSTRAINT
[PK_MetricDefinitions] PRIMARY KEY CLUSTERED 
(
    [MetricId] ASC
) WITH (STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
GO

ALTER TABLE [dbo].[tbhpcmon_MetricDefinitions] ADD CONSTRAINT
[UC_MetricDefinitions] UNIQUE (Alias)
GO

/*
 * Create Minute Metric Values table
 */

CREATE TABLE [dbo].[tbhpcmon_MinuteMetricValues]
(
    [TimeStamp]     [datetime]          NOT NULL,
    [NodeId]        [uniqueidentifier]  NOT NULL,
    [MetricId]      [int]               NOT NULL,
    [InstanceId]    [int]               NOT NULL,
    [Value]         [float]             NOT NULL
CONSTRAINT [PK_tbhpcmon_MinuteMetricValues] PRIMARY KEY CLUSTERED
(
    [TimeStamp] ASC,
    [NodeId] ASC,
    [MetricId] ASC,
    [InstanceId] ASC
))
GO

/*
 * Create Hourly Metric Values table
 */

CREATE TABLE [dbo].[tbhpcmon_HourMetricValues]
(
    [TimeStamp]     [datetime]          NOT NULL,
    [NodeId]        [uniqueidentifier]  NOT NULL,
    [MetricId]      [int]               NOT NULL,
    [InstanceId]    [int]               NOT NULL,
    [Value]         [float]             NOT NULL
CONSTRAINT [PK_tbhpcmon_HourMetricValues] PRIMARY KEY CLUSTERED
(
    [TimeStamp] ASC,
    [NodeId] ASC,
    [MetricId] ASC,
    [InstanceId] ASC
))
GO

/*
 * Create Daily Metric Values table
 */

CREATE TABLE [dbo].[tbhpcmon_DayMetricValues]
(
    [TimeStamp]     [datetime]          NOT NULL,
    [NodeId]        [uniqueidentifier]  NOT NULL,
    [MetricId]      [int]               NOT NULL,
    [InstanceId]    [int]               NOT NULL,
    [Value]         [float]             NOT NULL
CONSTRAINT [PK_tbhpcmon_DayMetricValues] PRIMARY KEY CLUSTERED
(
    [TimeStamp] ASC,
    [NodeId] ASC,
    [MetricId] ASC,
    [InstanceId] ASC
))
GO

/*
 * Create Metric Values view
 */

CREATE VIEW [dbo].[tbhpcmon_vMetricValues]
AS
    SELECT * FROM [dbo].[tbhpcmon_MinuteMetricValues]
    UNION ALL
    SELECT * FROM [dbo].[tbhpcmon_HourMetricValues]
    UNION ALL
    SELECT * FROM [dbo].[tbhpcmon_DayMetricValues]
GO

/*
 * Create Metric Instance Names table
 */

CREATE TABLE [dbo].[tbhpcmon_MetricInstanceNames]
(
    [InstanceId]    [int]           IDENTITY(1,1) NOT NULL,
    [Name]          [nvarchar](256)  NOT NULL UNIQUE
) ON [PRIMARY]
GO

ALTER TABLE [dbo].[tbhpcmon_MetricInstanceNames] ADD CONSTRAINT
[PK_MetricInstanceNames] PRIMARY KEY CLUSTERED 
(
    [InstanceId] ASC
) WITH (STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
GO

CREATE TABLE [dbo].[tbhpcmon_ConfigurationSettings](
    [Name]          [nvarchar](255) NOT NULL,
    [Value]         [nvarchar](max) NOT NULL
) ON [PRIMARY]
GO

/*
 * Create Add Metric sproc
 */

CREATE PROCEDURE [sphpcmon_AddMetric]
    @UpdateExisting     [bit],
    @MetricType         [int],
    @Alias              [nvarchar](256),
    @DisplayName        [nvarchar](256),
    @Description        [nvarchar](256),
    @Category           [nvarchar](256),
    @Name               [nvarchar](256),
    @InstanceFilter     [nvarchar](256),
    @BaseMetricAlias    [nvarchar](256),
    @Calculation        [int],
    @Minimum            [float],
    @Maximum            [float],
    @MetricId           [int] OUTPUT,
    @MetricTarget       [int],
    @SampleRate         [int],
    @StoreRate          [int]
AS
BEGIN
    SELECT @MetricId = [MetricId]
    FROM [dbo].[tbhpcmon_MetricDefinitions]
    WHERE Alias = @Alias

    IF @MetricId IS NULL
    BEGIN
        INSERT INTO
        [dbo].[tbhpcmon_MetricDefinitions](
			[MetricType],
			[Alias],
			[DisplayName],
			[Description],
			[Category],
			[Name],
			[InstanceFilter],
			[BaseMetricAlias],
			[Calculation],
			[Minimum],
			[Maximum],
			[MetricTarget],
			[SampleRate],
			[StoreRate])
        VALUES(
			@MetricType,
			@Alias,
			@DisplayName,
			@Description,
			@Category,
			@Name,
			@InstanceFilter,
			@BaseMetricAlias,
			@Calculation,
			@Minimum,
			@Maximum,
			@MetricTarget,
			@SampleRate,
			@StoreRate)
        SET @MetricId = @@IDENTITY
    END
	ELSE IF @UpdateExisting = 1
	BEGIN
        UPDATE
        [dbo].[tbhpcmon_MetricDefinitions]
		SET
			[MetricType] = @MetricType,
			[Alias] = @Alias,
			[DisplayName] = @DisplayName,
			[Description] = @Description,
			[Category] = @Category,
			[Name] = @Name,
			[InstanceFilter] = @InstanceFilter,
			[BaseMetricAlias] = @BaseMetricAlias,
			[Calculation] = @Calculation,
			[Minimum] = @Minimum,
			[Maximum] = @Maximum,
			[MetricTarget] = @MetricTarget,
			[SampleRate] = @SampleRate,
			[StoreRate] = @StoreRate
        WHERE [MetricId] = @MetricId
	END
END
GO

/*
 * Create Delete Metric sproc
 */

CREATE PROCEDURE [sphpcmon_DeleteMetric]
    @MetricId [int]
AS
BEGIN
    DECLARE @MetricAlias nvarchar(256)
    DECLARE @DependentMetricAlias nvarchar(max)

    SELECT @MetricAlias = Alias
    FROM [dbo].[tbhpcmon_MetricDefinitions]
    WHERE [MetricId] = @MetricId

    IF @MetricAlias IS NULL
    BEGIN
        RAISERROR('metric with Id %d does not exist.', 16, 1, @MetricId)
        RETURN
    END

	SELECT @DependentMetricAlias = COALESCE(@DependentMetricAlias + N',', N'') + Convert(nvarchar(256), Alias)
	FROM [dbo].[tbhpcmon_MetricDefinitions]
	WHERE [BaseMetricAlias] = @MetricAlias

    IF @DependentMetricAlias IS NOT NULL
    BEGIN
        RAISERROR('can not delete metric %s because calculated metric %s depend(s) on it.', 16, 1, @MetricAlias, @DependentMetricAlias)
        RETURN
    END

    DELETE FROM [dbo].[tbhpcmon_MetricDefinitions]
    WHERE [MetricId] = @MetricId
END
GO 


/* 
 * Create ParseIntIds helper function
 *
 * Parses a list of comma separated ints, returns a table that 
 * contains the ints.
 */

CREATE FUNCTION [sphpcmon_ParseIntIds] (@intIdStr [nvarchar](MAX))
RETURNS @ParsedIds TABLE
(
    [ParsedId] [int]
)
AS
BEGIN
    DECLARE @strLen [int]
    DECLARE @startIndex [int]
    DECLARE @endIndex [int]
    DECLARE @intString [nvarchar](100)
    DECLARE @counterId [int]

    SET @startIndex = 0
    SET @endIndex = 0
    SET @strLen = LEN(@intIdStr)

    WHILE(@endIndex < @strLen)
    BEGIN
        SET @endIndex = CHARINDEX(',',@intIdStr, @startIndex)
        IF (@endIndex = 0)
        BEGIN
            SET @endIndex = @strLen + 1
        END

        SET @intString = SUBSTRING(@intIdStr, @startIndex, @endIndex - @startIndex)
        INSERT @ParsedIds values(@intString)
        SET @startIndex = @endIndex + 1
    END
RETURN
END
GO

 
/* 
 * Create Find Metrics stored procedure
 */

CREATE PROCEDURE [sphpcmon_FindMetrics]
    @MetricTarget [int] = NULL,
    @MetricIdStr [nvarchar](MAX) = NULL
AS
BEGIN
    IF (@MetricTarget IS NULL and @MetricIdStr IS NULL)
    BEGIN
        SELECT  [MetricId],
                [MetricType],
                [Alias],
                [DisplayName],
                [Description],
                [Category],
                [Name],
                [InstanceFilter],
                [BaseMetricAlias],
                [Calculation],
                [Minimum],
                [Maximum],
                [MetricTarget],
                [SampleRate],
                [StoreRate]
        FROM [tbhpcmon_MetricDefinitions]
    END
    ELSE IF (@MetricTarget IS NULL) /* MetricIdStr is not null */
    BEGIN
        SELECT  [MetricId],
                [MetricType],
                [Alias],
                [DisplayName],
                [Description],
                [Category],
                [Name],
                [InstanceFilter],
                [BaseMetricAlias],
                [Calculation],
                [Minimum],
                [Maximum],
                [MetricTarget],
                [SampleRate],
                [StoreRate]
        FROM [sphpcmon_ParseIntIds](@MetricIdStr)
        INNER JOIN [tbhpcmon_MetricDefinitions]
        ON [tbhpcmon_MetricDefinitions].[MetricId] = ParsedId
        OPTION(FORCE ORDER)
    END
    ELSE IF (@MetricIdStr IS NULL) /* @MetricTarget is not null */
    BEGIN
        SELECT  [tbhpcmon_MetricDefinitions].[MetricId],
                [MetricType],
                [Alias],
                [DisplayName],
                [Description],
                [Category],
                [Name],
                [InstanceFilter],
                [BaseMetricAlias],
                [Calculation],
                [Minimum],
                [Maximum],
                [MetricTarget],
                [SampleRate],
                [StoreRate]
        FROM [tbhpcmon_MetricDefinitions]
        WHERE ([MetricTarget] & @MetricTarget) != 0
    END
END
GO 

/*
 * Create Add Metric Target sproc
 */

CREATE PROCEDURE [sphpcmon_AddMetricTarget]
    @MetricId           [int],
    @MetricTarget       [int]
AS
BEGIN
    UPDATE [dbo].[tbhpcmon_MetricDefinitions]
    SET [MetricTarget] = [MetricTarget] | @MetricTarget
    WHERE MetricId = @MetricId
END
GO

/*
 * Create Delete Metric Target sproc
 */

CREATE PROCEDURE [sphpcmon_DeleteMetricTarget]
    @MetricId [int],
    @MetricTarget [int]
AS
BEGIN
    UPDATE [dbo].[tbhpcmon_MetricDefinitions]
    SET [MetricTarget] &= ~([MetricTarget] | @MetricTarget)
    WHERE MetricId = @MetricId
END
GO 

/*
 * Create Add Metric Instance Name sproc
 */

CREATE PROCEDURE [sphpcmon_AddMetricInstanceName]
    @Name       [nvarchar](256),
    @InstanceId [int] OUTPUT
AS
BEGIN
    INSERT INTO [dbo].[tbhpcmon_MetricInstanceNames]([Name]) 
    VALUES(@Name)
    SET @InstanceId = @@IDENTITY
END
GO

/* 
 * Create Find Instances stored procedure
 */

CREATE PROCEDURE [sphpcmon_FindInstances]
    @InstanceIdStr [nvarchar](MAX) = NULL
AS
BEGIN
    IF (@InstanceIdStr IS NULL)
    BEGIN
        SELECT  [InstanceId],
                [Name]
        FROM [tbhpcmon_MetricInstanceNames]
    END
    ELSE
    BEGIN
        SELECT  [InstanceId],
                [Name]
        FROM [sphpcmon_ParseIntIds](@InstanceIdStr)
        INNER JOIN [tbhpcmon_MetricInstanceNames]
        ON [tbhpcmon_MetricInstanceNames].[InstanceId] = ParsedId OPTION(FORCE ORDER)
    END
END
GO

/*
 * Create Delete Metric Target sproc
 */

CREATE PROCEDURE [sphpcmon_DeleteMetricInstanceName]
    @InstanceId [int]
AS
BEGIN
    DELETE FROM [dbo].[tbhpcmon_MetricInstanceNames] WHERE [InstanceId] = @InstanceId
END
GO 

/*
 * Create Purge Metric Values sproc
 */

CREATE PROCEDURE [sphpcmon_PurgeMetricValues](
    @MinuteBefore [datetime],
    @HourBefore [datetime],
    @DayBefore [datetime]
    )
AS
BEGIN
    DELETE TOP (1000000) FROM [dbo].[tbhpcmon_MinuteMetricValues] WHERE [TimeStamp] < @MinuteBefore
    DELETE TOP (1000000) FROM [dbo].[tbhpcmon_HourMetricValues] WHERE [TimeStamp] < @HourBefore
    DELETE TOP (1000000) FROM [dbo].[tbhpcmon_DayMetricValues] WHERE [TimeStamp] < @DayBefore
END
GO


/*
 * Create Read Metric Values sproc
 */

CREATE PROCEDURE [sphpcmon_ReadMetricValues](
    @NodeIdList [GuidList] READONLY,
    @Before [datetime],
    @After [datetime],
    @MetricIdList IntList READONLY
    )
AS
BEGIN
    DECLARE @JoinNodeId bit = 0
    DECLARE @JoinMetricId bit = 0

    IF EXISTS(SELECT * FROM @NodeIdList)
    BEGIN
	    SET @JoinNodeId = 1
    END

    IF EXISTS(SELECT * FROM @MetricIdList)
    BEGIN
        SET @JoinMetricId = 1
    END

    IF @JoinNodeId = 1 AND @JoinMetricId = 1
    BEGIN
        SELECT mv.TimeStamp,
            mv.NodeId,
            mv.MetricId,
            mv.InstanceId,
            mv.Value
        FROM tbhpcmon_vMetricValues mv
        JOIN @NodeIdList nl ON nl.Id = mv.NodeId
        JOIN @MetricIdList ml ON ml.Id = mv.MetricId
        WHERE mv.TimeStamp >= @After
        AND mv.TimeStamp < @Before
        ORDER BY TimeStamp ASC
    END
    ELSE IF @JoinNodeId = 1
    BEGIN
        SELECT mv.TimeStamp,
            mv.NodeId,
            mv.MetricId,
            mv.InstanceId,
            mv.Value
        FROM tbhpcmon_vMetricValues mv
        JOIN @NodeIdList nl ON nl.Id = mv.NodeId
        WHERE mv.TimeStamp >= @After
        AND mv.TimeStamp < @Before
        ORDER BY TimeStamp ASC
    END
    ELSE IF @JoinMetricId = 1
    BEGIN
        SELECT mv.TimeStamp,
            mv.NodeId,
            mv.MetricId,
            mv.InstanceId,
            mv.Value
        FROM tbhpcmon_vMetricValues mv
        JOIN @MetricIdList ml ON ml.Id = mv.MetricId
        WHERE mv.TimeStamp >= @After
        AND mv.TimeStamp < @Before
        ORDER BY TimeStamp ASC
    END
    ELSE
    BEGIN
        SELECT mv.TimeStamp,
            mv.NodeId,
            mv.MetricId,
            mv.InstanceId,
            mv.Value
        FROM tbhpcmon_vMetricValues mv
        WHERE mv.TimeStamp >= @After
        AND mv.TimeStamp < @Before
        ORDER BY TimeStamp ASC
    END
END
GO 


/*
 * Create Get Configuration Setting sproc
 */

CREATE PROCEDURE [dbo].[sphpcmon_GetConfigurationSettings]
    @Name nvarchar(255)
AS
BEGIN   
    SELECT Value FROM [dbo].[tbhpcmon_ConfigurationSettings] WHERE Name = @Name
END
GO


/*
 * Create Set Configuration Setting sproc
 */

CREATE PROCEDURE [dbo].[sphpcmon_SetConfigurationSettings]
    @Name nvarchar(255),
    @Value nvarchar(max)
AS
BEGIN   
    IF EXISTS (SELECT * FROM [dbo].[tbhpcmon_ConfigurationSettings] WHERE Name = @Name)
    BEGIN
        UPDATE [dbo].[tbhpcmon_ConfigurationSettings]
        SET Value = @Value
        WHERE Name = @Name
    END
    ELSE
    BEGIN
        INSERT INTO [dbo].[tbhpcmon_ConfigurationSettings](Name, Value)
        VALUES(@Name, @Value)
    END
END
GO
DECLARE @dbname varchar(100)
SET @dbname=QUOTENAME(DB_NAME())
BEGIN TRY
  EXEC('ALTER DATABASE '+@dbname+' SET AUTO_CLOSE OFF')
  EXEC('ALTER DATABASE '+@dbname+' SET RECOVERY SIMPLE')
END TRY
BEGIN CATCH
END CATCH
GO
IF EXISTS(SELECT * FROM sys.extended_properties WHERE name='dbVersion') exec sp_dropextendedproperty 'dbVersion'
go
exec sp_addextendedproperty 'dbVersion', '5.1'
go
