

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_DropDefaultConstraint]')) 
DROP PROCEDURE dbo.SP_DropDefaultConstraint
GO
CREATE PROCEDURE SP_DropDefaultConstraint
    @tablename    nvarchar(100),
    @colname    nvarchar(100)
AS
BEGIN
    DECLARE @defname VARCHAR(100), @cmd VARCHAR(1000)
    SET @defname = 
    (SELECT name 
    FROM sysobjects so JOIN sysconstraints sc
    ON so.id = sc.constid 
    WHERE object_name(so.parent_obj) = @tablename 
    AND so.xtype = 'D'
    AND sc.colid = 
     (SELECT colid FROM syscolumns 
     WHERE id = object_id('dbo.'+@tablename) AND 
     name = @colname))
    SET @cmd = 'ALTER TABLE ' + @tablename + ' DROP CONSTRAINT ' + @defname
    EXEC(@cmd)
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_DropColumn]')) 
DROP PROCEDURE dbo.SP_DropColumn
GO
CREATE PROCEDURE SP_DropColumn
    @tablename    nvarchar(100),
    @colname    nvarchar(100)
AS
BEGIN
    DECLARE @cmd VARCHAR(1000)
    EXEC SP_DropDefaultConstraint @tablename, @colname
    SET @cmd = 'ALTER TABLE ' + @tablename + ' DROP COLUMN ' + @colname
    EXEC(@cmd)
END
GO

-- Rename the Zombies to Zombie
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Zombie]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    EXEC sp_rename 'Zombies', 'Zombie'
    ALTER TABLE Zombie ALTER COLUMN JobId int NULL
END
GO

-- Deprecate the CachedProcedures (later)
--IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[CachedProcedures]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
--DROP TABLE [dbo].[CachedProcedures]
--GO

-- Create a table of integers for efficient parametric task expansion
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Numbers]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP TABLE [dbo].[Numbers]
GO
CREATE TABLE Numbers
(
Number int Not NULL PRIMARY KEY CLUSTERED
) On [Primary]
GO

DECLARE @num INT
SET @num = 1
BEGIN TRANSACTION
WHILE ( @num <= 1000000)
BEGIN
    INSERT Numbers( Number ) VALUES( @num )
    SET @num = @num+ 1
END
COMMIT TRANSACTION
GO

-- Rename the StoreNodeCounters to NodeCounter
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[NodeCounter]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[StoreNodeCounters]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
        EXEC sp_rename 'StoreNodeCounters', 'NodeCounter'
END
GO

-- Rename the StoreResourceCounters to ResourceCounter
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[ResourceCounter]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[StoreResourceCounters]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
        EXEC sp_rename 'StoreResourceCounters', 'ResourceCounter'
END
GO

-- Rename the NodeResourceCounters to NodeResourceCounter
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[NodeResourceCounter]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[NodeResourceCounters]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
        EXEC sp_rename 'NodeResourceCounters', 'NodeResourceCounter'
END
GO

-- Rename the StoreJobCounters to JobCounter
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[JobCounter]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[StoreJobCounters]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
        EXEC sp_rename 'StoreJobCounters', 'JobCounter'
END
GO

-- Rename the StoreTaskCounters to TaskCounter
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[TaskCounter]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[StoreTaskCounters]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
        EXEC sp_rename 'StoreTaskCounters', 'TaskCounter'
END
GO

-- Rename the ParametricTaskCounters to ParametricTaskCounter
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[ParametricTaskCounter]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[ParametricTaskCounters]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
        EXEC sp_rename 'ParametricTaskCounters', 'ParametricTaskCounter'
END
GO

-- Rename the JobTaskCounters to JobTaskCounter
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[JobTaskCounter]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[JobTaskCounters]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
        EXEC sp_rename 'JobTaskCounters', 'JobTaskCounter'
END
GO

-- Rename the Schd_NodeNames to NodeName
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[NodeName]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Schd_NodeNames]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
        EXEC sp_rename 'Schd_NodeNames', 'NodeName'
END
GO

-- Rename the Schd_NodeHistory to NodeHistory
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[NodeHistory]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Schd_NodeHistory]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
        EXEC sp_rename 'Schd_NodeHistory', 'NodeHistory'
END
GO

-- Rename the Schd_JobHistory to JobHistory
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[JobHistory]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Schd_JobHistory]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    BEGIN
        EXEC sp_rename 'Schd_JobHistory', 'JobHistory'

        ALTER TABLE JobHistory
        ADD 
        ActorSid            nvarchar(80)        NULL,
        CancelRequest       int                 NULL
    END
END
GO

-- Rename the Schd_SourceNames to ClientSourceName
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[ClientSourceName]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Schd_SourceNames]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
        EXEC sp_rename 'Schd_SourceNames', 'ClientSourceName'
END
GO

-- Rename the Schd_ProjectNames to JobProjectName
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[JobProjectName]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Schd_ProjectNames]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
        EXEC sp_rename 'Schd_ProjectNames', 'JobProjectName'
END
GO

-- Rename the TaskGroupNames to TaskGroup
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[TaskGroup]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[TaskGroupNames]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
        EXEC sp_rename 'TaskGroupNames', 'TaskGroup'
END
GO

-- Rename the TaskGroupDependencies to TaskGroupDependency
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[TaskGroupDependency]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[TaskGroupDependencies]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
        EXEC sp_rename 'TaskGroupDependencies', 'TaskGroupDependency'
END
GO

-- Rename the TaskEnvVars to TaskEnvironmentVariable
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[TaskEnvironmentVariable]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[TaskEnvVars]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    BEGIN
        EXEC sp_rename 'TaskEnvVars', 'TaskEnvironmentVariable'

        -- Change the column length of VariableValue to nvarchar(max)
        EXEC SP_DropDefaultConstraint 'TaskEnvironmentVariable', 'VariableValue'

        ALTER TABLE TaskEnvironmentVariable 
        ALTER COLUMN VariableValue nvarchar(max) NOT NULL 

        ALTER TABLE TaskEnvironmentVariable
        ADD CONSTRAINT DF_TaskEnvironmentVariable_VariableValue DEFAULT('') FOR VariableValue 
    END
END
GO

-- Add the JobEnvironmentVariable table
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[JobEnvironmentVariable]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP TABLE [dbo].[JobEnvironmentVariable]
GO
CREATE TABLE JobEnvironmentVariable
(
    JobId               int             NOT NULL DEFAULT(0),
    VariableName        nvarchar(128)   NOT NULL,
    VariableValue       nvarchar(max)   NOT NULL DEFAULT('')
) ON [PRIMARY]
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[JobShrinkRequest]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP TABLE [dbo].[JobShrinkRequest]
GO
CREATE TABLE JobShrinkRequest
(
    JobId               int             NOT NULL DEFAULT(0),
    ResourceId          int             NOT NULL DEFAULT(0),
    NodeId              int             NOT NULL DEFAULT(0),
    ShrinkRequest       int             NOT NULL DEFAULT(0)
) ON [PRIMARY]

GO


-- Rename the ResourceReservations to ResourceReservation
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[ResourceReservation]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[ResourceReservations]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
        EXEC sp_rename 'ResourceReservations', 'ResourceReservation'
END
GO

-- Rename the Profiles to JobTemplate

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Profiles]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    -- 1. Backup all SecurityDescriptors
    IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[BackupSecDesc]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
        DROP TABLE [dbo].[BackupSecDesc]

    SELECT Name, SecurityDescriptor INTO BackupSecDesc FROM Profiles

    -- 2. Drop Profiles
    DROP TABLE Profiles
END
GO

/*
** Create JobTemplate table
*/

IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[JobTemplate]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
CREATE TABLE JobTemplate
(
    Id                      int             IDENTITY(1,1) NOT NULL PRIMARY KEY CLUSTERED,
    Name                    nvarchar(50)    unique NULL,
    Enabled                 bit             NOT NULL CONSTRAINT [DF_JobTemplate_Enabled]  DEFAULT (1),
    CreateTime              datetime        NOT NULL DEFAULT(GETUTCDATE()),
    ModifyTime              datetime        NULL,
    TimeStamp               timestamp       NOT NULL,
    SecurityDescriptor      nvarchar(2000)  NULL,
    Description             nvarchar(255)   NULL,
) ON [PRIMARY]
END
GO


-- 1.Rename the ProfileItems to JobTemplateItem
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[JobTemplateItem]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[ProfileItems]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
        EXEC sp_rename 'ProfileItems', 'JobTemplateItem'
END
GO

-- 2. Clean the table
DELETE FROM JobTemplateItem
GO

-- Rename the PropertyNames to CustomPropertyName
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[CustomPropertyName]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[PropertyNames]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
        EXEC sp_rename 'PropertyNames', 'CustomPropertyName'
END
GO

-- Rename the ExtendedJobProperties to ExtendedJobProperty
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[ExtendedJobProperty]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[ExtendedJobProperties]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
        EXEC sp_rename 'ExtendedJobProperties', 'ExtendedJobProperty'
END
GO

-- Rename the ExtendedTaskProperties to ExtendedTaskProperty
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[ExtendedTaskProperty]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[ExtendedTaskProperties]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
        EXEC sp_rename 'ExtendedTaskProperties', 'ExtendedTaskProperty'
END
GO

-- Add the JobMessage table
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[JobMessage]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP TABLE [dbo].[JobMessage]
GO
CREATE TABLE JobMessage
(
    Id                      int         IDENTITY(1,1) NOT NULL PRIMARY KEY CLUSTERED,
    LastMessageTime         datetime    NOT NULL DEFAULT(GETUTCDATE()),
    JobId                   int         NOT NULL DEFAULT(0),
    MessageType             int         NOT NULL DEFAULT(0),
    MessageCode             int         NOT NULL DEFAULT(0),
    MessageSubCode          int         NOT NULL DEFAULT(0),
    MessageCount            int         NOT NULL DEFAULT(0),
    ExtraText            nvarchar(max)  NOT NULL DEFAULT('')
) ON [PRIMARY]
GO 

-- Add the JobCancelRequest table
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[JobCancelRequest]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP TABLE [dbo].[JobCancelRequest]
GO
CREATE TABLE JobCancelRequest
(
    JobId                  int             NOT NULL PRIMARY KEY CLUSTERED,
    Request                int             NOT NULL,
) ON [PRIMARY]
GO

-- Add the TaskCancelRequest table
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[TaskCancelRequest]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP TABLE [dbo].[TaskCancelRequest]
GO
CREATE TABLE TaskCancelRequest
(
    TaskId                  int             NOT NULL PRIMARY KEY CLUSTERED,
    Request                    int             NOT NULL,
) ON [PRIMARY]
GO

-- Drop the Schd_Owners table (Credential and SID table have already been created in the pre-check program)
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Schd_Owners]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP TABLE [dbo].[Schd_Owners]
GO

-- Deal with the Jobs table
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Job]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    -- 1. Alter the Jobs table and remove the RequestCancel, Username, Password, OwnerId
    DROP INDEX Jobs.jobs_reqcancel_idx

    DROP STATISTICS Jobs.jobs_reqcancel_stats

    EXEC SP_DropColumn 'Jobs', 'RequestCancel'

    EXEC SP_DropColumn 'Jobs', 'Username'

    EXEC SP_DropColumn 'Jobs', 'Password'

    EXEC SP_DropColumn 'Jobs', 'OwnerId'
    
    EXEC SP_DropColumn 'Jobs', 'ParentJobID'
    
    EXEC SP_DropColumn 'Jobs', 'ChildJobID'
        

    -- 2. Alter the Jobs table and add the column CredentialId, IsAuthenticated, Progress, ProgressMessage
    -- HasNodePropTask, HasNodeReleaseTask, HasServiceTask, IsMultiPhase, TargetResourceCount
    ALTER TABLE Jobs
    ADD
        ExcludedNodes       nvarchar(max)   NULL,
        CredentialId        int             NOT NULL DEFAULT(1),
        IsAuthenticated     bit             NOT NULL DEFAULT(0),
        Progress            int             NULL,
        ProgressMessage     nvarchar(80)    NULL,
        HasNodePrepTask     bit             NOT NULL DEFAULT(0),
        HasNodeReleaseTask  bit             NOT NULL DEFAULT(0),
        HasServiceTask      bit             NOT NULL DEFAULT(0),
        IsMultiPhase        bit             NOT NULL DEFAULT(0),
        TargetResourceCount int             NULL,
        HoldUntil           datetime        NULL,
        NotifyOnStart       bit             NOT NULL DEFAULT(0),
        NotifyOnCompletion  bit             NOT NULL DEFAULT(0),
        ProxyNode           nvarchar(64)    NULL,
        ProfileMinResources int             NULL,
        ProfileMaxResources int             NULL
        

    ALTER TABLE Jobs DROP COLUMN ComputedNodeList

    EXEC sp_rename 'Jobs.[Priority]', 'ExpandedPriority', 'COLUMN'

    -- 3. Rename the Jobs table to Job
    EXEC sp_rename 'Jobs', 'Job'
END
GO

-- 4. Import the CredentialId data and drop the JobCredentialMapping table
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[JobCredentialMapping]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    UPDATE Job 
    SET Job.CredentialId = JobCredentialMapping.CredentialId
    FROM Job, JobCredentialMapping 
    WHERE Job.Id = JobCredentialMapping.JobId

    DROP TABLE [dbo].[JobCredentialMapping]

    UPDATE Job
    SET ExpandedPriority = ExpandedPriority * 1000
    WHERE ExpandedPriority IS NOT NULL AND ExpandedPriority < 5
END
GO

-- 5. Fix a bug in V2 that Configuring jobs can have EndTime
UPDATE Job
SET EndTime = NULL
WHERE State = 1 AND EndTime is not NULL

-- Deal with the Tasks_Main2 table
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Task]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    -- 1. Add in the FailedNodeId column
    -- 2. Drop the InstanceValue column
    -- 3. Alter the Tasks_Main2 table to remove RequestCancel
    ALTER TABLE Tasks_Main2
    ADD
        FailedNodeId       int         NOT NULL DEFAULT(0)
    
    ALTER TABLE Tasks_Main2 DROP COLUMN InstanceValue

    DROP INDEX Tasks_Main2.tasks_reqcancel_idx

    DROP STATISTICS Tasks_Main2.tasks_reqcancel_stats

    EXEC SP_DropColumn 'Tasks_Main2', 'RequestCancel'

    -- 2. Rename Tasks_Main2 to Task
    EXEC sp_rename 'Tasks_Main2', 'Task'
END
GO


-- Deal with the Tasks_Settings2 table

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Tasks_Settings2]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Tasks_Settings2]') AND name = N'StartValueNew')
BEGIN
    -- 1. Create StartValueNew, EndValueNew, IncrementalValueNew, Type, CurrentInstanceId/Value column
    ALTER TABLE Tasks_Settings2
    ADD
        StartValueNew       int         NOT NULL DEFAULT(0),
        EndValueNew         int         NOT NULL DEFAULT(0),
        IncrementValueNew   int         NOT NULL DEFAULT(1),
        Type                int         NOT NULL DEFAULT(0),
        CurrentInstanceId   int         NOT NULL DEFAULT(0),
        IsServiceConcluded  bit         NOT NULL DEFAULT(0)
END
GO


IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[TaskDetail]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN

    -- 2. Import the Start/End/IncValue column's value to the Start/End/IncValueNew columns; 
    UPDATE Tasks_Settings2
    SET StartValueNew = COALESCE(StartValue, 0),
        EndValueNew = COALESCE(EndValue, 0),
        IncrementValueNew = COALESCE(IncrementValue, 1)

    -- 3. Import the Type value from IsParametric
    UPDATE Tasks_Settings2
    SET Type = 1
    WHERE IsParametric = 1

    -- 4. Drop the StartValue, EndValue, IncrementValue, IsParametric
    EXEC SP_DropColumn 'Tasks_Settings2', 'StartValue'

    EXEC SP_DropColumn 'Tasks_Settings2', 'EndValue'

    EXEC SP_DropColumn 'Tasks_Settings2', 'IncrementValue'

    EXEC SP_DropColumn 'Tasks_Settings2', 'IsParametric'

    -- 5. Rename the New columns 
    EXEC sp_rename 'Tasks_Settings2.[StartValueNew]','StartValue', 'COLUMN'

    EXEC sp_rename 'Tasks_Settings2.[EndValueNew]','EndValue', 'COLUMN'

    EXEC sp_rename 'Tasks_Settings2.[IncrementValueNew]','IncrementValue', 'COLUMN'

    -- 6. Set the CurrentInstanceId/Value
    UPDATE Tasks_Settings2
    SET CurrentInstanceId = -1
    WHERE IncrementValue > 0 AND EndValue > StartValue

    -- 6. Alter the DependsOnTasks column to nvarchar(max)
    ALTER TABLE Tasks_Settings2
    ALTER COLUMN DependsOnTasks nvarchar(max)   NULL

    -- 7. Rename the Tasks_Settings2 table to TaskDetail
    EXEC sp_rename 'Tasks_Settings2', 'TaskDetail'
END
GO

--Deal with Nodes table
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Node]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    -- 1. Alter the Nodes table and add IsForcedOffline column
    ALTER TABLE Nodes
    ADD IsForcedOffline bit NOT NULL DEFAULT(0)

    -- 2. Rename Nodes to Node
    EXEC sp_rename 'Nodes', 'Node'
END
GO

--Deal with Resources table
IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Resource]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    -- 1. Alter the Resources table and add JobPhase column
    ALTER TABLE Resources
    ADD JobPhase int NOT NULL DEFAULT(0)

    -- 2. Change all Job/TaskId = 0 to Job/TaskId = NULL
    UPDATE Resources SET JobId = NULL WHERE JobId = 0

    UPDATE Resources SET TaskId = NULL WHERE TaskId = 0

    -- 3. Drop the Default constraint for JobId and TaskId columns
    EXEC SP_DropDefaultConstraint 'Resources', 'JobId'

    EXEC SP_DropDefaultConstraint 'Resources', 'TaskId'

    -- 4. Rename Resources to Resource
    EXEC sp_rename 'Resources', 'Resource'
END
GO


-- Alter the AllocationHistory table and add CoreId column
IF NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[AllocationHistory]') AND name = N'CoreId')
BEGIN
    ALTER TABLE AllocationHistory
    ADD CoreId int NOT NULL DEFAULT(0)
END
GO

IF NOT EXISTS (SELECT * from dbo.[JobMessage])
BEGIN
    -- Add error messages from existing jobs into the job messages table
    INSERT INTO JobMessage
    (
        LastMessageTime, 
        JobId, 
        MessageType, 
        MessageCode, 
        MessageSubCode, 
        MessageCount, 
        ExtraText
    )
    SELECT
        GETUTCDATE(),
        Id,
        CASE WHEN (State=512 OR State=1024) THEN 3 ELSE 2 END,  -- Message Type is JobCancellation iff the state is Canceled or Canceling
        ErrorCode,
        0,
        1,
        ErrorParams
    FROM Job
    WHERE ErrorCode <> 0
END
GO


-- Deal with indexes
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[JobEnvironmentVariable]') AND name = N'IDX_JobEnvironmentVariable_JobId')
    CREATE NONCLUSTERED INDEX IDX_JobEnvironmentVariable_JobId ON JobEnvironmentVariable(JobId)
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TaskEnvironmentVariable]') AND name = N'IDX_TaskEnvironmentVariable_JobId')
    CREATE NONCLUSTERED INDEX IDX_TaskEnvironmentVariable_JobId ON TaskEnvironmentVariable (JobId)
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[JobShrinkRequest]') AND name = N'IDX_JobShrinkRequest_JobId')
    CREATE CLUSTERED INDEX IDX_JobShrinkRequest_JobId ON JobShrinkRequest(JobId,ResourceId)
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[JobMessage]') AND name = N'IDX_JobMessage_JobId')
    CREATE NONCLUSTERED INDEX IDX_JobMessage_JobId ON JobMessage(JobId)
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Zombie]') AND name = N'IDX_Zombie_Closed')
    EXEC sp_rename 'Zombie.zombies_closed_idx', 'IDX_Zombie_Closed', 'INDEX'
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[NodeName]') AND name = N'IDX_NodeName_NodeNameGuid')
    EXEC sp_rename 'NodeName.NodeNames_Idx', 'IDX_NodeName_NodeNameGuid', 'INDEX'
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[NodeName]') AND name = N'IDX_NodeName_Guid')
    CREATE NONCLUSTERED INDEX IDX_NodeName_Guid on NodeName(GUID) 
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[JobHistory]') AND name = N'IDX_JobHistory_JobId')
    EXEC sp_rename 'JobHistory.sjh_jobid_idx', 'IDX_JobHistory_JobId', 'INDEX'
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Credential]') AND name = N'IDX_Credential_RunAsId')
    CREATE NONCLUSTERED INDEX IDX_Credential_RunAsId ON Credential(RunAsId)
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Credential]') AND name = N'IDX_Credential_OwnerId')
    CREATE NONCLUSTERED INDEX IDX_Credential_OwnerId ON Credential(OwnerId)
GO
IF EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Job]') AND name = N'jobs_state_idx')
    DROP INDEX Job.jobs_state_idx
GO
IF EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Job]') AND name = N'jobs_project')
    DROP INDEX Job.jobs_project
GO
IF EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Job]') AND name = N'jobs_jobid_requeue')
    DROP INDEX Job.jobs_jobid_requeue
GO
IF EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Job]') AND name = N'jobs_queue_pri_idx')
    DROP INDEX Job.jobs_queue_pri_idx
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Job]') AND name = N'IDX_Job_ForScheduler')
    CREATE NONCLUSTERED INDEX IDX_Job_ForScheduler ON job (ExpandedPriority DESC, submittime ASC) INCLUDE (state, type, preemptable, HoldUntil) WHERE state IN (16, 32, 64, 1024)
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Job]') AND name = N'IDX_Job_ForValidator')
    CREATE NONCLUSTERED INDEX IDX_Job_ForValidator ON job (ExpandedPriority DESC, submittime ASC) INCLUDE (state, type) WHERE state IN (2,4)
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Job]') AND name = N'IDX_Job_StateType')
    CREATE NONCLUSTERED INDEX IDX_Job_StateType ON job (state,type) 
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Job]') AND name = N'IDX_Job_ProjectId')
    CREATE NONCLUSTERED INDEX IDX_Job_ProjectId ON job( projectid )
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Job]') AND name = N'IDX_Job_CredentialId')
    CREATE NONCLUSTERED INDEX IDX_Job_CredentialId ON job( credentialId )
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Job]') AND name = N'IDX_Job_HoldUntil')
    CREATE NONCLUSTERED INDEX IDX_Job_HoldUntil ON job (HoldUntil ) where HoldUntil is not null
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TaskGroup]') AND name = N'IDX_TaskGroup_JobId')
    EXEC sp_rename 'TaskGroup.TaskGroupNames_JobId_idx', 'IDX_TaskGroup_JobId', 'INDEX'
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TaskGroupDependency]') AND name = N'PK_TaskGroupDependency')
    EXEC sp_rename 'TaskGroupDependency.TaskGroupDependencies_PK', 'PK_TaskGroupDependency', 'INDEX'
GO
IF EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Task]') AND name = N'tasks2_taskrecordid_idx')
    DROP INDEX Task.tasks2_taskrecordid_idx
GO
IF EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Task]') AND name = N'tasks2_recordidstate_idx')
    DROP INDEX Task.tasks2_recordidstate_idx
GO
IF EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Task]') AND name = N'tasks2_taskinstanceid_idx')
    DROP INDEX Task.tasks2_taskinstanceid_idx
GO
IF EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Task]') AND name = N'tasks2_state_idx')
    DROP INDEX Task.tasks2_state_idx
GO
IF EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Task]') AND name = N'tasks2_parentjobid_idx')
    DROP INDEX Task.tasks2_parentjobid_idx
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Task]') AND name = N'IDX_Task_RecordIdInstanceId')
    CREATE NONCLUSTERED INDEX IDX_Task_RecordIdInstanceId ON Task(recordid, instanceId) INCLUDE (parentjobid)
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Task]') AND name = N'IDX_Task_ParentJobIdInstanceId')
    CREATE NONCLUSTERED INDEX IDX_Task_ParentJobIdInstanceId ON Task(parentjobid,instanceid) INCLUDE ( recordid )
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Task]') AND name = N'IDX_Task_Submitted')
    CREATE NONCLUSTERED INDEX IDX_Task_Submitted ON Task(parentjobid, instanceid) INCLUDE (requeuecount) WHERE state = 2
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Task]') AND name = N'IDX_Task_Closed')
    CREATE NONCLUSTERED INDEX IDX_Task_Closed ON Task(instanceid, parentjobid) WHERE closed = 1 AND state IN (32,64,1024)
GO
IF EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TaskDetail]') AND name = N'Tasks2_TaskGroupId_idx')
    DROP INDEX TaskDetail.Tasks2_TaskGroupId_idx
GO
IF EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TaskDetail]') AND name = N'Tasks2_TaskGroupIdRecordId_idx')
    DROP INDEX TaskDetail.Tasks2_TaskGroupIdRecordId_idx
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TaskDetail]') AND name = N'IDX_TaskDetail_GroupIdRecordId')
    CREATE NONCLUSTERED INDEX IDX_TaskDetail_GroupIdRecordId ON TaskDetail(GroupId, recordId) include (currentInstanceId)
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TaskEnvironmentVariable]') AND name = N'IDX_TaskEnvironmentVariable_TaskId')
    EXEC sp_rename 'TaskEnvironmentVariable.TaskEnvVars_TaskId', 'IDX_TaskEnvironmentVariable_TaskId', 'INDEX'
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Node]') AND name = N'IDX_Node_State')
    EXEC sp_rename 'Node.Nodes_State_Idx', 'IDX_Node_State', 'INDEX'
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Node]') AND name = N'IDX_Node_Name')
    EXEC sp_rename 'Node.Node_name_idx', 'IDX_Node_Name', 'INDEX'
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Node]') AND name = N'IDX_Node_SID')
    EXEC sp_rename 'Node.Node_sid_idx', 'IDX_Node_SID', 'INDEX'
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Node]') AND name = N'IDX_Node_Reachable')
    EXEC sp_rename 'Node.Node_reachable_idx', 'IDX_Node_Reachable', 'INDEX'    
GO

IF EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Node]') AND name = N'Node_nodeID_idx')
    DROP INDEX Node.Node_nodeID_idx
GO
IF EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Resource]') AND name = N'resources_job_idx')
    DROP INDEX Resource.resources_job_idx
GO
IF EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Resource]') AND name = N'resources_nodeId_idx')
    DROP INDEX Resource.resources_nodeId_idx
GO
IF EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Resource]') AND name = N'resources_taskid_idx')
    DROP INDEX Resource.resources_taskid_idx
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Resource]') AND name = N'IDX_Resource_JobIdNodeIdSocketId')
    CREATE NONCLUSTERED INDEX IDX_Resource_JobIdNodeIdSocketId ON Resource(jobid, nodeid, socketid) include (jobtype, reachable, movetooffline) 
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Resource]') AND name = N'IDX_Resource_NodeIdSocketId')
    CREATE NONCLUSTERED INDEX IDX_Resource_NodeIdSocketId ON Resource(nodeid, socketid) include (jobtype, reachable, movetooffline) 
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Resource]') AND name = N'IDX_Resource_TaskId')
    CREATE NONCLUSTERED INDEX IDX_Resource_TaskId ON Resource(taskid) 
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Resource]') AND name = N'IDX_Resource_Unreachable')    
    CREATE NONCLUSTERED INDEX IDX_Resource_Unreachable ON Resource(id) where reachable = 0
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Resource]') AND name = N'IDX_Resource_MoveToOffline')    
    CREATE NONCLUSTERED INDEX IDX_Resource_MoveToOffline ON Resource(id)  where movetooffline = 1 
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[JobTemplate]') AND name = N'IDX_JobTemplate_Name')
    CREATE UNIQUE NONCLUSTERED INDEX IDX_JobTemplate_Name ON JobTemplate(Name)
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[JobTemplateItem]') AND name = N'IDX_JobTemplateItem_JobTemplateId')
    EXEC sp_rename 'JobTemplateItem.ProfileItems_ItemJobTemplateId_idx', 'IDX_JobTemplateItem_JobTemplateId', 'INDEX'
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[AllocationHistory]') AND name = N'IDX_AllocationHistory_TaskIdTaskRequeueCount')
    EXEC sp_rename 'AllocationHistory.ah_task_count_idx', 'IDX_AllocationHistory_TaskIdTaskRequeueCount', 'INDEX'
GO
IF EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[AllocationHistory]') AND name = N'ah_task_node_idx')
    DROP INDEX AllocationHistory.ah_task_node_idx
GO
IF EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[AllocationHistory]') AND name = N'ah_job_count_idx')
    DROP INDEX AllocationHistory.ah_job_count_idx
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[AllocationHistory]') AND name = N'IDX_AllocationHistory_JobIdTaskIdJobRequeueCount')
    CREATE NONCLUSTERED INDEX IDX_AllocationHistory_JobIdTaskIdJobRequeueCount ON allocationhistory( jobid, taskid, jobrequeuecount ) INCLUDE (resourceId)
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[AllocationHistory]') AND name = N'IDX_AllocationHistory_Stage')
    CREATE NONCLUSTERED INDEX [IDX_AllocationHistory_Stage] ON [dbo].[AllocationHistory] ([ModifyTime] ASC,[TaskId] ASC) INCLUDE ( [Id])
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[NodeAllocationHistory]') AND name = N'IDX_NodeAllocationHistory_JobId')
    EXEC sp_rename 'NodeAllocationHistory.nah_jobidx', 'IDX_NodeAllocationHistory_JobId', 'INDEX'
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[NodeAllocationHistory]') AND name = N'IDX_NodeAllocationHistory_TaskIdNodeId')
    EXEC sp_rename 'NodeAllocationHistory.nah_task_node_idx', 'IDX_NodeAllocationHistory_TaskIdNodeId', 'INDEX'
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[CustomPropertyName]') AND name = N'IDX_CustomPropertyName_ObjectType')
    EXEC sp_rename 'CustomPropertyName.PropertyNames_objectType_idx', 'IDX_CustomPropertyName_ObjectType', 'INDEX'
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ExtendedJobProperty]') AND name = N'PK_ExtendedJobProperty')
    EXEC sp_rename 'ExtendedJobProperty.jobExtProps_pk', 'PK_ExtendedJobProperty', 'INDEX'
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ExtendedJobProperty]') AND name = N'IDX_ExtendedJobProperty_PropertyId')
    EXEC sp_rename 'ExtendedJobProperty.JobExtProps_propertyId_idx', 'IDX_ExtendedJobProperty_PropertyId', 'INDEX'
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ExtendedTaskProperty]') AND name = N'PK_ExtendedTaskProperty')
    EXEC sp_rename 'ExtendedTaskProperty.TaskExtProps_pk', 'PK_ExtendedTaskProperty', 'INDEX'
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ExtendedTaskProperty]') AND name = N'IDX_ExtendedTaskProperty_PropertyId')
    EXEC sp_rename 'ExtendedTaskProperty.TaskExtProps_propertyId_idx', 'IDX_ExtendedTaskProperty_PropertyId', 'INDEX'
GO

-- Deal with statistics
IF EXISTS (SELECT * FROM sys.stats WHERE object_id = OBJECT_ID(N'[dbo].[Job]') AND name = N'jobs_state_stats')
BEGIN
    DROP STATISTICS Job.jobs_state_stats
    CREATE STATISTICS STAT_Job_StateType ON job( state, type )
END
GO

IF EXISTS (SELECT * FROM sys.stats WHERE object_id = OBJECT_ID(N'[dbo].[Task]') AND name = N'tasks_state_stats')
BEGIN
    DROP STATISTICS Task.tasks_state_stats
    DROP STATISTICS Task.task_parent_stats
    CREATE STATISTICS STAT_Task_ParnetJobId ON Task( parentjobid)
END
GO

IF EXISTS (SELECT * FROM sys.stats WHERE object_id = OBJECT_ID(N'[dbo].[Node]') AND name = N'node_reachable_stats')
BEGIN
    DROP STATISTICS Node.node_reachable_stats
    CREATE STATISTICS STAT_Node_Reachable ON Node ( reachable )
END
GO


IF EXISTS (SELECT * FROM sys.stats WHERE object_id = OBJECT_ID(N'[dbo].[AllocationHistory]') AND name = N'allocationhistory_nodeid_idx')
BEGIN
    DROP STATISTICS AllocationHistory.allocationhistory_nodeid_idx
END
GO

-- Remove all previous store procedures
IF EXISTS (SELECT id FROM sysobjects WHERE id=object_id(N'Schd_CreateJobDefaultTaskGroup'))
DROP PROCEDURE Schd_CreateJobDefaultTaskGroup
GO
IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'Schd_UpdateTaskGroupMaxMinCores'))
DROP PROCEDURE Schd_UpdateTaskGroupMaxMinCores
GO
IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'Schd_UpdateTaskGroupMaxMinNodes'))
DROP PROCEDURE Schd_UpdateTaskGroupMaxMinNodes
GO
IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'Schd_UpdateTaskGroupMaxMinSockets'))
DROP PROCEDURE Schd_UpdateTaskGroupMaxMinSockets
GO
IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'Schd_SubmitJobTasks'))
DROP PROCEDURE Schd_SubmitJobTasks
GO
IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'Schd_QueueParametricTasksFromSubmitted'))
DROP PROCEDURE Schd_QueueParametricTasksFromSubmitted
GO
IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'Schd_SetCancelParametricTasks'))
DROP PROCEDURE Schd_SetCancelParametricTasks
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_DeleteParametricTasks]')) 
DROP PROCEDURE dbo.SchedulerStore_DeleteParametricTasks
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[InsertParametricChild]')) 
DROP PROCEDURE dbo.InsertParametricChild
GO
IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'Schd_NextTaskId'))
DROP PROCEDURE Schd_NextTaskId
GO
IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'Schd_UpdateLimitTime'))
DROP PROCEDURE Schd_UpdateLimitTime
GO
IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'Schd_CreateResource'))
DROP PROCEDURE Schd_CreateResource
GO
IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'Schd_BringResourcesOnline'))
DROP PROCEDURE Schd_BringResourcesOnline
GO
IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'Schd_TakeResourcesOffline'))
DROP PROCEDURE Schd_TakeResourcesOffline
GO
IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'SchedulerStore_NodeGuidToId'))
DROP PROCEDURE SchedulerStore_NodeGuidToId
GO
IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'SchedulerStore_NodeNameToId'))
DROP PROCEDURE SchedulerStore_NodeNameToId
GO
IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'SchedulerStore_NodeSidToId'))
DROP PROCEDURE SchedulerStore_NodeSidToId
GO
IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'CloseJobEntry'))
DROP PROCEDURE CloseJobEntry
GO
IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'CloseTaskEntry'))
DROP PROCEDURE CloseTaskEntry
GO
IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'UpdateTaskNodeStats'))
DROP PROCEDURE UpdateTaskNodeStats
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[OpenJobAllocationEntry]')) 
DROP PROCEDURE dbo.OpenJobAllocationEntry
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[OpenTaskAllocationEntry]')) 
DROP PROCEDURE dbo.OpenTaskAllocationEntry
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[AddJobHistoryRecord]')) 
DROP PROCEDURE dbo.AddJobHistoryRecord
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_FetchExtendedJobProps]')) 
DROP PROCEDURE dbo.SchedulerStore_FetchExtendedJobProps
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_SetExtendedJobProp]')) 
DROP PROCEDURE dbo.SchedulerStore_SetExtendedJobProp
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_SetExtendedTaskProp]')) 
DROP PROCEDURE dbo.SchedulerStore_SetExtendedTaskProp
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_FetchExtendedTaskProps]')) 
DROP PROCEDURE dbo.SchedulerStore_FetchExtendedTaskProps
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_CreatePropertyName]')) 
DROP PROCEDURE dbo.SchedulerStore_CreatePropertyName
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_FindPropertyName]')) 
DROP PROCEDURE dbo.SchedulerStore_FindPropertyName
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_GetPropertyInfo]')) 
DROP PROCEDURE dbo.SchedulerStore_GetPropertyInfo
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_GetProperties]')) 
DROP PROCEDURE dbo.SchedulerStore_GetProperties
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_AddNode2]'))
DROP PROCEDURE dbo.SchedulerStore_AddNode2
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_UpdateHistoricNodeProps]'))
DROP PROCEDURE dbo.SchedulerStore_UpdateHistoricNodeProps
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_WriteNodeEvent]'))
DROP PROCEDURE dbo.SchedulerStore_WriteNodeEvent
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_WriteNodeEvent2]'))
DROP PROCEDURE dbo.SchedulerStore_WriteNodeEvent2
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_NodePropsForResourceQuery]')) 
DROP PROCEDURE dbo.SchedulerStore_NodePropsForResourceQuery
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_AddTaskGroupDependency]')) 
DROP PROCEDURE dbo.SchedulerStore_AddTaskGroupDependency
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_RemoveTaskGroup]')) 
DROP PROCEDURE dbo.SchedulerStore_RemoveTaskGroup
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_FetchTaskGroupDependencies]')) 
DROP PROCEDURE dbo.SchedulerStore_FetchTaskGroupDependencies
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_GetDefaultTaskGroup]')) 
DROP PROCEDURE dbo.SchedulerStore_GetDefaultTaskGroup
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_GetTaskGroupsForJob]')) 
DROP PROCEDURE dbo.SchedulerStore_GetTaskGroupsForJob
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_DeleteJob2]')) 
DROP PROCEDURE dbo.SchedulerStore_DeleteJob2
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_DeleteTask2]')) 
DROP PROCEDURE dbo.SchedulerStore_DeleteTask2
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_FindTaskIdByInstanceId]')) 
DROP PROCEDURE dbo.SchedulerStore_FindTaskIdByInstanceId
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_FindTaskId]')) 
DROP PROCEDURE dbo.SchedulerStore_FindTaskId
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_FindTaskParentId]')) 
DROP PROCEDURE dbo.SchedulerStore_FindTaskParentId
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_DeleteProfileItem]')) 
DROP PROCEDURE dbo.SchedulerStore_DeleteProfileItem
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_FindProfileItem]')) 
DROP PROCEDURE dbo.SchedulerStore_FindProfileItem
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_ModifyProfileItem]')) 
DROP PROCEDURE dbo.SchedulerStore_ModifyProfileItem
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_AddProfileItem]')) 
DROP PROCEDURE dbo.SchedulerStore_AddProfileItem
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_DeleteProfileItems]')) 
DROP PROCEDURE dbo.SchedulerStore_DeleteProfileItems
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_ListProfileItems]')) 
DROP PROCEDURE dbo.SchedulerStore_ListProfileItems
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_GetProfileItem]')) 
DROP PROCEDURE dbo.SchedulerStore_GetProfileItem
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_RemoveNode]')) 
DROP PROCEDURE dbo.SchedulerStore_RemoveNode
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_RemoveNodeResource]')) 
DROP PROCEDURE dbo.SchedulerStore_RemoveNodeResource
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_UpdateResourceJobType]')) 
DROP PROCEDURE dbo.SchedulerStore_UpdateResourceJobType
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_UpdateResourceJobType2]')) 
DROP PROCEDURE dbo.SchedulerStore_UpdateResourceJobType2
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_NodeIdToNodeBaseProps]')) 
DROP PROCEDURE dbo.SchedulerStore_NodeIdToNodeBaseProps
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_NodeNameToNodeBaseProps]')) 
DROP PROCEDURE dbo.SchedulerStore_NodeNameToNodeBaseProps
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_NodeSidToNodeBaseProps]')) 
DROP PROCEDURE dbo.SchedulerStore_NodeSidToNodeBaseProps
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_IdToNodeId]')) 
DROP PROCEDURE dbo.SchedulerStore_IdToNodeId
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_VerifyJobId]')) 
DROP PROCEDURE dbo.SchedulerStore_VerifyJobId
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_NodeIdAndTaskIdToAllocationId]')) 
DROP PROCEDURE dbo.SchedulerStore_NodeIdAndTaskIdToAllocationId
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_NodeIdJobIdAndTaskIdToAllocationId]')) 
DROP PROCEDURE dbo.SchedulerStore_NodeIdJobIdAndTaskIdToAllocationId
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_ResourceIdAndJobIdToAllocationId]')) 
DROP PROCEDURE dbo.SchedulerStore_ResourceIdAndJobIdToAllocationId
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_ResourceIdAndTaskIdToAllocationId]')) 
DROP PROCEDURE dbo.SchedulerStore_ResourceIdAndTaskIdToAllocationId
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_SetTaskEnvVar]')) 
DROP PROCEDURE dbo.SchedulerStore_SetTaskEnvVar
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_AddTaskEnvVar]')) 
DROP PROCEDURE dbo.SchedulerStore_AddTaskEnvVar
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_ListAllTaskEnvVar]')) 
DROP PROCEDURE dbo.SchedulerStore_ListAllTaskEnvVar
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_ListTaskEnvVar]')) 
DROP PROCEDURE dbo.SchedulerStore_ListTaskEnvVar
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_DeleteTaskEnvVars]')) 
DROP PROCEDURE dbo.SchedulerStore_DeleteTaskEnvVars
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_GetTaskPids]')) 
DROP PROCEDURE dbo.SchedulerStore_GetTaskPids
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_GetJobPids]')) 
DROP PROCEDURE dbo.SchedulerStore_GetJobPids
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_JobAllocatedNodes]')) 
DROP PROCEDURE dbo.SchedulerStore_JobAllocatedNodes
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_JobAllocatedSockets]')) 
DROP PROCEDURE dbo.SchedulerStore_JobAllocatedSockets
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_JobAllocatedCores]')) 
DROP PROCEDURE dbo.SchedulerStore_JobAllocatedCores
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_TaskAllocatedNodes]')) 
DROP PROCEDURE dbo.SchedulerStore_TaskAllocatedNodes
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_TaskAllocatedSockets]')) 
DROP PROCEDURE dbo.SchedulerStore_TaskAllocatedSockets
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_TaskAllocatedCores]')) 
DROP PROCEDURE dbo.SchedulerStore_TaskAllocatedCores
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_AddChildJob]')) 
DROP PROCEDURE dbo.SchedulerStore_AddChildJob
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_ResetNodeResourceCounters]')) 
DROP PROCEDURE dbo.SchedulerStore_ResetNodeResourceCounters
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_ResetStoreResourceCounters]')) 
DROP PROCEDURE dbo.SchedulerStore_ResetStoreResourceCounters
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_ResetStoreNodeCounters]')) 
DROP PROCEDURE dbo.SchedulerStore_ResetStoreNodeCounters
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_ResetStoreJobCounters]')) 
DROP PROCEDURE dbo.SchedulerStore_ResetStoreJobCounters
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_ResetStoreTaskCounters]')) 
DROP PROCEDURE dbo.SchedulerStore_ResetStoreTaskCounters
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_DeleteOldJobs]')) 
DROP PROCEDURE dbo.SchedulerStore_DeleteOldJobs
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_ResetStoreCounters]')) 
DROP PROCEDURE dbo.SchedulerStore_ResetStoreCounters
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_PeriodicCounterFix]')) 
DROP PROCEDURE dbo.SchedulerStore_PeriodicCounterFix
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_736367912]')) 
DROP PROCEDURE dbo.SchedulerStore_736367912
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_324521873]')) 
DROP PROCEDURE dbo.SchedulerStore_324521873
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_CleanupZombieResources]')) 
DROP PROCEDURE dbo.SchedulerStore_CleanupZombieResources
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SchedulerStore_1050912345]')) 
DROP PROCEDURE dbo.SchedulerStore_1050912345
GO
IF EXISTS (SELECT id FROM sysobjects WHERE id=object_id(N'SchedulerStore_1556400730'))
DROP PROCEDURE SchedulerStore_1556400730
GO
IF EXISTS (SELECT id FROM sysobjects WHERE id=object_id(N'SchedulerStore_1736864592'))
DROP PROCEDURE SchedulerStore_1736864592
GO
IF EXISTS (SELECT id FROM sysobjects WHERE id=object_id(N'SchedulerStore_1029188861'))
DROP PROCEDURE SchedulerStore_1029188861
GO
IF EXISTS (SELECT id FROM sysobjects WHERE id=object_id(N'SchedulerStore_735993146'))
DROP PROCEDURE SchedulerStore_735993146
GO
IF EXISTS (SELECT id FROM sysobjects WHERE id=object_id(N'schedulerstore_1111301383'))
DROP PROCEDURE schedulerstore_1111301383
GO
IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'SchedulerStore_AggregateJobs_539543714'))
DROP PROCEDURE SchedulerStore_AggregateJobs_539543714
GO
IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'SchedulerStore_AggregateJobs_267906877'))
DROP PROCEDURE SchedulerStore_AggregateJobs_267906877
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_DropColumn]')) 
DROP PROCEDURE dbo.SP_DropColumn
GO
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_DropDefaultConstraint]')) 
DROP PROCEDURE dbo.SP_DropDefaultConstraint
GO

-- Remove Management stuff

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ParseDefinitions]') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
DROP FUNCTION [dbo].[ParseDefinitions]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ParseInstanceIds]') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
DROP FUNCTION [dbo].[ParseInstanceIds]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ParseIntIds]') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
DROP FUNCTION [dbo].[ParseIntIds]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ParseNameValues]') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
DROP FUNCTION [dbo].[ParseNameValues]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ParseStringList]') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
DROP FUNCTION [dbo].[ParseStringList]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ParseUniqueIds]') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
DROP FUNCTION [dbo].[ParseUniqueIds]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[QuoteAuthString]') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
DROP FUNCTION [dbo].[QuoteAuthString]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TranslateOwnerId]') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
DROP FUNCTION [dbo].[TranslateOwnerId]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TranslateOwnerSid]') AND type in (N'FN', N'IF', N'TF', N'FS', N'FT'))
DROP FUNCTION [dbo].[TranslateOwnerSid]
GO


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ActionDependencies]') AND type in (N'U'))
DROP TABLE [dbo].[ActionDependencies]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ActionSettingValues]') AND type in (N'U'))
DROP TABLE [dbo].[ActionSettingValues]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Actions]') AND type in (N'U'))
DROP TABLE [dbo].[Actions]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Aspects]') AND type in (N'U'))
DROP TABLE [dbo].[Aspects]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ChangeReferences]') AND type in (N'U'))
DROP TABLE [dbo].[ChangeReferences]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ChangeSettingValues]') AND type in (N'U'))
DROP TABLE [dbo].[ChangeSettingValues]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ChangeUpdateSequence]') AND type in (N'U'))
DROP TABLE [dbo].[ChangeUpdateSequence]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DayCounters]') AND type in (N'U'))
DROP TABLE [dbo].[DayCounters]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Definitions]') AND type in (N'U'))
DROP TABLE [dbo].[Definitions]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[HourCounters]') AND type in (N'U'))
DROP TABLE [dbo].[HourCounters]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[InstancePermissions]') AND type in (N'U'))
DROP TABLE [dbo].[InstancePermissions]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[LogEntries]') AND type in (N'U'))
DROP TABLE [dbo].[LogEntries]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Managers]') AND type in (N'U'))
DROP TABLE [dbo].[Managers]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Members]') AND type in (N'U'))
DROP TABLE [dbo].[Members]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[MinuteCounters]') AND type in (N'U'))
DROP TABLE [dbo].[MinuteCounters]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[PerformanceCatalog]') AND type in (N'U'))
DROP TABLE [dbo].[PerformanceCatalog]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Relationships]') AND type in (N'U'))
DROP TABLE [dbo].[Relationships]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[SettingValues]') AND type in (N'U'))
DROP TABLE [dbo].[SettingValues]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Instances]') AND type in (N'U'))
DROP TABLE [dbo].[Instances]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Documents]') AND type in (N'U'))
DROP TABLE [dbo].[Documents]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Changes]') AND type in (N'U'))
DROP TABLE [dbo].[Changes]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[UserAuthorization]') AND type in (N'U'))
DROP TABLE [dbo].[UserAuthorization]
GO


IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AddAction]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[AddAction]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AddActionDependency]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[AddActionDependency]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AddActionSetting]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[AddActionSetting]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AddAspect]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[AddAspect]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AddAuthorizedUser]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[AddAuthorizedUser]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AddChange]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[AddChange]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AddChangeFinalize]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[AddChangeFinalize]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AddChangeSetting]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[AddChangeSetting]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AddCounter]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[AddCounter]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AddDocument]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[AddDocument]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AddInstance]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[AddInstance]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AddLogEntry]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[AddLogEntry]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AddManager]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[AddManager]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AddMember]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[AddMember]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AddPermission]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[AddPermission]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AddRelationship]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[AddRelationship]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[AddValue]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[AddValue]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DeleteCounter]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[DeleteCounter]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FindChanges]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FindChanges]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FindCounters]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FindCounters]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FindDocuments]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FindDocuments]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FindInstances]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[FindInstances]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[GetActionDependencies]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[GetActionDependencies]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[GetActions]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[GetActions]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[GetActionSettings]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[GetActionSettings]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[GetAspects]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[GetAspects]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[GetAuthorizedUsers]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[GetAuthorizedUsers]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[GetChange]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[GetChange]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[GetChangeInstances]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[GetChangeInstances]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[GetChangeSettings]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[GetChangeSettings]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[GetCurrentSequenceId]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[GetCurrentSequenceId]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[GetDocument]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[GetDocument]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[GetInstance]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[GetInstance]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[GetLogEntries]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[GetLogEntries]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[GetManager]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[GetManager]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[GetMembers]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[GetMembers]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[GetPermissions]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[GetPermissions]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[GetRelationships]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[GetRelationships]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[GetValues]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[GetValues]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[PurgeCounterValues]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[PurgeCounterValues]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ReadCounterValues]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[ReadCounterValues]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[UpdateChangeInstance]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[UpdateChangeInstance]
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[UpdateTaskNodeStats]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].UpdateTaskNodeStats
GO


IF  EXISTS (SELECT * FROM sys.types st JOIN sys.schemas ss ON st.schema_id = ss.schema_id WHERE st.name = N'ChangeName' AND ss.name = N'dbo')
DROP TYPE [dbo].[ChangeName]
GO
IF  EXISTS (SELECT * FROM sys.types st JOIN sys.schemas ss ON st.schema_id = ss.schema_id WHERE st.name = N'DefinitionOrMemberName' AND ss.name = N'dbo')
DROP TYPE [dbo].[DefinitionOrMemberName]
GO
IF  EXISTS (SELECT * FROM sys.types st JOIN sys.schemas ss ON st.schema_id = ss.schema_id WHERE st.name = N'DocumentCulture' AND ss.name = N'dbo')
DROP TYPE [dbo].[DocumentCulture]
GO
IF  EXISTS (SELECT * FROM sys.types st JOIN sys.schemas ss ON st.schema_id = ss.schema_id WHERE st.name = N'DocumentFullName' AND ss.name = N'dbo')
DROP TYPE [dbo].[DocumentFullName]
GO
IF  EXISTS (SELECT * FROM sys.types st JOIN sys.schemas ss ON st.schema_id = ss.schema_id WHERE st.name = N'DocumentPlatform' AND ss.name = N'dbo')
DROP TYPE [dbo].[DocumentPlatform]
GO
IF  EXISTS (SELECT * FROM sys.types st JOIN sys.schemas ss ON st.schema_id = ss.schema_id WHERE st.name = N'DocumentPublicKeyToken' AND ss.name = N'dbo')
DROP TYPE [dbo].[DocumentPublicKeyToken]
GO
IF  EXISTS (SELECT * FROM sys.types st JOIN sys.schemas ss ON st.schema_id = ss.schema_id WHERE st.name = N'DocumentVersion' AND ss.name = N'dbo')
DROP TYPE [dbo].[DocumentVersion]
GO
IF  EXISTS (SELECT * FROM sys.types st JOIN sys.schemas ss ON st.schema_id = ss.schema_id WHERE st.name = N'InstanceName' AND ss.name = N'dbo')
DROP TYPE [dbo].[InstanceName]
GO
IF  EXISTS (SELECT * FROM sys.types st JOIN sys.schemas ss ON st.schema_id = ss.schema_id WHERE st.name = N'OperatorName' AND ss.name = N'dbo')
DROP TYPE [dbo].[OperatorName]
GO
IF  EXISTS (SELECT * FROM sys.types st JOIN sys.schemas ss ON st.schema_id = ss.schema_id WHERE st.name = N'UserName' AND ss.name = N'dbo')
DROP TYPE [dbo].[UserName]
GO


IF  EXISTS (SELECT * FROM sys.assemblies asms WHERE asms.name = N'ccpSqlHelpers')
DROP ASSEMBLY [ccpSqlHelpers]
GO

IF  EXISTS (SELECT 1 FROM fn_listextendedproperty('dbVersion', default, default, default, default, default, default))
    exec sp_updateextendedproperty 'dbVersion', '3'
ELSE
    exec sp_addextendedproperty 'dbVersion', '3'
GO

SET ANSI_PADDING OFF