SET ANSI_PADDING ON
GO

--Turn off AUTO_CLOSE
DECLARE @sqlScript NVARCHAR(2000);
SET @sqlScript = ('ALTER DATABASE ' + QUOTENAME(DB_NAME()) + N' SET AUTO_CLOSE OFF');
EXEC sp_executesql @sqlScript;
GO

-- Utility
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_DropDefaultConstraint]')) 
DROP PROCEDURE dbo.SP_DropDefaultConstraint
GO
CREATE PROCEDURE SP_DropDefaultConstraint
    @tablename    nvarchar(100),
    @colname    nvarchar(100)
AS
BEGIN
    DECLARE @defname VARCHAR(100), @cmd VARCHAR(1000)
    SET @defname = 
    (SELECT name 
    FROM sysobjects so JOIN sysconstraints sc
    ON so.id = sc.constid 
    WHERE object_name(so.parent_obj) = @tablename 
    AND so.xtype = 'D'
    AND sc.colid = 
     (SELECT colid FROM syscolumns 
     WHERE id = object_id('dbo.'+@tablename) AND 
     name = @colname))
    SET @cmd = 'ALTER TABLE ' + @tablename + ' DROP CONSTRAINT ' + @defname
    EXEC(@cmd)
END
GO

-- Modify Job Table
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Job]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Job]') AND name = N'EmailAddress')
BEGIN
    ALTER TABLE Job ADD  EmailAddress nvarchar(256) NULL
END
GO

-- Modify Task table
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Task]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	-- V3SP1 Changes the ErrorParam column to nvarchar(max)
    EXEC SP_DropDefaultConstraint 'Task', 'ErrorParams'

    ALTER TABLE Task 
    ALTER COLUMN ErrorParams nvarchar(max) NOT NULL 

    ALTER TABLE Task
    ADD CONSTRAINT DF_Task_ErrorParams DEFAULT('') FOR ErrorParams 

END
GO

-- Modify Task Details table

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[TaskDetail]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[TaskDetail]') AND name = N'FailJobOnFailure')
BEGIN
    ALTER TABLE TaskDetail ADD FailJobOnFailure bit NOT NULL DEFAULT(0);
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[TaskDetail]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[TaskDetail]') AND name = N'FailJobOnFailureCount')
BEGIN
    ALTER TABLE TaskDetail ADD FailJobOnFailureCount int NOT NULL DEFAULT(1);
END
GO

-- Modify Nodes table

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Node]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Node]') AND name = N'Availability')
BEGIN
    ALTER TABLE Node ADD Availability int NOT NULL DEFAULT(1);
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Node]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Node]') AND name = N'Location')
BEGIN
    ALTER TABLE Node ADD Location int NOT NULL DEFAULT(1);
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Node]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Node]') AND name = N'AzureDeploymentId')
BEGIN
    ALTER TABLE Node ADD AzureDeploymentId nvarchar(64) NULL;
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Node]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Node]') AND name = N'AzureNodeAddress')
BEGIN
    ALTER TABLE Node ADD AzureNodeAddress nvarchar(256) NULL;
END
GO

-- Modify Resource table

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Resource]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
	AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Resource]') AND name = N'IsAvailable')
BEGIN
	ALTER TABLE Resource ADD IsAvailable bit NOT NULL DEFAULT(1);
END
GO


--- Modify JobTemplateItem table
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[JobTemplateItem]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    ALTER TABLE   JobTemplateItem ALTER COLUMN  DefaultString  nvarchar(max)   NULL
    ALTER TABLE   JobTemplateItem ALTER COLUMN  StringValues   nvarchar(max)   NULL
    ALTER TABLE   JobTemplateItem ALTER COLUMN  RequiredValues nvarchar(max)   NULL    
END


IF EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Resource]') AND name = N'IDX_Resource_JobIdNodeIdSocketId')
    DROP INDEX Resource.IDX_Resource_JobIdNodeIdSocketId;
GO

CREATE NONCLUSTERED INDEX IDX_Resource_JobIdNodeIdSocketId ON Resource(JobId, NodeId, SocketId) include (JobType, Reachable, MoveToOffline, IsAvailable);
GO

IF EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Resource]') AND name = N'IDX_Resource_NodeIdSocketId')
    DROP INDEX Resource.IDX_Resource_NodeIdSocketId;
GO

CREATE NONCLUSTERED INDEX IDX_Resource_NodeIdSocketId ON Resource(NodeId, SocketId) include (JobType, Reachable, MoveToOffline, IsAvailable);
GO

-- Add Azure deployment table

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[AzureDeployment]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    DROP TABLE AzureDeployment;
END
GO

IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[AzureDeployment]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
CREATE TABLE AzureDeployment
(
	DeploymentId			nvarchar(64)	NOT NULL,
	SubscriptionId			nvarchar(128)	NULL,
	ServiceName				nvarchar(128)	NULL,
	StorageConnectionString nvarchar(1024) NULL,
	ProxyMultiplicity		int				NOT NULL DEFAULT(1)
) ON [PRIMARY]
END
GO


-- Add the reusable flag to the credentials table
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Credential]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
	AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Credential]') AND name = N'Reusable')
BEGIN
	ALTER TABLE Credential ADD Reusable bit NOT NULL DEFAULT(1)
END
GO


-- Stored Procedure

/*
 * Set the availability flag on all resources belonging to on a node
 */

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'SP_SetResourceAvailability'))
DROP PROCEDURE SP_SetResourceAvailability
GO

CREATE PROCEDURE SP_SetResourceAvailability(
    @nodeId   int,
    @now      datetime,
    @isAvailable bit
)
AS
BEGIN
    UPDATE Resource
    SET
       IsAvailable=@isAvailable, ModifyTime=@now
    WHERE
       NodeId=@nodeId AND Phantom = 0
END
GO


/* For Azure */
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_AddAzureDeployment]')) 
DROP PROCEDURE dbo.SP_AddAzureDeployment
GO

CREATE PROCEDURE SP_AddAzureDeployment(
	@DeploymentId nvarchar(64),
	@SubscriptionId nvarchar(128),
	@ServiceName nvarchar(128),
	@StorageConnectionString nvarchar(1024),
	@ProxyMultiplicity int
)
AS
BEGIN
	IF NOT EXISTS(SELECT DeploymentId FROM AzureDeployment WHERE DeploymentId = @DeploymentId)
		INSERT INTO AzureDeployment (DeploymentId, SubscriptionId, ServiceName, StorageConnectionString, ProxyMultiplicity) VALUES ( @DeploymentId, @SubscriptionId, @ServiceName, @StorageConnectionString, @ProxyMultiplicity)
	-- If exists, just ignore it. The upper level logic ensures that same deployment id will have the same configuration
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_RemoveAzureDeployment]')) 
DROP PROCEDURE dbo.SP_RemoveAzureDeployment
GO

CREATE PROCEDURE SP_RemoveAzureDeployment(
	@DeploymentId nvarchar(64)
)
AS
BEGIN
	DELETE FROM AzureDeployment WHERE DeploymentId = @DeploymentId
END
GO

/* 
** Remove Node SP 
**
** This procedure will remove a node from the system.
** As part of doing this step, it will update the 
** counter for the system as well.
*/

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_RemoveNode]')) 
DROP PROCEDURE dbo.SP_RemoveNode
GO
CREATE PROCEDURE SP_RemoveNode 
    @id int,
	@deletedDeployId nvarchar(64) out
AS
BEGIN
    SET NOCOUNT ON;

    UPDATE ResourceCounter
    SET
        ResourceCounter.Total                 = ResourceCounter.Total               - NodeResourceCounter.Total,
        ResourceCounter.Offline               = ResourceCounter.Offline             - NodeResourceCounter.Offline,
        ResourceCounter.Idle                  = ResourceCounter.Idle                - NodeResourceCounter.Idle,
        ResourceCounter.ScheduledReserve      = ResourceCounter.ScheduledReserve    - NodeResourceCounter.ScheduledReserve,
        ResourceCounter.JobScheduled          = ResourceCounter.JobScheduled        - NodeResourceCounter.JobScheduled,
        ResourceCounter.ReadyForTask          = ResourceCounter.ReadyForTask        - NodeResourceCounter.ReadyForTask,
        ResourceCounter.TaskScheduled         = ResourceCounter.TaskScheduled       - NodeResourceCounter.JobTaskScheduled,
        ResourceCounter.JobTaskScheduled      = ResourceCounter.JobTaskScheduled    - NodeResourceCounter.JobTaskDispatched,
        ResourceCounter.TaskDispatched        = ResourceCounter.TaskDispatched      - NodeResourceCounter.TaskDispatched,
        ResourceCounter.JobTaskDispatched     = ResourceCounter.JobTaskDispatched   - NodeResourceCounter.JobTaskDispatched,
        ResourceCounter.TaskRunning           = ResourceCounter.TaskRunning         - NodeResourceCounter.TaskRunning,
        ResourceCounter.CloseTask             = ResourceCounter.CloseTask           - NodeResourceCounter.CloseTask,
        ResourceCounter.CloseTaskDispatched   = ResourceCounter.CloseTaskDispatched - NodeResourceCounter.CloseTaskDispatched,
        ResourceCounter.TaskClosed            = ResourceCounter.TaskClosed          - NodeResourceCounter.TaskClosed,
        ResourceCounter.CloseJob              = ResourceCounter.CloseJob            - NodeResourceCounter.CloseJob
    FROM NodeResourceCounter WHERE NodeResourceCounter.NodeId = @id

    UPDATE NodeCounter
    SET
        Total = Total - 1,
        Offline = Offline - 1
    
    delete from ResourceReservation
    where ResourceId in ( select ID from Resource where NodeId=@id)

    delete from Resource
    where NodeId=@id

    delete from NodeResourceCounter
    where NodeId=@id
    
	SET @deletedDeployId = NULL;

	-- Get the deployment Id first
	DECLARE @azureDeployId nvarchar(64)
	SELECT @azureDeployId = AzureDeploymentId FROM Node WHERE Id = @id

	-- Delete the node
    delete from Node 
    where Id=@id        

	-- Delete the deployment if this is the only node left for that deployment
	-- If the deployment Id is null, then don't do this
	IF(@azureDeployId IS NOT NULL)
	BEGIN
 		IF (NOT EXISTS(SELECT 1 FROM Node WHERE AzureDeploymentId = @azureDeployId))
		BEGIN
			SET @deletedDeployId = @azureDeployId
		END
	END
END
GO


/* For Server Side caching of credentials */
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_LoadCredentials]')) 
DROP PROCEDURE dbo.SP_LoadCredentials
GO

CREATE PROCEDURE SP_LoadCredentials
AS
BEGIN
    SELECT Credential.Id, SID_User.Name, SID_Owner.Name, SID_User.SID, SID_Owner.SID, Credential.Reusable, Credential.Password
    FROM Credential
    LEFT OUTER JOIN SID AS SID_User ON Credential.RunAsId = SID_User.Id
    LEFT OUTER JOIN SID AS SID_Owner ON Credential.OwnerId = SID_Owner.Id
END
GO

/* Set user name and password */
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_SetUserPassword]'))
DROP PROCEDURE dbo.SP_SetUserPassword
GO

CREATE PROCEDURE SP_SetUserPassword
    @runasName nvarchar(80),
    @runasSid nvarchar(80),
    @ownerName nvarchar(80),
    @ownerSid nvarchar(80),
    @password varbinary(800),
    @reusable int
AS
BEGIN

    SET NOCOUNT OFF;

    DECLARE @runasId as int
    DECLARE @prevRunAsName as nvarchar(80)

    SELECT @runasId = Id, @prevRunAsName = Name
    FROM SID
    WHERE SID=@runasSid

    IF(@runasId is NULL)
    BEGIN
        INSERT INTO SID
            (SID, Name)
        VALUES
            (@runasSid, @runasName)

        SELECT @runasId = @@IDENTITY
    END
    ELSE
        IF(@prevRunAsName <> @runasName)
        BEGIN
            UPDATE SID 
            SET Name = @runasName
            WHERE Id = @runasId
        END
        

    DECLARE @ownerId as int
    DECLARE @prevOwnerName as nvarchar(80)

    SELECT @ownerId = Id, @prevOwnerName = Name
    FROM SID
    WHERE SID=@ownerSid

    IF(@ownerId is NULL)
    BEGIN
        INSERT INTO SID
            (SID, Name)
        VALUES
            (@ownerSid, @ownerName)

        SELECT @ownerId = @@IDENTITY
    END
    ELSE
        IF(@prevOwnerName <> @ownerName)
        BEGIN
            UPDATE SID
            SET Name = @ownerName
            WHERE Id = @ownerId
        END

    DECLARE @credentialId as int

    SELECT @credentialId = Id 
    FROM Credential
    WHERE RunAsId=@runasId AND OwnerId=@ownerId

    IF(@credentialId is NULL)
    BEGIN
       if(@reusable is null)
       BEGIN
        INSERT INTO dbo.Credential
            (RunAsId, OwnerId, Password)
        VALUES
            (@runasId, @ownerId, @password)
       END
       ELSE
       BEGIN
        INSERT INTO dbo.Credential
            (RunAsId, OwnerId, Password, Reusable)
        VALUES
            (@runasId, @ownerId, @password, @reusable)
       END
    END
    ELSE
    BEGIN
        if(@reusable is not null)
        BEGIN
            UPDATE dbo.Credential
            SET Password=@password, Reusable=@reusable
            WHERE Id = @credentialId
        END
        ELSE
        BEGIN
            UPDATE dbo.Credential 
            SET Password=@password
            WHERE Id = @credentialId
        END
    END

END
GO

/* Set the reusable flag on a particular credential */
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_SetCredentialReusableFlag]'))
DROP PROCEDURE dbo.SP_SetCredentialReusableFlag
GO

CREATE PROCEDURE SP_SetCredentialReusableFlag
    @credId int,
    @reusable int
AS
BEGIN
 SET NOCOUNT ON;

 UPDATE dbo.Credential 
 SET Reusable = @reusable
 where Id=@credId

END
GO


-- Remove the limits on string values arguments
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_AddJobTemplateItem]')) 
DROP PROCEDURE dbo.SP_AddJobTemplateItem
GO

CREATE PROCEDURE SP_AddJobTemplateItem
    @JobTemplateId          int,
    @PropId                 int,
    @PropType               int,
    @DefaultInt             int,
    @MaxInt                 int,
    @MinInt                 int,
    @PropName               nvarchar(50),
    @ReadOnly               int,
    @DefaultBool            int,
    @DefaultString          nvarchar(max),
    @StringValues           nvarchar(max),
    @DefaultEnum            int,
    @EnumValues             nvarchar(128),
    @BoolValues             nvarchar(64),
    @RequiredValues         nvarchar(max)
AS
BEGIN
    SET NOCOUNT ON;

    BEGIN
    -- Insert new record
        INSERT INTO dbo.JobTemplateItem
        (
            JobTemplateId,
            PropId,
            PropType,
            DefaultInt,
            MaxInt,
            MinInt,
            PropName,
            ReadOnly,
            DefaultBool,
            DefaultString,
            StringValues,
            DefaultEnum,
            EnumValues,
            BoolValues,
            RequiredValues
        )
        VALUES
        (
            @JobTemplateId,
            @PropId,
            @PropType,
            @DefaultInt,
            @MaxInt,
            @MinInt,
            @PropName,
            @ReadOnly,
            @DefaultBool,
            @DefaultString,
            @StringValues,
            @DefaultEnum,
            @EnumValues,
            @BoolValues,
            @RequiredValues
        )

    END
END
GO

EXEC sp_updateextendedproperty @name = N'dbVersion', @value = '3.1';
GO

SET ANSI_PADDING OFF
GO