SET ANSI_PADDING ON
GO

-- Modify Node Table
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Node]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Node]') AND name = N'DnsSuffix')
BEGIN
    ALTER TABLE Node ADD DnsSuffix nvarchar(256) NULL
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Node]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Node]') AND name = N'Affinity')
BEGIN
    ALTER TABLE Node ADD Affinity bit NULL;
END
GO

-- Modify Credential Table
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Credential]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
	AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Credential]') AND name = N'Certificate')
BEGIN
	ALTER TABLE Credential ADD  Certificate varbinary(max)  NULL
END
GO


/* Set user name and password */
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_SetUserPassword]'))
DROP PROCEDURE dbo.SP_SetUserPassword
GO

CREATE PROCEDURE SP_SetUserPassword
    @runasName nvarchar(80),
    @runasSid nvarchar(80),
    @ownerName nvarchar(80),
    @ownerSid nvarchar(80),
    @password varbinary(800),
    @reusable int
AS
BEGIN

    SET NOCOUNT OFF;

    DECLARE @runasId as int
    DECLARE @prevRunAsName as nvarchar(80)

    SELECT @runasId = Id, @prevRunAsName = Name
    FROM SID
    WHERE SID=@runasSid

    IF(@runasId is NULL)
    BEGIN
        INSERT INTO SID
            (SID, Name)
        VALUES
            (@runasSid, @runasName)

        SELECT @runasId = @@IDENTITY
    END
    ELSE
        IF(@prevRunAsName <> @runasName)
        BEGIN
            UPDATE SID 
            SET Name = @runasName
            WHERE Id = @runasId
        END
        

    DECLARE @ownerId as int
    DECLARE @prevOwnerName as nvarchar(80)

    SELECT @ownerId = Id, @prevOwnerName = Name
    FROM SID
    WHERE SID=@ownerSid

    IF(@ownerId is NULL)
    BEGIN
        INSERT INTO SID
            (SID, Name)
        VALUES
            (@ownerSid, @ownerName)

        SELECT @ownerId = @@IDENTITY
    END
    ELSE
        IF(@prevOwnerName <> @ownerName)
        BEGIN
            UPDATE SID
            SET Name = @ownerName
            WHERE Id = @ownerId
        END

    DECLARE @credentialId as int

    SELECT @credentialId = Id 
    FROM Credential
    WHERE RunAsId=@runasId AND OwnerId=@ownerId

    IF(@credentialId is NULL)
    BEGIN
       if(@reusable is null)
       BEGIN
        INSERT INTO dbo.Credential
            (RunAsId, OwnerId, Password)
        VALUES
            (@runasId, @ownerId, @password)
       END
       ELSE
       BEGIN
        INSERT INTO dbo.Credential
            (RunAsId, OwnerId, Password, Reusable)
        VALUES
            (@runasId, @ownerId, @password, @reusable)
       END
    END
    ELSE
    BEGIN
        if(@reusable is not null)
        BEGIN
            UPDATE dbo.Credential
            SET Password=@password, Reusable=@reusable, Certificate=NULL
            WHERE Id = @credentialId
        END
        ELSE
        BEGIN
            UPDATE dbo.Credential 
            SET Password=@password, Certificate=NULL
            WHERE Id = @credentialId
        END
    END

END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_SetUserPasswordCertificate]'))
DROP PROCEDURE dbo.SP_SetUserPasswordCertificate
GO


CREATE PROCEDURE SP_SetUserPasswordCertificate
    @runasName nvarchar(80),
    @runasSid nvarchar(80),
    @ownerName nvarchar(80),
    @ownerSid nvarchar(80),
    @password varbinary(800),
    @reusable int,
    @certificate varbinary(max)
AS
BEGIN

    SET NOCOUNT OFF;

    DECLARE @runasId as int
    DECLARE @prevRunAsName as nvarchar(80)

    SELECT @runasId = Id, @prevRunAsName = Name
    FROM SID
    WHERE SID=@runasSid

    IF(@runasId is NULL)
    BEGIN
        INSERT INTO SID
            (SID, Name)
        VALUES
            (@runasSid, @runasName)

        SELECT @runasId = @@IDENTITY
    END
    ELSE
        IF(@prevRunAsName <> @runasName)
        BEGIN
            UPDATE SID 
            SET Name = @runasName
            WHERE Id = @runasId
        END
        

    DECLARE @ownerId as int
    DECLARE @prevOwnerName as nvarchar(80)

    SELECT @ownerId = Id, @prevOwnerName = Name
    FROM SID
    WHERE SID=@ownerSid

    IF(@ownerId is NULL)
    BEGIN
        INSERT INTO SID
            (SID, Name)
        VALUES
            (@ownerSid, @ownerName)

        SELECT @ownerId = @@IDENTITY
    END
    ELSE
        IF(@prevOwnerName <> @ownerName)
        BEGIN
            UPDATE SID
            SET Name = @ownerName
            WHERE Id = @ownerId
        END

    DECLARE @credentialId as int

    SELECT @credentialId = Id 
    FROM Credential
    WHERE RunAsId=@runasId AND OwnerId=@ownerId

    IF(@credentialId is NULL)
    BEGIN
       if(@reusable is null)
       BEGIN
        INSERT INTO dbo.Credential
            (RunAsId, OwnerId, Password, Certificate)
        VALUES
            (@runasId, @ownerId, @password, @certificate)
       END
       ELSE
       BEGIN
        INSERT INTO dbo.Credential
            (RunAsId, OwnerId, Password, Reusable, Certificate)
        VALUES
            (@runasId, @ownerId, @password, @reusable, @certificate)
       END
    END
    ELSE
    BEGIN
        if(@reusable is not null)
        BEGIN
            UPDATE dbo.Credential
            SET Password=@password, Reusable=@reusable, Certificate=@certificate
            WHERE Id = @credentialId
        END
        ELSE
        BEGIN
            UPDATE dbo.Credential 
            SET Password=@password, Certificate=@certificate
            WHERE Id = @credentialId
        END
    END

END
GO


IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_LoadCredentials]')) 
DROP PROCEDURE dbo.SP_LoadCredentials
GO

CREATE PROCEDURE SP_LoadCredentials
AS
BEGIN
    SELECT Credential.Id, SID_User.Name, SID_Owner.Name, SID_User.SID, SID_Owner.SID, Credential.Reusable, Credential.Password, Credential.Certificate
    FROM Credential
    LEFT OUTER JOIN SID AS SID_User ON Credential.RunAsId = SID_User.Id
    LEFT OUTER JOIN SID AS SID_Owner ON Credential.OwnerId = SID_Owner.Id
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Pool]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP TABLE [dbo].[Pool]
GO

CREATE TABLE Pool
(
    Id                      int             IDENTITY(1,1) NOT NULL PRIMARY KEY CLUSTERED,
    Name                    nvarchar(64)    NOT NULL,
    Weight                  int             NOT NULL DEFAULT(0),
) ON [PRIMARY] 
GO

INSERT INTO Pool (Name, Weight) VALUES ('Default',100)
GO


IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Job]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
	AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Job]') AND name = N'PoolId')
BEGIN
	ALTER TABLE Job ADD  PoolId     int             NOT NULL DEFAULT(1)
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[JobHistory]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
	AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[JobHistory]') AND name = N'PoolId')
BEGIN
	ALTER TABLE JobHistory ADD  PoolId     int             NULL
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_SetAllJobsOfPoolToDefaultPool]')) 
DROP PROCEDURE dbo.SP_SetAllJobsOfPoolToDefaultPool
GO

CREATE PROCEDURE SP_SetAllJobsOfPoolToDefaultPool(
	@PoolId int
)
AS
BEGIN
	UPDATE Job SET PoolId =1 where PoolId=@PoolId and State >= 16
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_AddPool]')) 
DROP PROCEDURE dbo.SP_AddPool
GO

-- return -2 if the number of pool has reached the limit, -1 if a pool of this name already exists, pool id for a pool succesfully added
CREATE PROCEDURE SP_AddPool(
    @PoolName nvarchar(64),
    @PoolWeight int,
    @NumPoolLimit int
)
AS
BEGIN
IF ((SELECT COUNT(ID) from Pool WITH (UPDLOCK) ) >= @NumPoolLimit)
BEGIN 
    SELECT -2
END
ELSE
BEGIN
    IF NOT EXISTS (SELECT ID from Pool where Name=@PoolName)
    BEGIN
        INSERT INTO Pool(Name,Weight) VALUES (@PoolName,@PoolWeight)
        SELECT ID FROM Pool where Name = @PoolName
    END
    ELSE
    BEGIN
        SELECT -1
    END
END
END
GO



IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_AddJobHistoryRecord]')) 
DROP PROCEDURE dbo.SP_AddJobHistoryRecord
GO

CREATE PROCEDURE SP_AddJobHistoryRecord(
        @JobEvent           int,
        @EventTime          datetime,
        @JobId              int,
        @RequeueCount       int,
        @SubmitTime         datetime,
        @StartTime          datetime,
        @EndTime            datetime,
        @HasGrown           bit,
        @HasShrunk          bit,
        @projectId          int,
        @JobTemplateId          int,
        @ActorSid           nvarchar(80),
        @CancelRequest      int,
        @PoolId             int
        )
AS
BEGIN

    INSERT INTO JobHistory
        (Event, EventTime, JobId, RequeueId, JobTemplateId, ProjectId, SubmitTime, StartTime, EndTime, HasGrown, HasShrunk, ActorSid, CancelRequest, PoolId)
    VALUES
        (@JobEvent, @EventTime, @JobId, @RequeueCount, @JobTemplateId, @ProjectId, @SubmitTime, @StartTime, @EndTime, @HasGrown, @HasShrunk, @ActorSid, @CancelRequest, @PoolId)    

END
GO


IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_SetJobRequestCancelValue]')) 
DROP PROCEDURE dbo.SP_SetJobRequestCancelValue
GO

CREATE PROCEDURE SP_SetJobRequestCancelValue
    @jobId int,
    @request int
AS
BEGIN
    DECLARE @OldRequest as int
    DECLARE @ShouldUpdate as int

    IF (@request <> 0)
    BEGIN
        SELECT @OldRequest=Request FROM JobCancelRequest WHERE JobId = @jobId

        IF (@OldRequest is not NULL)
        BEGIN
            SET @ShouldUpdate = 1
            IF (@OldRequest = 7) BEGIN SET @ShouldUpdate=0 END
            IF (@OldRequest in (1,5,6) AND @Request in (0,2,3,4,8)) BEGIN SET @ShouldUpdate=0 END

            IF (@ShouldUpdate = 1)
            BEGIN
                UPDATE JobCancelRequest 
                SET Request = @request
                WHERE JobId = @jobId
            END
        END    
        ELSE
        BEGIN
            INSERT INTO JobCancelRequest (JobId, Request)
            VALUES (@jobId, @request)
        END
    END
END
GO


IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_SetTaskRequestCancelValue]')) 
DROP PROCEDURE dbo.SP_SetTaskRequestCancelValue
GO

CREATE PROCEDURE SP_SetTaskRequestCancelValue
    @taskId int,
    @request int
AS
BEGIN
    IF (@request <> 0)
    BEGIN
        DECLARE @OldRequest as int
        DECLARE @ShouldUpdate as int
        SELECT @OldRequest=Request FROM TaskCancelRequest WHERE TaskId = @taskId

        IF (@OldRequest is not NULL)
        BEGIN
            SET @ShouldUpdate = 1
            IF (@OldRequest = 7) BEGIN SET @ShouldUpdate=0 END
            IF (@OldRequest in (1,5,6) AND @Request in (0,2,3,4,8)) BEGIN SET @ShouldUpdate=0 END
        
            IF (@ShouldUpdate = 1)
            BEGIN
                UPDATE TaskCancelRequest 
                SET Request = @request
                WHERE TaskId = @taskId
            END
        END
        ELSE    
        BEGIN
            INSERT INTO TaskCancelRequest (TaskId, Request)
            VALUES (@taskId, @request)
        END
    END
   
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[ParametricTaskCounter]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[ParametricTaskCounter]') AND name = N'FailedNonPreemption')
BEGIN
    ALTER TABLE ParametricTaskCounter ADD FailedNonPreemption int NOT NULL DEFAULT(0);
END
GO




/*
** Create the SubmtJobTasks SP
**
** This procedure will submit all configuring tasks 
*/

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'SP_SubmitJobTasks'))
DROP PROCEDURE SP_SubmitJobTasks
GO


CREATE PROCEDURE SP_SubmitJobTasks(
    @jobId              int,
    @now                datetime
    )
AS
BEGIN

    SET NOCOUNT OFF

    -- Get a count of all the master records.  These
    -- are not counted as "real" tasks that will run
    DECLARE @masterCount as int

    SELECT @masterCount=COUNT(InstanceId) 
    FROM Task 
    WHERE ParentJobId=@jobId AND InstanceId = -1 AND state = 1

    -- Submit all the tasks
    DECLARE @allCount as int
    
    UPDATE Task SET
        State=2, PrevState=1, SubmitTime=@now, ModifyTime=@now
    WHERE
        ParentJobId=@jobId AND State=1 

    SET @allCount = @@ROWCOUNT

    -- Calculate the total size of all parametric sweep tasks
    DECLARE @paramTasksCount as int
    
    IF @masterCount <> 0
    BEGIN

        SELECT @paramTasksCount =
        SUM(dbo.F_NumUnexpandedTasks(TaskDetail.StartValue, TaskDetail.EndValue, TaskDetail.IncrementValue, TaskDetail.CurrentInstanceId))
        FROM Task
        INNER JOIN TaskDetail ON Task.RecordId = TaskDetail.RecordId
        WHERE ParentJobId=@jobId and InstanceId=-1 and state = 2
    END
    ELSE
    BEGIN
        SET @paramTasksCount=0
    END

    -- Now we need to update the task counts for the job
    -- taking into account the number of rows that were
    -- just changed minus the master records, and plus the size of parametric sweep tasks
    -- NOTE here we assume none of the parametric sweep tasks has already been expanded.
    
    DECLARE @count as int
    SET @count = @allCount - @masterCount + @paramTasksCount
    SET NOCOUNT ON

    -- Get the record IDs for later updating the ParametricTaskCounter table
    SELECT recordId INTO #tempRecordIds FROM Task WHERE ParentJobId = @jobId AND instanceId = -1
    
    ---- 
    ---- Ensure this ordering remains consistent with other updates to avoid deadlocks.
    ----
        
    -- Update the TaskGroup counters
    UPDATE TaskGroup SET
        Submitted = Submitted + Configuring, Configuring = 0
    WHERE JobId=@jobId
    
    -- Update the task counters for the Job        
    UPDATE JobTaskCounter SET
        Configuring = Configuring - @count, Submitted = Submitted + @count
    WHERE 
        JobId=@jobId

    -- Update the master counters for the store
    UPDATE TaskCounter SET
        Configuring = Configuring - @count, Submitted = Submitted + @count
        
    -- Update any parametric task counters
    UPDATE ParametricTaskCounter SET 
        Configuring = 0, Submitted = Submitted + Configuring
    WHERE recordId IN (SELECT recordId FROM #tempRecordIds)

    IF EXISTS( SELECT * FROM tempdb.dbo.sysobjects WHERE ID = OBJECT_ID(N'tempdb..#tempRecordIds'))
    BEGIN
        DROP TABLE #tempRecordIds
    END

    SET NOCOUNT ON
    
    
END
GO



/*
** DeleteJob
**
** This procedure is used to delete a single job from the scheduler store.
** Jobs may only be deleted if they are in the configuring or terminal states.
*/

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_DeleteJob]')) 
DROP PROCEDURE dbo.SP_DeleteJob
GO
CREATE PROCEDURE SP_DeleteJob
    @JobId int,
    @deleted bit out
AS
BEGIN
    SET NOCOUNT ON;

    -- Make sure it is safe to delete the job.
    SET @deleted = 0
    IF (NOT EXISTS(SELECT Id FROM Job WHERE Id=@JobId AND State IN (1, 128, 256, 512)))
    BEGIN    
        RETURN
    END

    -- Update the store counters, taking this job off the correct
    -- counter.

    UPDATE JobCounter SET
        Total           = Total - 1,
        Configuring     = Configuring   - dbo.F_Eq(Job.State, 1),
        Submitted       = Submitted     - dbo.F_Eq(Job.State, 2),
        Validating      = Validating    - dbo.F_Eq(Job.State, 4),
        Queued          = Queued        - dbo.F_Eq(Job.State, 16),
        Running         = Running       - dbo.F_Eq(Job.State, 32),
        Finishing       = Finishing     - dbo.F_Eq(Job.State, 64),
        Finished        = Finished      - dbo.F_Eq(Job.State, 128),
        Failed          = Failed        - dbo.F_Eq(Job.State, 256),
        Canceled        = Canceled      - dbo.F_Eq(Job.State, 512),
        Canceling       = Canceling     - dbo.F_Eq(Job.State, 1024)
    FROM Job WITH(updlock) WHERE Job.ID=@JobId

    -- Update the store task counters

    UPDATE TaskCounter SET
        TaskCounter.Total =         TaskCounter.Total       - JobTaskCounter.Total,
        TaskCounter.Configuring =   TaskCounter.Configuring - JobTaskCounter.Configuring,
        TaskCounter.Submitted =     TaskCounter.Submitted   - JobTaskCounter.Submitted,
        TaskCounter.Validating =    TaskCounter.Validating  - JobTaskCounter.Validating,
        TaskCounter.Queued =        TaskCounter.Queued      - JobTaskCounter.Queued,
        TaskCounter.Dispatching =   TaskCounter.Dispatching - JobTaskCounter.Dispatching,
        TaskCounter.Running =       TaskCounter.Running     - JobTaskCounter.Running,
        TaskCounter.Finishing =     TaskCounter.Finishing   - JobTaskCounter.Finishing,
        TaskCounter.Finished =      TaskCounter.Finished    - JobTaskCounter.Finished,
        TaskCounter.Failed =        TaskCounter.Failed      - JobTaskCounter.Failed,
        TaskCounter.Canceled =      TaskCounter.Canceled    - JobTaskCounter.Canceled,
        TaskCounter.Canceling =     TaskCounter.Canceling   - JobTaskCounter.Canceling
    FROM JobTaskCounter WITH(updlock) WHERE JobTaskCounter.JobId = @JobId

    -- create a temporary table of the taskids of all the tasks being deleted
    SELECT Id
    INTO #tid
    FROM Task WITH (UPDLOCK)
    WHERE ParentJobId=@JobId
    
    -- create a temporary table of the recordids of all the tasks being deleted
    SELECT DISTINCT(RecordId)
    INTO #r
    FROM Task WITH (UPDLOCK)
    WHERE Id IN (SELECT Id from #tid)
    
    DELETE FROM ParametricTaskCounter WHERE RecordId IN ( SELECT RecordId FROM #r)

    DELETE FROM TaskGroupDependency WHERE JobId=@JobId

    DELETE FROM AllocationHistory WHERE JobId=@JobId
    
    DELETE FROM NodeAllocationHistory WHERE JobId=@JobId
    
    DELETE FROM ExtendedJobProperty WHERE JobId=@JobId
    
    DELETE FROM ExtendedTaskProperty WHERE JobId=@JobId
    
    DELETE FROM JobEnvironmentVariable WHERE JobId=@JobId
    
    DELETE FROM TaskEnvironmentVariable WHERE JobId=@JobId
    
    DELETE FROM TaskCancelRequest WHERE TaskId IN (SELECT id FROM #tid)

    DELETE FROM JobCancelRequest WHERE JobId=@JobId

    DELETE FROM Task WHERE Id IN (SELECT Id from #tid)
    
    DELETE FROM TaskDetail WHERE RecordId IN (SELECT RecordId FROM #r)
        
    DELETE FROM TaskGroup WHERE JobId=@JobId                
    
    DELETE FROM JobMessage WHERE JobId=@JobId
    
    DELETE FROM JobTaskCounter WHERE JobId=@JobId
    
    DELETE FROM JobHistory WHERE JobId=@JobId

    DELETE FROM Job WHERE ID=@JobId

    DROP TABLE #r
    
    DROP TABLE #tid

    SET @deleted=1
END
GO


/*
** DeleteOldJobs
*/

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_DeleteOldJobs]')) 
DROP PROCEDURE dbo.SP_DeleteOldJobs
GO

CREATE PROCEDURE SP_DeleteOldJobs(
    @dayAgeLimit int,
    @batchSize int,
    @deletedCount int out
)
AS
BEGIN

    -- Delete the jobs
    -- Only delete jobs that are older than the limit and only
    -- jobs that are in a ended state (failed, finished, canceled)
    -- and have been completed at least two days.

    SET NOCOUNT OFF

    IF EXISTS( SELECT * FROM tempdb.dbo.sysobjects WHERE ID = OBJECT_ID(N'tempdb..#t'))
    BEGIN
        DROP TABLE #t
    END

    --
    -- Grab the next 100 rows to delete.
    --
    SELECT TOP(@batchSize) id
    INTO #t
    FROM Job WITH (UPDLOCK)
    WHERE Job.endtime < DATEADD(DAY, -@dayAgeLimit, GETUTCDATE()) AND Job.state IN ( 128, 256, 512 )

    SET @deletedCount = @@rowcount

    SET NOCOUNT ON

    --
    -- Break if there is no more work to do.
    --
    IF @deletedCount > 0 
    BEGIN

        --
        -- Delete all the rows within a transaction. this would be updated to delete all the tables
        --
        
        -- create a temporary table of the taskids of all the tasks being deleted
        SELECT Id
        INTO #tid
        FROM Task WITH (UPDLOCK)
        WHERE ParentJobId IN (SELECT id from #t)
        
        -- create a temporary table of the recordids of all the tasks being deleted
        SELECT DISTINCT(RecordId)
        INTO #r
        FROM Task WITH (UPDLOCK)
        WHERE Id IN (SELECT Id from #tid)
    
        DELETE FROM ParametricTaskCounter WHERE RecordId IN ( SELECT RecordId FROM #r)

        DELETE FROM TaskGroupDependency WHERE JobId IN ( SELECT id FROM #t )
        
        DELETE FROM NodeAllocationHistory WHERE JobId IN ( SELECT id FROM #t )

        DELETE FROM AllocationHistory WHERE JobId IN ( SELECT id FROM #t )                
        
        DELETE FROM ExtendedJobProperty WHERE JobId IN ( SELECT id FROM #t )
        
        DELETE FROM ExtendedTaskProperty WHERE JobId IN ( SELECT id FROM #t )
        
        DELETE FROM TaskEnvironmentVariable WHERE JobId IN ( SELECT id FROM #t )
        
        DELETE FROM JobEnvironmentVariable WHERE JobId IN ( SELECT id FROM #t )
        
        DELETE FROM TaskCancelRequest WHERE TaskId IN (SELECT id FROM #tid)

        DELETE FROM JobCancelRequest WHERE JobId IN (SELECT id FROM #t)
        
        DELETE FROM Task WHERE ParentJobId IN ( SELECT id FROM #t )
    
        DELETE FROM TaskDetail WHERE RecordId IN (SELECT RecordId FROM #r)
        
        DELETE FROM TaskGroup WHERE JobId IN ( SELECT id FROM #t )        
        
        DELETE FROM JobMessage WHERE JobId IN ( SELECT id FROM #t )
        
        DELETE FROM JobTaskCounter WHERE JobId IN ( SELECT id FROM #t )
        
        DELETE FROM JobHistory WHERE JobId IN ( SELECT id from #t )

        DELETE FROM Job WHERE ID IN ( SELECT id FROM #t )               

        DELETE FROM JobProjectName 
            WHERE Id NOT IN (SELECT ProjectId FROM Job GROUP BY ProjectId) 
                AND Name <> ('')
                
       
        DROP TABLE #r
        DROP TABLE #tid

    END -- if @@rowcount > 0

    -- Update the global counters for jobs and tasks
    exec SP_ResetJobCounter;
    exec SP_ResetTaskCounter;

    SELECT Id FROM #t

END
GO



EXEC sp_updateextendedproperty @name = N'dbVersion', @value = '3.2';
GO

SET ANSI_PADDING OFF
GO