SET ANSI_PADDING ON
GO

/* For Azure */
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_AddAzureDeployment]')) 
DROP PROCEDURE dbo.SP_AddAzureDeployment
GO

CREATE PROCEDURE SP_AddAzureDeployment(
	@DeploymentId nvarchar(64),
	@SubscriptionId nvarchar(128),
	@ServiceName nvarchar(128),
	@StorageConnectionString nvarchar(1024),
	@ProxyMultiplicity int
)
AS
BEGIN
	IF NOT EXISTS(SELECT DeploymentId FROM AzureDeployment WITH(UPDLOCK) WHERE DeploymentId = @DeploymentId)
		INSERT INTO AzureDeployment (DeploymentId, SubscriptionId, ServiceName, StorageConnectionString, ProxyMultiplicity) VALUES ( @DeploymentId, @SubscriptionId, @ServiceName, @StorageConnectionString, @ProxyMultiplicity)
	-- If exists, just ignore it. The upper level logic ensures that same deployment id will have the same configuration
END
GO

-- Modify Job Table
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Job]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Job]') AND name = N'RuntimeType')
BEGIN
    ALTER TABLE Job ADD RuntimeType int	NOT Null DEFAULT(0)
END
GO


-- Modify SP_CreateResource
IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'SP_CreateResource'))
DROP PROCEDURE SP_CreateResource
GO

CREATE PROCEDURE SP_CreateResource(
    @JobType            int,
    @NodeId             int,
    @State              int,
    @ModifyTime         datetime,
    @CoreID             int,
    @SocketID           bigint,
    @Reachable          int
    )
AS
BEGIN

    DECLARE @newId int
    
    INSERT INTO Resource
        (JobType, NodeId, State, ModifyTime, CoreID, SocketID, PrevState, JobId, TaskId, Reachable)
    VALUES
        (@JobType, @NodeId, @State, @ModifyTime, @CoreID, @SocketID, @State, NULL, NULL, @Reachable)
        
    SELECT @newId = @@IDENTITY
    
    INSERT INTO ResourceReservation (ResourceId) VALUES (@newId)
    
END
GO



-- Please Keep this at last
-- Increase DbVersion
EXEC sp_updateextendedproperty @name = N'dbVersion', @value = '3.3';
GO

SET ANSI_PADDING OFF
GO