/*This file should contain all the upgrade statements from v3sp3 to v4 */

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Job]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Job]') AND name = N'NodeGroupOp')
BEGIN
    ALTER TABLE Job ADD NodeGroupOp         int             NULL
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Job]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Job]') AND name = N'SelectedComputeNodeListId')
BEGIN
    ALTER TABLE Job ADD SelectedComputeNodeListId int		NOT NULL DEFAULT(-1)
END
GO


IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Job]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Job]') AND name = N'SingleNode')
BEGIN
    ALTER TABLE Job ADD SingleNode bit	NULL
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Job]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Job]') AND name = N'JobValidExitCodes')
BEGIN
    ALTER TABLE Job ADD JobValidExitCodes nvarchar(256)         NULL
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Job]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Job]') AND name = N'HasUnfinishedParentJobs')
BEGIN
    ALTER TABLE Job ADD HasUnfinishedParentJobs  bit  NOT NULL DEFAULT(0)
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Job]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Job]') AND name = N'FailDependentTasks')
BEGIN
    ALTER TABLE Job ADD FailDependentTasks  bit  NOT NULL DEFAULT(0)
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[TaskDetail]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[TaskDetail]') AND name = N'TaskValidExitCodes')
BEGIN
    ALTER TABLE TaskDetail ADD TaskValidExitCodes nvarchar(256)    NULL
END
GO


IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Job]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
    ALTER TABLE Job ALTER COLUMN Name  nvarchar(128)   NULL
END
GO

/* For Azure */
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_AddAzureDeployment]')) 
DROP PROCEDURE dbo.SP_AddAzureDeployment
GO

CREATE PROCEDURE SP_AddAzureDeployment(
	@DeploymentId nvarchar(64),
	@SubscriptionId nvarchar(128),
	@ServiceName nvarchar(128),
	@StorageConnectionString nvarchar(1024),
	@ProxyMultiplicity int,
	@AlreadyExists int out
)
AS
BEGIN
    SET @AlreadyExists = 0
	IF NOT EXISTS(SELECT DeploymentId FROM AzureDeployment WITH(UPDLOCK) WHERE DeploymentId = @DeploymentId)
		INSERT INTO AzureDeployment (DeploymentId, SubscriptionId, ServiceName, StorageConnectionString, ProxyMultiplicity) VALUES ( @DeploymentId, @SubscriptionId, @ServiceName, @StorageConnectionString, @ProxyMultiplicity)
    ELSE
        -- If exists, we should return that as an output parameter
        SET @AlreadyExists = 1
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_RemoveAzureDeployment]')) 
DROP PROCEDURE dbo.SP_RemoveAzureDeployment
GO

CREATE PROCEDURE SP_RemoveAzureDeployment(
	@DeploymentId nvarchar(64),
	@DoesnotExist int out
)
AS
BEGIN
    SET @DoesnotExist = 0
    IF NOT EXISTS(SELECT DeploymentId FROM AzureDeployment WITH(UPDLOCK) WHERE DeploymentId = @DeploymentId)
       SET @DoesnotExist = 1
	ELSE
	BEGIN
	   UPDATE Node set AzureDeploymentId = null where AzureDeploymentId = @DeploymentId
	   DELETE FROM AzureDeployment WHERE DeploymentId = @DeploymentId	   
    END
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Node]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Node]') AND name = N'AzureNodeInstanceName')
BEGIN
    ALTER TABLE Node ADD AzureNodeInstanceName   nvarchar(256)  NULL
END
GO


IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[JobDependency]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
   CREATE TABLE JobDependency
   (
    JobDependencyId     int             IDENTITY(1,1) NOT NULL PRIMARY KEY CLUSTERED, 
    ParentJobId         int             NOT NULL,
    ChildJobId          int             NOT NULL,
   )
END
GO

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[JobDependency]') AND name = N'IDX_JobDependency_ParentJobIds')
    CREATE INDEX IDX_JobDependency_ParentJobIds ON JobDependency (ParentJobId)
GO

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[JobDependency]') AND name = N'IDX_JobDependency_ChildJobIds')
    CREATE INDEX IDX_JobDependency_ChildJobIds ON JobDependency (ChildJobId)
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_DeleteJobDependencies]')) 
DROP PROCEDURE dbo.SP_DeleteJobDependencies
GO

CREATE PROCEDURE SP_DeleteJobDependencies(
    @ChildJobId int
)
AS
BEGIN
    DELETE FROM JobDependency WHERE ChildJobId = @ChildJobId
END
GO


IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_GetParentJobIds]')) 
DROP PROCEDURE dbo.SP_GetParentJobIds
GO

CREATE PROCEDURE SP_GetParentJobIds(
    @ChildJobId int
)
AS
BEGIN
    SELECT ParentJobId FROM JobDependency WHERE ChildJobId = @ChildJobId ORDER BY ParentJobId ASC
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_GetChildJobIds]')) 
DROP PROCEDURE dbo.SP_GetChildJobIds
GO
CREATE PROCEDURE SP_GetChildJobIds(
    @ParentJobId int
)
AS
BEGIN
    SELECT ChildJobId FROM JobDependency WHERE ParentJobId = @ParentJobId ORDER BY ChildJobId ASC
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Job]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Job]') AND name = N'EstimatedProcessMemory')
BEGIN
    ALTER TABLE Job ADD EstimatedProcessMemory    int          NULL

END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Job]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Job]') AND name = N'HasParentJobs')
BEGIN
    ALTER TABLE Job ADD HasParentJobs       bit             NOT NULL DEFAULT(0)
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Resource]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Resource]') AND name = N'MemoryUsed')
BEGIN
    ALTER TABLE Resource ADD MemoryUsed              int          NOT NULL DEFAULT(0)
END
GO


IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[JobHistory]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
	AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[JobHistory]') AND name = N'ParentJobIds')
BEGIN	
	ALTER TABLE JobHistory ADD ParentJobIds		nvarchar(4000)	NULL
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[JobHistory]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
	AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[JobHistory]') AND name = N'ChildJobIds')
BEGIN	
	ALTER TABLE JobHistory ADD ChildJobIds		nvarchar(4000)	NULL
END
GO
	
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_AddJobHistoryRecord]')) 
DROP PROCEDURE dbo.SP_AddJobHistoryRecord
GO

CREATE PROCEDURE SP_AddJobHistoryRecord(
        @JobEvent           int,
        @EventTime          datetime,
        @JobId              int,
        @RequeueCount       int,
        @SubmitTime         datetime,
        @StartTime          datetime,
        @EndTime            datetime,
        @HasGrown           bit,
        @HasShrunk          bit,
        @projectId          int,
        @JobTemplateId          int,
        @ActorSid           nvarchar(80),
        @CancelRequest      int,
        @PoolId             int,
		@ParentJobIds		nvarchar(4000),
		@ChildJobIds		nvarchar(4000)
        )
AS
BEGIN

    INSERT INTO JobHistory
        (Event, EventTime, JobId, RequeueId, JobTemplateId, ProjectId, SubmitTime, StartTime, EndTime, HasGrown, HasShrunk, ActorSid, CancelRequest, PoolId, ParentJobIds, ChildJobIds)
    VALUES
        (@JobEvent, @EventTime, @JobId, @RequeueCount, @JobTemplateId, @ProjectId, @SubmitTime, @StartTime, @EndTime, @HasGrown, @HasShrunk, @ActorSid, @CancelRequest, @PoolId, @ParentJobIds, @ChildJobIds)    

END
GO


IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[AzureDeployment]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
	AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[AzureDeployment]') AND name = N'EncryptedStorageConnectionString')
BEGIN	
	ALTER TABLE AzureDeployment DROP COLUMN StorageConnectionString
	ALTER TABLE AzureDeployment ADD EncryptedStorageConnectionString varbinary(max) NULL
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_AddAzureDeployment]')) 
DROP PROCEDURE dbo.SP_AddAzureDeployment
GO

CREATE PROCEDURE SP_AddAzureDeployment(
	@DeploymentId nvarchar(64),
	@SubscriptionId nvarchar(128),
	@ServiceName nvarchar(128),
	@EncryptedStorageConnectionString varbinary(max),
	@ProxyMultiplicity int,
	@AlreadyExists int out
)
AS
BEGIN
    SET @AlreadyExists = 0
	IF NOT EXISTS(SELECT DeploymentId FROM AzureDeployment WITH(UPDLOCK) WHERE DeploymentId = @DeploymentId)
		INSERT INTO AzureDeployment (DeploymentId, SubscriptionId, ServiceName, EncryptedStorageConnectionString, ProxyMultiplicity) VALUES ( @DeploymentId, @SubscriptionId, @ServiceName, @EncryptedStorageConnectionString, @ProxyMultiplicity)
    ELSE
        -- If exists, we should return that as an output parameter
        SET @AlreadyExists = 1
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_ResetResourceCounter]')) 
DROP PROCEDURE dbo.SP_ResetResourceCounter
GO

CREATE PROCEDURE SP_ResetResourceCounter
AS
BEGIN
    SET NOCOUNT ON;

    -- Update the store resource counters

    UPDATE ResourceCounter SET 
        Offline                 = _Offline,
        Idle                    = _Idle,
        ScheduledReserve        = _ScheduledReserve,
        JobScheduled            = _JobScheduled,
        ReadyForTask            = _ReadyForTask,
        TaskScheduled           = _TaskScheduled,
        JobTaskScheduled        = _JobTaskScheduled,
        TaskDispatched          = _TaskDispatched,
        JobTaskDispatched       = _JobTaskDispatched,
        TaskRunning             = _TaskRunning,
        CloseTask               = _CloseTask,
        CloseTaskDispatched     = _CloseTaskDispatched,
        TaskClosed              = _TaskClosed,
        CloseJob                = _CloseJob,
        Total                   = _offline + _idle + _ScheduledReserve + _JobScheduled + 
                                  _ReadyForTask + _TaskScheduled + _JobTaskScheduled + 
                                  _TaskDispatched + _JobTaskDispatched + _TaskRunning +
                                  _CloseTask + _CloseTaskDispatched + _TaskClosed + _CloseJob
    FROM
    (
        SELECT 
            [0]     as _Offline,
            [1]     as _Idle,
            [2]     as _ScheduledReserve,
            [4]     as _JobScheduled,
            [8]     as _ReadyForTask,
            [16]    as _TaskScheduled,
            [32]    as _JobTaskScheduled,
            [64]    as _TaskDispatched,
            [128]   as _JobTaskDispatched,
            [256]   as _TaskRunning,
            [512]   as _CloseTask,
            [1024]  as _CloseTaskDispatched,
            [2048]  as _TaskClosed,
            [4096]  as _CloseJob
        FROM (SELECT id,state FROM Resource WITH (UPDLOCK, TABLOCK) where Resource.Phantom = 0) as TEMP
        PIVOT (COUNT(id) FOR state in ([0],[1],[2],[4],[8],[16],[32],[64],[128],[256],[512],[1024],[2048],[4096])) as pvt
    ) as a

END
GO





-- Please Keep this at last
-- Increase DbVersion
EXEC sp_updateextendedproperty @name = N'dbVersion', @value = '4.0';
GO

SET ANSI_PADDING OFF
GO

