/*This file should contain all the upgrade statements from 4.3 to 4.4 */

Alter PROCEDURE SP_RemoveTaskGroup
    @jobId int,
    @groupId int
AS
BEGIN

    SET NOCOUNT ON;
    
    DECLARE @defaultGroup int
    
    SELECT @defaultGroup = ID
    FROM TaskGroup
    WHERE JobId=@jobId AND IsDefault=1

    UPDATE TaskDetail SET GroupId=@defaultGroup WHERE GroupId=@groupId
    
    DELETE FROM TaskGroupDependency WHERE ParentId=@groupId
    DELETE FROM TaskGroupDependency WHERE ChildId=@groupId
	DELETE FROM TaskGroup where ID=@groupId

END
GO

/* extended data column */
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Credential]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Credential]') AND name = N'ExtendedData')
BEGIN
    ALTER TABLE Credential ADD ExtendedData NVARCHAR(MAX) NULL
END
GO

/* Set user extended data */
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_SetUserExtendedData]'))
DROP PROCEDURE dbo.SP_SetUserExtendedData
GO

CREATE PROCEDURE SP_SetUserExtendedData
    @runasName nvarchar(80),
    @runasSid nvarchar(80),
    @ownerName nvarchar(80),
    @ownerSid nvarchar(80),
    @extendedData nvarchar(max)
AS
BEGIN

    SET NOCOUNT OFF;

    DECLARE @runasId as int
    DECLARE @prevRunAsName as nvarchar(80)

    SELECT @runasId = Id, @prevRunAsName = Name
    FROM SID
    WHERE SID=@runasSid

    IF(@runasId is NULL)
    BEGIN
        INSERT INTO SID
            (SID, Name)
        VALUES
            (@runasSid, @runasName)

        SELECT @runasId = @@IDENTITY
    END
    ELSE
        IF(@prevRunAsName <> @runasName)
        BEGIN
            UPDATE SID 
            SET Name = @runasName
            WHERE Id = @runasId
        END
        

    DECLARE @ownerId as int
    DECLARE @prevOwnerName as nvarchar(80)

    SELECT @ownerId = Id, @prevOwnerName = Name
    FROM SID
    WHERE SID=@ownerSid

    IF(@ownerId is NULL)
    BEGIN
        INSERT INTO SID
            (SID, Name)
        VALUES
            (@ownerSid, @ownerName)

        SELECT @ownerId = @@IDENTITY
    END
    ELSE
        IF(@prevOwnerName <> @ownerName)
        BEGIN
            UPDATE SID
            SET Name = @ownerName
            WHERE Id = @ownerId
        END

    DECLARE @credentialId as int

    SELECT @credentialId = Id 
    FROM Credential
    WHERE RunAsId=@runasId AND OwnerId=@ownerId

    IF(@credentialId is NULL)
    BEGIN
        INSERT INTO dbo.Credential
            (RunAsId, OwnerId, ExtendedData)
        VALUES
            (@runasId, @ownerId, @extendedData)
    END
    ELSE
    BEGIN
        UPDATE dbo.Credential 
        SET ExtendedData=@extendedData
        WHERE Id = @credentialId
    END
END
GO


EXEC sp_updateextendedproperty @name = N'dbVersion', @value = '4.4';