/*This file should contain all the upgrade statements from v4sp4 to v4sp5 */

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_DeleteTask]')) 
DROP PROCEDURE dbo.SP_DeleteTask
GO
CREATE PROCEDURE SP_DeleteTask
    @TaskId int,
    @deleted bit out
AS
BEGIN
    SET NOCOUNT ON;

    DECLARE @recordId AS INT
    DECLARE @jobId AS INT
    DECLARE @state AS INT
    DECLARE @instanceId AS INT
   
    SET @deleted = 0
   
    -- Obtain record id, and make sure that it is safe to delete the task
    SELECT @recordId=RecordId, @jobId=ParentJobId, @state=State, @instanceId=InstanceId
    FROM Task WITH(UPDLOCK)
    WHERE ID=@TaskId AND State IN (1, 128, 256, 512) AND InstanceId<=0
    IF (@recordId IS NULL)
    BEGIN
        RETURN
    END
                
    -- Update the task counters    
    UPDATE TaskCounter SET
        TaskCounter.Total        = TaskCounter.Total         - case when @instanceId<0 then ParametricTaskCounter.Total        else 1                     end,
        TaskCounter.Configuring  = TaskCounter.Configuring   - case when @instanceId<0 then ParametricTaskCounter.Configuring  else dbo.F_Eq(@state, 1)    end,
        TaskCounter.Submitted    = TaskCounter.Submitted     - case when @instanceId<0 then ParametricTaskCounter.Submitted    else dbo.F_Eq(@state, 2)    end,
        TaskCounter.Validating   = TaskCounter.Validating    - case when @instanceId<0 then ParametricTaskCounter.Validating   else dbo.F_Eq(@state, 4)    end,
        TaskCounter.Queued       = TaskCounter.Queued        - case when @instanceId<0 then ParametricTaskCounter.Queued       else dbo.F_Eq(@state, 8)    end,
        TaskCounter.Dispatching  = TaskCounter.Dispatching   - case when @instanceId<0 then ParametricTaskCounter.Dispatching  else dbo.F_Eq(@state, 16)   end,
        TaskCounter.Running      = TaskCounter.Running       - case when @instanceId<0 then ParametricTaskCounter.Running      else dbo.F_Eq(@state, 32)   end,
        TaskCounter.Finishing    = TaskCounter.Finishing     - case when @instanceId<0 then ParametricTaskCounter.Finishing    else dbo.F_Eq(@state, 64)   end,
        TaskCounter.Finished     = TaskCounter.Finished      - case when @instanceId<0 then ParametricTaskCounter.Finished     else dbo.F_Eq(@state, 128)  end,
        TaskCounter.Failed       = TaskCounter.Failed        - case when @instanceId<0 then ParametricTaskCounter.Failed       else dbo.F_Eq(@state, 256)  end,
        TaskCounter.Canceled     = TaskCounter.Canceled      - case when @instanceId<0 then ParametricTaskCounter.Canceled     else dbo.F_Eq(@state, 512)  end,
        TaskCounter.Canceling    = TaskCounter.Canceling     - case when @instanceId<0 then ParametricTaskCounter.Canceling    else dbo.F_Eq(@state, 1024) end
    FROM ParametricTaskCounter WITH(updlock) WHERE RecordId=@recordId

    UPDATE JobTaskCounter SET
        JobTaskCounter.Total        = JobTaskCounter.Total         - case when @instanceId<0 then ParametricTaskCounter.Total        else 1                     end,
        JobTaskCounter.Configuring  = JobTaskCounter.Configuring   - case when @instanceId<0 then ParametricTaskCounter.Configuring  else dbo.F_Eq(@state, 1)    end,
        JobTaskCounter.Submitted    = JobTaskCounter.Submitted     - case when @instanceId<0 then ParametricTaskCounter.Submitted    else dbo.F_Eq(@state, 2)    end,
        JobTaskCounter.Validating   = JobTaskCounter.Validating    - case when @instanceId<0 then ParametricTaskCounter.Validating   else dbo.F_Eq(@state, 4)    end,
        JobTaskCounter.Queued       = JobTaskCounter.Queued        - case when @instanceId<0 then ParametricTaskCounter.Queued       else dbo.F_Eq(@state, 8)    end,
        JobTaskCounter.Dispatching  = JobTaskCounter.Dispatching   - case when @instanceId<0 then ParametricTaskCounter.Dispatching  else dbo.F_Eq(@state, 16)   end,
        JobTaskCounter.Running      = JobTaskCounter.Running       - case when @instanceId<0 then ParametricTaskCounter.Running      else dbo.F_Eq(@state, 32)   end,
        JobTaskCounter.Finishing    = JobTaskCounter.Finishing     - case when @instanceId<0 then ParametricTaskCounter.Finishing    else dbo.F_Eq(@state, 64)   end,
        JobTaskCounter.Finished     = JobTaskCounter.Finished      - case when @instanceId<0 then ParametricTaskCounter.Finished     else dbo.F_Eq(@state, 128)  end,
        JobTaskCounter.Failed       = JobTaskCounter.Failed        - case when @instanceId<0 then ParametricTaskCounter.Failed       else dbo.F_Eq(@state, 256)  end,
        JobTaskCounter.Canceled     = JobTaskCounter.Canceled      - case when @instanceId<0 then ParametricTaskCounter.Canceled     else dbo.F_Eq(@state, 512)  end,
        JobTaskCounter.Canceling    = JobTaskCounter.Canceling     - case when @instanceId<0 then ParametricTaskCounter.Canceling    else dbo.F_Eq(@state, 1024) end
    FROM JobTaskCounter
    CROSS JOIN ParametricTaskCounter
    WHERE JobTaskCounter.JobId=@jobId AND ParametricTaskCounter.RecordId=@recordId

    UPDATE TaskGroup SET
        TaskGroup.Total        = TaskGroup.Total         - case when @instanceId<0 then ParametricTaskCounter.Total        else 1                     end,
        TaskGroup.Configuring  = TaskGroup.Configuring   - case when @instanceId<0 then ParametricTaskCounter.Configuring  else dbo.F_Eq(@state, 1)    end,
        TaskGroup.Submitted    = TaskGroup.Submitted     - case when @instanceId<0 then ParametricTaskCounter.Submitted    else dbo.F_Eq(@state, 2)    end,
        TaskGroup.Validating   = TaskGroup.Validating    - case when @instanceId<0 then ParametricTaskCounter.Validating   else dbo.F_Eq(@state, 4)    end,
        TaskGroup.Queued       = TaskGroup.Queued        - case when @instanceId<0 then ParametricTaskCounter.Queued       else dbo.F_Eq(@state, 8)    end,
        TaskGroup.Dispatching  = TaskGroup.Dispatching   - case when @instanceId<0 then ParametricTaskCounter.Dispatching  else dbo.F_Eq(@state, 16)   end,
        TaskGroup.Running      = TaskGroup.Running       - case when @instanceId<0 then ParametricTaskCounter.Running      else dbo.F_Eq(@state, 32)   end,
        TaskGroup.Finishing    = TaskGroup.Finishing     - case when @instanceId<0 then ParametricTaskCounter.Finishing    else dbo.F_Eq(@state, 64)   end,
        TaskGroup.Finished     = TaskGroup.Finished      - case when @instanceId<0 then ParametricTaskCounter.Finished     else dbo.F_Eq(@state, 128)  end,
        TaskGroup.Failed       = TaskGroup.Failed        - case when @instanceId<0 then ParametricTaskCounter.Failed       else dbo.F_Eq(@state, 256)  end,
        TaskGroup.Canceled     = TaskGroup.Canceled      - case when @instanceId<0 then ParametricTaskCounter.Canceled     else dbo.F_Eq(@state, 512)  end,
        TaskGroup.Canceling    = TaskGroup.Canceling     - case when @instanceId<0 then ParametricTaskCounter.Canceling    else dbo.F_Eq(@state, 1024) end
    FROM TaskGroup
    INNER JOIN TaskDetail
    ON TaskGroup.Id=TaskDetail.GroupId AND TaskDetail.RecordId=@recordId
    INNER JOIN ParametricTaskCounter
    ON ParametricTaskCounter.RecordId=@recordId

    -- Create a temporary table of IDs of all the sub-tasks in the current task
    SELECT Id
    INTO #r
    FROM Task WITH (UPDLOCK)
    WHERE RecordId=@recordId

    -- Delete the task and sub-tasks from all tables

    DELETE FROM ParametricTaskCounter WHERE RecordId=@recordId

    DELETE FROM ExtendedTaskProperty WHERE TaskId IN (SELECT Id FROM #r)        
    
    DELETE FROM TaskEnvironmentVariable WHERE TaskId IN (SELECT Id FROM #r)
    
    DELETE FROM AllocationHistory WHERE TaskId IN (SELECT Id FROM #r)
    
    DELETE FROM NodeAllocationHistory WHERE TaskId IN (SELECT Id FROM #r)
    
    DELETE FROM Task WHERE RecordId=@recordId
    
    DELETE FROM TaskDetail WHERE RecordId=@recordId        

    DROP TABLE #r

    SET @deleted = 1
END
GO

/* NumGpus column */
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[Node]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[Node]') AND name = N'NumGpus')
BEGIN
    ALTER TABLE Node ADD NumGpus INT        NOT NULL DEFAULT(0)
END
GO

EXEC sp_updateextendedproperty @name = N'dbVersion', @value = '4.5';