/*This file should contain all the upgrade statements from v4sp5 qfe4 to qfe5 */

USE [HPCScheduler]
GO

/* Skip whole script if Sql Server version is less than 11 */
DECLARE @sqlServerVer int	
SELECT @sqlServerVer = PARSENAME(CONVERT(VARCHAR(32), CAST(SERVERPROPERTY('ProductVersion') AS NVARCHAR(128))), 4)
IF @sqlServerVer < 11
SET NOEXEC ON

IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[AllocationHistoryId]') AND type = 'SO')
BEGIN
	DECLARE @start int
	IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[AllocationHistory]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    BEGIN
        SELECT @start = max(id) from dbo.AllocationHistory
        IF @start is null
        SET @start = 0
    End
	ELSE
	SET @start = 0

	DECLARE @sql NVARCHAR(MAX)
	SET @sql = 
	'CREATE SEQUENCE [dbo].[AllocationHistoryId]
		AS int 
		START WITH ' + CONVERT(VARCHAR(10), @start + 1) +'
        INCREMENT BY 1
        CYCLE'    
	EXEC(@sql)
END
GO

/* Delete all constraints on dbo.AllocationHistory */
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[AllocationHistory]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	BEGIN TRANSACTION
	DECLARE @SQL NVARCHAR(MAX) = N'';
	SELECT @SQL += N'
	ALTER TABLE ' + OBJECT_NAME(PARENT_OBJECT_ID) + ' DROP CONSTRAINT ' + OBJECT_NAME(OBJECT_ID) + ';' 
	FROM SYS.OBJECTS
	WHERE TYPE_DESC LIKE '%CONSTRAINT' AND OBJECT_NAME(PARENT_OBJECT_ID) = 'AllocationHistory';
	EXECUTE(@SQL)
	COMMIT
END
GO

BEGIN TRANSACTION
CREATE TABLE dbo.Tmp_AllocationHistory
	(
	Id int NOT NULL,
	ResourceId int NOT NULL,
	CoreId int NOT NULL,
	SocketId bigint NOT NULL,
	NodeId int NOT NULL,
	JobId int NOT NULL,
	TaskId int NOT NULL,
	JobRequeueCount int NOT NULL,
	TaskRequeueCount int NOT NULL,
	StartTime datetime NULL,
	EndTime datetime NULL,
	ModifyTime datetime NULL
	)
GO

ALTER TABLE dbo.Tmp_AllocationHistory SET (LOCK_ESCALATION = TABLE)
ALTER TABLE dbo.Tmp_AllocationHistory ADD CONSTRAINT DF_AllocationHistory_Id DEFAULT (NEXT VALUE FOR [dbo].[AllocationHistoryId]) FOR Id
ALTER TABLE dbo.Tmp_AllocationHistory ADD CONSTRAINT DF__Allocatio__TaskId DEFAULT ((0)) FOR TaskId
ALTER TABLE dbo.Tmp_AllocationHistory ADD CONSTRAINT DF__Allocatio__JobRequeueCount DEFAULT ((0)) FOR JobRequeueCount
ALTER TABLE dbo.Tmp_AllocationHistory ADD CONSTRAINT DF__Allocatio__TaskRequeueCount DEFAULT ((0)) FOR TaskRequeueCount
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[AllocationHistory]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	IF EXISTS(SELECT TOP 1 id FROM dbo.AllocationHistory)
	EXEC('INSERT INTO dbo.Tmp_AllocationHistory (Id, ResourceId, CoreId, SocketId, NodeId, JobId, TaskId, JobRequeueCount, TaskRequeueCount, StartTime, EndTime, ModifyTime)
	SELECT Id, ResourceId, CoreId, SocketId, NodeId, JobId, TaskId, JobRequeueCount, TaskRequeueCount, StartTime, EndTime, ModifyTime FROM dbo.AllocationHistory WITH (HOLDLOCK TABLOCKX)')
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[AllocationHistory]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP TABLE dbo.AllocationHistory
GO

EXECUTE sp_rename N'dbo.Tmp_AllocationHistory', N'AllocationHistory', 'OBJECT' 
GO

ALTER TABLE [dbo].[AllocationHistory] ADD PRIMARY KEY CLUSTERED ([Id])
CREATE NONCLUSTERED INDEX IDX_AllocationHistory_TaskIdTaskRequeueCount ON dbo.AllocationHistory (TaskId, TaskRequeueCount) 
CREATE NONCLUSTERED INDEX IDX_AllocationHistory_JobIdTaskIdJobRequeueCount ON dbo.AllocationHistory (JobId, TaskId, JobRequeueCount) INCLUDE (ResourceId) 
CREATE NONCLUSTERED INDEX IDX_AllocationHistory_Stage ON dbo.AllocationHistory (ModifyTime, TaskId) INCLUDE (Id)  
ALTER TABLE dbo.AllocationHistory ADD CONSTRAINT CK_AllocationHistory_FK_JobId FOREIGN KEY (JobId) REFERENCES dbo.Job (ID)

COMMIT
GO

SET NOEXEC OFF