/*This file should contain all the upgrade statements from v5rtm to update1 */

update ClusterParameter set StringValue = '1000' where ParamName = 'EXCLUDEDNODESLIMIT'
GO

/*
** data types
*/
if type_id('int_list') is null
  create type int_list as table (id int not null primary key)

if type_id('int3_list') is null
  create type int3_list as table (v1 int not null, v2 int not null, v3 int not null, primary key (v1, v2, v3))
GO

/*
** UpdateTaskGroupsMaxMinCores
**
** This procedure is used to update the MaxMinCores
** for TaskGroups to a new value.
*/
IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'SP_UpdateTaskGroupsMaxMinCores'))
DROP PROCEDURE SP_UpdateTaskGroupsMaxMinCores
GO

CREATE PROCEDURE SP_UpdateTaskGroupsMaxMinCores(
    @parentJobId        int,
    @taskgroupIds       int_list readonly
    )
AS
BEGIN
    UPDATE tg SET 
        tg.MaxMinCores= 
            (              
                    SELECT COALESCE( MAX(TaskDetail.MinCores),0)
                     -- We can put it here because this query is executed every 30 seconds
                     -- And if we miss the task state change once we can get it the next time anyway
                    FROM Task WITH (NOLOCK) 
                    INNER JOIN TaskDetail
                    ON TaskDetail.RecordId = Task.RecordId
                    WHERE TaskDetail.GroupId = tg.ID
                    AND Task.ParentJobId = @parentJobId
                    AND NOT (Task.State in (0x80, 0x100, 0x200)) 
                    AND ( Task.InstanceId = 0 or Task.InstanceId = -1)
             )       
    FROM TaskGroup tg
    WHERE tg.ID IN (SELECT ID FROM @taskgroupIds)
END
GO

/*
** UpdateTaskGroupsMaxMinNodes
**
** This procedure is used to update the MaxMinNodes
** for TaskGroups to a new value.
*/

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'SP_UpdateTaskGroupsMaxMinNodes'))
DROP PROCEDURE SP_UpdateTaskGroupsMaxMinNodes
GO

CREATE PROCEDURE SP_UpdateTaskGroupsMaxMinNodes(
    @parentJobId        int,
    @taskgroupIds       int_list readonly
    )
AS
BEGIN
    UPDATE tg SET 
        tg.MaxMinCores= 
            (              
                    SELECT COALESCE( MAX(TaskDetail.MinNodes),0)
                     -- We can put it here because this query is executed every 30 seconds
                     -- And if we miss the task state change once we can get it the next time anyway
                    FROM Task WITH (NOLOCK) 
                    INNER JOIN TaskDetail
                    ON TaskDetail.RecordId = Task.RecordId
                    WHERE TaskDetail.GroupId = tg.ID
                    AND Task.ParentJobId = @parentJobId
                    AND NOT (Task.State in (0x80, 0x100, 0x200)) 
                    AND ( Task.InstanceId = 0 or Task.InstanceId = -1)
             )       
    FROM TaskGroup tg
    WHERE tg.ID IN (SELECT ID FROM @taskgroupIds)
END
GO

/*
** UpdateTaskGroupsMaxMinSockets
**
** This procedure is used to update the MaxMinSockets
** for TaskGroups to a new value.
*/

IF EXISTS (SELECT id from sysobjects WHERE id=object_id(N'SP_UpdateTaskGroupsMaxMinSockets'))
DROP PROCEDURE SP_UpdateTaskGroupsMaxMinSockets
GO

CREATE PROCEDURE SP_UpdateTaskGroupsMaxMinSockets(
    @parentJobId        int,
    @taskgroupIds       int_list readonly
    )
AS
BEGIN
    UPDATE tg SET 
        tg.MaxMinCores= 
            (              
                    SELECT COALESCE( MAX(TaskDetail.MinSockets),0)
                     -- We can put it here because this query is executed every 30 seconds
                     -- And if we miss the task state change once we can get it the next time anyway
                    FROM Task WITH (NOLOCK) 
                    INNER JOIN TaskDetail
                    ON TaskDetail.RecordId = Task.RecordId
                    WHERE TaskDetail.GroupId = tg.ID
                    AND Task.ParentJobId = @parentJobId
                    AND NOT (Task.State in (0x80, 0x100, 0x200)) 
                    AND ( Task.InstanceId = 0 or Task.InstanceId = -1)
             )       
    FROM TaskGroup tg
    WHERE tg.ID IN (SELECT ID FROM @taskgroupIds)
END
GO

/*
** Create AddTaskGroupDependencyComplete SP
*/

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_AddTaskGroupDependencyComplete]')) 
DROP PROCEDURE dbo.SP_AddTaskGroupDependencyComplete
GO
CREATE PROCEDURE SP_AddTaskGroupDependencyComplete
    @JobId int,
    @ParentId int,
    @ChildId int
AS
BEGIN
    SET NOCOUNT ON;

	BEGIN TRAN

    IF NOT EXISTS (SELECT JobId FROM TaskGroupDependency WITH(UPDLOCK, HOLDLOCK) WHERE JobId = @JobId AND ParentId = @ParentId AND ChildId = @ChildId)

		INSERT INTO dbo.TaskGroupDependency 
			(JobId, ParentId, ChildId)
		VALUES
			(@JobId, @ParentId, @ChildId)

	IF NOT EXISTS (SELECT JobId FROM TaskGroupDependency WITH(UPDLOCK, HOLDLOCK) WHERE JobId = @JobId AND ParentId = @ChildId AND ChildId = 0)
	
		INSERT INTO dbo.TaskGroupDependency 
			(JobId, ParentId, ChildId)
		VALUES
			(@JobId, @ChildId, 0)

	COMMIT TRAN
END
GO


/*
** Modify JobHistory Table
*/

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[JobHistory]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[JobHistory]') AND name = N'Operator')
BEGIN
    ALTER TABLE JobHistory ADD Operator nvarchar(50) NULL
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[JobHistory]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[JobHistory]') AND name = N'PropChange')
BEGIN
    ALTER TABLE JobHistory ADD PropChange nvarchar(4000) NULL;
END
GO


/*
** Create the AddJobPropChangeRecord SP
*/

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_AddJobPropChangeRecord]')) 
DROP PROCEDURE dbo.SP_AddJobPropChangeRecord
GO

CREATE PROCEDURE SP_AddJobPropChangeRecord(
        @JobEvent           int,
        @EventTime          datetime,
        @JobId              int,
        @Operator           nvarchar(50),
        @PropChange	        nvarchar(4000)
        )
AS
BEGIN

    INSERT INTO JobHistory
        (Event, EventTime, JobId, Operator, PropChange)
    VALUES
        (@JobEvent, @EventTime, @JobId, @Operator, @PropChange)   

END
GO

/* Modify the SP_LoadCredentials SP */
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_LoadCredentials]')) 
DROP PROCEDURE dbo.SP_LoadCredentials
GO

CREATE PROCEDURE SP_LoadCredentials
AS
BEGIN
    SELECT Credential.Id, SID_User.Name, SID_Owner.Name, SID_User.SID, SID_Owner.SID, Credential.Reusable, Credential.Password, Credential.Certificate, SID_User.UserRole, Credential.ExtendedData
    FROM Credential
    LEFT OUTER JOIN SID AS SID_User ON Credential.RunAsId = SID_User.Id
    LEFT OUTER JOIN SID AS SID_Owner ON Credential.OwnerId = SID_Owner.Id
END
GO


/* Modify the SP_SetUserPassword SP */
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[SP_SetUserPassword]'))
DROP PROCEDURE dbo.SP_SetUserPassword
GO

CREATE PROCEDURE SP_SetUserPassword
    @runasName nvarchar(80),
    @runasSid nvarchar(80),
    @ownerName nvarchar(80),
    @ownerSid nvarchar(80),
    @password varbinary(800),
    @reusable int,
    @userRole int
AS
BEGIN

    SET NOCOUNT OFF;

    DECLARE @runasId as int
    DECLARE @prevRunAsName as nvarchar(80)
    DECLARE @prevUserRole as int

    SELECT @runasId = Id, @prevRunAsName = Name, @prevUserRole = UserRole
    FROM SID
    WHERE SID=@runasSid

    IF(@runasId is NULL)
    BEGIN
        INSERT INTO SID
            (SID, Name, UserRole)
        VALUES
            (@runasSid, @runasName, @userRole)

        SELECT @runasId = @@IDENTITY
    END
    ELSE
        IF(@prevRunAsName <> @runasName OR 
			(@prevUserRole IS NULL AND @userRole IS NOT NULL) OR
			(@prevUserRole IS NOT NULL AND @userRole IS NULL) OR
			@prevUserRole <> @userRole)
        BEGIN
            UPDATE SID 
            SET Name = @runasName, UserRole = @userRole
            WHERE Id = @runasId
        END
        

    DECLARE @ownerId as int
    DECLARE @prevOwnerName as nvarchar(80)

    SELECT @ownerId = Id, @prevOwnerName = Name
    FROM SID
    WHERE SID=@ownerSid

    IF(@ownerId is NULL)
    BEGIN
        INSERT INTO SID
            (SID, Name)
        VALUES
            (@ownerSid, @ownerName)

        SELECT @ownerId = @@IDENTITY
    END
    ELSE
        IF(@prevOwnerName <> @ownerName)
        BEGIN
            UPDATE SID
            SET Name = @ownerName
            WHERE Id = @ownerId
        END

    DECLARE @credentialId as int

    SELECT @credentialId = Id 
    FROM Credential
    WHERE RunAsId=@runasId AND OwnerId=@ownerId

    IF(@credentialId is NULL)
    BEGIN
       if(@reusable is null)
       BEGIN
        INSERT INTO dbo.Credential
            (RunAsId, OwnerId, Password, Reusable)
        VALUES
            (@runasId, @ownerId, @password, 0)
       END
       ELSE
       BEGIN
        INSERT INTO dbo.Credential
            (RunAsId, OwnerId, Password, Reusable)
        VALUES
            (@runasId, @ownerId, @password, @reusable)
       END
    END
    ELSE
    BEGIN
        if(@reusable is not null)
        BEGIN
            UPDATE dbo.Credential
            SET Password=@password, Reusable=@reusable, Certificate=NULL
            WHERE Id = @credentialId
        END
        ELSE
        BEGIN
            UPDATE dbo.Credential 
            SET Password=@password, Certificate=NULL
            WHERE Id = @credentialId
        END
    END
END
GO

/* Change AllocationHistory table to generate ID useing SEQUENCE
   Skip whole script if Sql Server version is less than 11 */
DECLARE @sqlServerVer int	
SELECT @sqlServerVer = PARSENAME(CONVERT(VARCHAR(32), CAST(SERVERPROPERTY('ProductVersion') AS NVARCHAR(128))), 4)
IF @sqlServerVer < 11
SET NOEXEC ON
GO

IF NOT EXISTS (SELECT id from dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[AllocationHistoryId]') AND type = 'SO')
BEGIN
	DECLARE @start int
	IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[AllocationHistory]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    BEGIN
        SELECT @start = max(id) from dbo.AllocationHistory
        IF @start is null
        SET @start = 0
    End
	ELSE
	SET @start = 0

	DECLARE @sql NVARCHAR(MAX)
	SET @sql = 
	'CREATE SEQUENCE [dbo].[AllocationHistoryId]
		AS int 
		START WITH ' + CONVERT(VARCHAR(10), @start + 1) +'
        INCREMENT BY 1
        CYCLE'    
	EXEC(@sql)
END
GO

/* Delete all constraints on dbo.AllocationHistory */
IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[AllocationHistory]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	BEGIN TRANSACTION
	DECLARE @SQL NVARCHAR(MAX) = N'';
	SELECT @SQL += N'
	ALTER TABLE ' + OBJECT_NAME(PARENT_OBJECT_ID) + ' DROP CONSTRAINT ' + OBJECT_NAME(OBJECT_ID) + ';' 
	FROM SYS.OBJECTS
	WHERE TYPE_DESC LIKE '%CONSTRAINT' AND OBJECT_NAME(PARENT_OBJECT_ID) = 'AllocationHistory';
	EXECUTE(@SQL)
	COMMIT
END
GO

BEGIN TRANSACTION
CREATE TABLE dbo.Tmp_AllocationHistory
	(
	Id int NOT NULL,
	ResourceId int NOT NULL,
	CoreId int NOT NULL,
	SocketId bigint NOT NULL,
	NodeId int NOT NULL,
	JobId int NOT NULL,
	TaskId int NOT NULL,
	JobRequeueCount int NOT NULL,
	TaskRequeueCount int NOT NULL,
	StartTime datetime NULL,
	EndTime datetime NULL,
	ModifyTime datetime NULL
	)
GO

ALTER TABLE dbo.Tmp_AllocationHistory SET (LOCK_ESCALATION = TABLE)
ALTER TABLE dbo.Tmp_AllocationHistory ADD CONSTRAINT DF_AllocationHistory_Id DEFAULT (NEXT VALUE FOR [dbo].[AllocationHistoryId]) FOR Id
ALTER TABLE dbo.Tmp_AllocationHistory ADD CONSTRAINT DF__Allocatio__TaskId DEFAULT ((0)) FOR TaskId
ALTER TABLE dbo.Tmp_AllocationHistory ADD CONSTRAINT DF__Allocatio__JobRequeueCount DEFAULT ((0)) FOR JobRequeueCount
ALTER TABLE dbo.Tmp_AllocationHistory ADD CONSTRAINT DF__Allocatio__TaskRequeueCount DEFAULT ((0)) FOR TaskRequeueCount
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[AllocationHistory]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
BEGIN
	IF EXISTS(SELECT TOP 1 id FROM dbo.AllocationHistory)
	EXEC('INSERT INTO dbo.Tmp_AllocationHistory (Id, ResourceId, CoreId, SocketId, NodeId, JobId, TaskId, JobRequeueCount, TaskRequeueCount, StartTime, EndTime, ModifyTime)
	SELECT Id, ResourceId, CoreId, SocketId, NodeId, JobId, TaskId, JobRequeueCount, TaskRequeueCount, StartTime, EndTime, ModifyTime FROM dbo.AllocationHistory WITH (HOLDLOCK TABLOCKX)')
END
GO

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[AllocationHistory]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP TABLE dbo.AllocationHistory
GO

EXECUTE sp_rename N'dbo.Tmp_AllocationHistory', N'AllocationHistory', 'OBJECT' 
GO

ALTER TABLE [dbo].[AllocationHistory] ADD PRIMARY KEY CLUSTERED ([Id])
CREATE NONCLUSTERED INDEX IDX_AllocationHistory_TaskIdTaskRequeueCount ON dbo.AllocationHistory (TaskId, TaskRequeueCount) 
CREATE NONCLUSTERED INDEX IDX_AllocationHistory_JobIdTaskIdJobRequeueCount ON dbo.AllocationHistory (JobId, TaskId, JobRequeueCount) INCLUDE (ResourceId) 
CREATE NONCLUSTERED INDEX IDX_AllocationHistory_Stage ON dbo.AllocationHistory (ModifyTime, TaskId) INCLUDE (Id)  
ALTER TABLE dbo.AllocationHistory ADD CONSTRAINT CK_AllocationHistory_FK_JobId FOREIGN KEY (JobId) REFERENCES dbo.Job (ID)

COMMIT
GO

SET NOEXEC OFF
GO

/*
** Modify TaskDetail Table
*/

IF EXISTS (SELECT id from dbo.sysobjects WHERE id = object_id(N'[dbo].[TaskDetail]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
    AND NOT EXISTS (SELECT id FROM [sys].[syscolumns] where id = object_id(N'[dbo].[TaskDetail]') AND name = N'RequestedNodeGroup')
BEGIN
    ALTER TABLE TaskDetail ADD RequestedNodeGroup nvarchar(64) NULL
END
GO

/*
** Index changes below
*/

/*
** Create IDX_TaskGroup_JobId_IsDefault
*/
IF EXISTS (SELECT * from sys.indexes WHERE name = N'IDX_TaskGroup_JobId_IsDefault')
DROP INDEX IDX_TaskGroup_JobId_IsDefault ON TaskGroup
GO

CREATE NONCLUSTERED INDEX IDX_TaskGroup_JobId_IsDefault ON TaskGroup(JobId, IsDefault) INCLUDE (ID)
go

/*
** Create IDX_TaskGroup_JobId_Name
*/
IF EXISTS (SELECT * from sys.indexes WHERE name = N'IDX_TaskGroup_JobId_Name')
DROP INDEX IDX_TaskGroup_JobId_Name ON TaskGroup
GO

CREATE NONCLUSTERED INDEX IDX_TaskGroup_JobId_Name ON TaskGroup(JobId, Name) INCLUDE (ID)
go

/*
** Create IDX_TaskGroup_Total
*/
IF EXISTS (SELECT * from sys.indexes WHERE name = N'IDX_TaskGroup_Total')
DROP INDEX IDX_TaskGroup_Total ON TaskGroup
GO

CREATE NONCLUSTERED INDEX IDX_TaskGroup_Total ON TaskGroup(Total) INCLUDE ([JobId], [Configuring], [Submitted], [Validating], [Queued], [Dispatching], [Running], [Finishing], [Finished], [Failed], [Canceled], [Canceling])
go

/*
** Create IDX_Resource_Reachable_JobId
*/
IF EXISTS (SELECT * from sys.indexes WHERE name = N'IDX_Resource_Reachable_JobId')
DROP INDEX IDX_Resource_Reachable_JobId ON Resource
GO

CREATE NONCLUSTERED INDEX IDX_Resource_Reachable_JobId ON Resource(Reachable, JobID) INCLUDE ([ID], [NodeId], [TaskID])	
go

/*
** Create IDX_Resource_MoveToOffline_Reachable
*/
IF EXISTS (SELECT * from sys.indexes WHERE name = N'IDX_Resource_MoveToOffline_Reachable')
DROP INDEX IDX_Resource_MoveToOffline_Reachable ON Resource
GO

CREATE NONCLUSTERED INDEX IDX_Resource_MoveToOffline_Reachable ON Resource([MoveToOffline], [Reachable], [IsAvailable],[State]) INCLUDE ([ID], [NodeId], [CoreID], [SocketID], [JobID], [TaskID], [JobPhase], [LimitTime], [RuntimeSeconds], [FirstTaskAllocation], [JobType], [Phantom], [MemoryUsed])	
go

/*
** Create IDX_NodeAllocationHistory_JobId_JobRequeueCount 
*/
IF EXISTS (SELECT * from sys.indexes WHERE name = N'IDX_NodeAllocationHistory_JobId')
DROP INDEX IDX_NodeAllocationHistory_JobId ON NodeAllocationHistory
GO

IF EXISTS (SELECT * from sys.indexes WHERE name = N'IDX_NodeAllocationHistory_JobId_JobRequeueCount')
DROP INDEX IDX_NodeAllocationHistory_JobId_JobRequeueCount ON NodeAllocationHistory
GO

CREATE INDEX IDX_NodeAllocationHistory_JobId_JobRequeueCount ON [dbo].[NodeAllocationHistory] ([JobId], [JobRequeueCount]) INCLUDE ([UserTime], MemoryUsed, KernelTime)	
go


/*
** Create IDX_JobHistory_EventTime
*/
IF EXISTS (SELECT * from sys.indexes WHERE name = N'IDX_JobHistory_EventTime')
DROP INDEX IDX_JobHistory_EventTime ON JobHistory
GO

CREATE NONCLUSTERED INDEX IDX_JobHistory_EventTime ON JobHistory ([EventTime]) INCLUDE ([Id])
go

/*
** Create [IDX_AllocationHistory_JobIdTaskIdJobRequeueCount]
*/
IF EXISTS (SELECT * from sys.indexes WHERE name = N'IDX_AllocationHistory_JobIdTaskIdJobRequeueCount')
DROP INDEX [IDX_AllocationHistory_JobIdTaskIdJobRequeueCount] ON [AllocationHistory]
GO

CREATE INDEX [IDX_AllocationHistory_JobIdTaskIdJobRequeueCount] ON [dbo].[AllocationHistory] ([JobId] ASC,[TaskId] ASC,[JobRequeueCount] ASC,[EndTime] ASC) INCLUDE ( [ResourceId],[NodeId])
go

/*
** Create [IDX_Task_ParentJobIdInstanceId]
*/
IF EXISTS (SELECT * from sys.indexes WHERE name = N'IDX_Task_ParentJobIdInstanceId')
DROP INDEX [IDX_Task_ParentJobIdInstanceId] ON Task
GO

CREATE INDEX [IDX_Task_ParentJobIdInstanceId] ON [dbo].[Task] ([ParentJobId],	[InstanceId] ) INCLUDE ( [RecordID], [State]) 
go

/*
** Create [IDX_Task_RecordIdInstanceId]
*/
IF EXISTS (SELECT * from sys.indexes WHERE name = N'IDX_Task_RecordIdInstanceId')
DROP INDEX [IDX_Task_RecordIdInstanceId] ON Task
GO

CREATE NONCLUSTERED INDEX IDX_Task_RecordIdInstanceId ON Task(RecordId, InstanceId) INCLUDE (ParentJobId, State)
go

/*
** Create [IDX_TaskDetail_GroupIdRecordId]
*/
IF EXISTS (SELECT * from sys.indexes WHERE name = N'IDX_TaskDetail_GroupIdRecordId')
DROP INDEX [IDX_TaskDetail_GroupIdRecordId] ON TaskDetail
GO

CREATE NONCLUSTERED INDEX IDX_TaskDetail_GroupIdRecordId ON TaskDetail(GroupId, recordId) include (currentInstanceId, MinCores, MinSockets, MinNodes)
go

/*
** Create IDX_Job_State_HasUnfinishedParentJobs
*/
IF EXISTS (SELECT * from sys.indexes WHERE name = N'IDX_Job_State_HasUnfinishedParentJobs')
DROP INDEX IDX_Job_State_HasUnfinishedParentJobs ON Job
GO

CREATE NONCLUSTERED INDEX IDX_Job_State_HasUnfinishedParentJobs ON job (State, HasUnfinishedParentJobs) INCLUDE (ID, PendingReason, HasParentJobs)
go

/*
** functions
*/
if object_id('fn_GetTaskState') is not null
drop function fn_GetTaskState
go
create function fn_GetTaskState(@name as nvarchar(max))
returns int
begin
    return case
        when @name = 'Configuring' then 0x1
        when @name = 'Submitted' then 0x2
        when @name = 'Validating' then 0x4
        when @name = 'Queued' then 0x8
        when @name = 'Dispatching' then 0x10
        when @name = 'Running' then 0x20
        when @name = 'Finishing' then 0x40
        when @name = 'Finished' then 0x80
        when @name = 'Failed' then 0x100
        when @name = 'Canceled' then 0x200
        when @name = 'Canceling' then 0x400
        when @name = 'All' then 0x7FF
        else 0
    end
end
go

/*
** views
*/
if object_id('v_TaskGroup') is not null
    drop view v_TaskGroup
go
create view v_TaskGroup as
select tg.JobID, tg.ID
, COUNT(case when (t.State & dbo.fn_GetTaskState(N'Configuring') > 0) then 1 else null end) as Configuring
, COUNT(case when (t.State & dbo.fn_GetTaskState(N'Submitted') > 0) then 1 else null end) as Submitted
, COUNT(case when (t.State & dbo.fn_GetTaskState(N'Validating') > 0) then 1 else null end) as Validating
, COUNT(case when (t.State & dbo.fn_GetTaskState(N'Queued') > 0) then 1 else null end) as Queued
, COUNT(case when (t.State & dbo.fn_GetTaskState(N'Dispatching') > 0) then 1 else null end) as Dispatching
, COUNT(case when (t.State & dbo.fn_GetTaskState(N'Running') > 0) then 1 else null end) as Running
, COUNT(case when (t.State & dbo.fn_GetTaskState(N'Finishing') > 0) then 1 else null end) as Finishing
, COUNT(case when (t.State & dbo.fn_GetTaskState(N'Finished') > 0) then 1 else null end) as Finished
, COUNT(case when (t.State & dbo.fn_GetTaskState(N'Failed') > 0) then 1 else null end) as Failed
, COUNT(case when (t.State & dbo.fn_GetTaskState(N'Canceled') > 0) then 1 else null end) as Canceled
, COUNT(case when (t.State & dbo.fn_GetTaskState(N'Canceling') > 0) then 1 else null end) as Canceling
from TaskGroup tg
left join TaskDetail td on td.GroupId = tg.id
left join Task t on td.RecordID = t.RecordID
group by tg.JobId, tg.ID

go

if object_id('SP_FetchTaskGroupStatistics') is not null
drop PROCEDURE SP_FetchTaskGroupStatistics
go
CREATE PROCEDURE SP_FetchTaskGroupStatistics
    @jobId int
AS
BEGIN
    -- SET NOCOUNT ON added to prevent extra result sets from
    -- interfering with SELECT statements.
    SET NOCOUNT ON;

    select ID
        , Submitted + Validating + Queued + Dispatching + Running + Finishing + Canceling as Active
        , Failed + Canceled as BadState
    from dbo.v_TaskGroup tg
    where tg.JobId = @jobId
end
GO

EXEC sp_updateextendedproperty @name = N'dbVersion', @value = '5.1'
GO